/* Egoboo - menu.c
 * Implements the main menu tree, using the code in Ui.*
 */

/*
   This file is part of Egoboo.

   Egoboo is free software: you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Egoboo is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Egoboo.  If not, see <http://www.gnu.org/licenses/>.
*/

//********************************************************************************************
// New menu code
//********************************************************************************************


#include "Ui.h"
#include "Menu.h"
#include "Log.h"
#include "egoboo.h"
#include <assert.h>

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
typedef enum mnu_states_e
{
  MM_Begin,
  MM_Entering,
  MM_Running,
  MM_Leaving,
  MM_Finish,
} MenuStates;

// All the different menus.  yay!
enum mnu_e
{
  mnu_NotImplemented,
  mnu_Main,
  mnu_SinglePlayer,
  mnu_MultiPlayer,
  mnu_ChooseModule,
  mnu_ChoosePlayer,
  mnu_TestResults,
  mnu_Options,
  mnu_VideoOptions,
  mnu_AudioOptions,
  mnu_InputOptions,
  mnu_NewPlayer,
  mnu_LoadPlayer,
  mnu_Network,
  mnu_HostGame,
  mnu_JoinGame,
  mnu_Inventory
};

//--------------------------------------------------------------------------------------------

// TEMPORARY!
#define NET_DONE_SENDING_FILES 10009
#define NET_NUM_FILES_TO_SEND  10010
static STRING mnu_filternamebuffer    = {0};
static STRING mnu_display_mode_buffer = {0};
static int    mnu_display_mode_index = 0;

#define MAXWIDGET 100
static int       mnu_widgetCount;
static ui_Widget mnu_widgetList[MAXWIDGET];

static int mnu_selectedPlayerCount = 0;
static int mnu_selectedInput[MAXPLAYER] = {0};
static int mnu_selectedPlayer[MAXPLAYER] = {0};
static int mnu_selectedModule = 0;

/* Variables for the model viewer in mnu_doChoosePlayer */
static float  mnu_modelAngle = 0;
static Uint32 mnu_modelIndex = 0;

/* Copyright text variables.  Change these to change how the copyright text appears */
const char mnu_copyrightText[] = "Welcome to Egoboo!\nhttp://home.no.net/egoboo\nVersion 2.7.5";
static int mnu_copyrightLeft = 0;
static int mnu_copyrightTop  = 0;

/* Options info text variables.  Change these to change how the options text appears */
const char mnu_optionsText[] = "Change your audio, input and video\nsettings here.";
static int mnu_optionsTextLeft = 0;
static int mnu_optionsTextTop  = 0;

/* Button labels.  Defined here for consistency's sake, rather than leaving them as constants */
static const char *mnu_mainMenuButtons[] =
{
  "New Game",
  "Load Game",
  "Network",
  "Options",
  "Quit",
  ""
};

static const char *mnu_singlePlayerButtons[] =
{
  "New Player",
  "Load Saved Player",
  "Back",
  ""
};

static const char *mnu_multiPlayerButtons[] =
{
  "Start Game",
  "Back",
  ""
};

static const char *mnu_optionsButtons[] =
{
  "Audio Options",
  "Input Controls",
  "Video Settings",
  "Back",
  ""
};

static const char *mnu_audioOptionsText[] =
{
  "N/A",    //Enable sound
  "N/A",    //Sound volume
  "N/A",    //Enable music
  "N/A",    //Music volume
  "N/A",    //Sound channels
  "N/A",    //Sound buffer
  "Save Settings",
  ""
};

static const char *mnu_videoOptionsText[] =
{
  "N/A",  //Antialaising
  "N/A",  //Particle Effects
  "N/A",  //Fast & ugly
  "N/A",  //Fullscreen
  "N/A",  //Reflections
  "N/A",  //Texture filtering
  "N/A",  //Shadows
  "N/A",  //Z bit
  "N/A",  //Fog
  "N/A",  //3D effects
  "N/A",  //Multi water layer
  "N/A",  //Max messages
  "N/A",  //Screen resolution
  "Save Settings",
  ""
};


/* Button position for the "easy" menus, like the main one */
static int mnu_buttonLeft = 0;
static int mnu_buttonTop = 0;

static bool_t mnu_startNewPlayer = bfalse;

/* The font used for drawing text.  It's smaller than the button font */
static Font *mnu_Font = NULL;

//--------------------------------------------------------------------------------------------
bool_t mnu_slideButton(ui_Widget * pw2, ui_Widget * pw1, float dx, float dy)
{
  if(NULL==pw2 || NULL==pw1) return bfalse;
  if(!ui_copyWidget(pw2,pw1)) return bfalse;

  pw2->x += dx;
  pw2->y += dy;

  return btrue;
}

//--------------------------------------------------------------------------------------------
// "Slidy" buttons used in some of the menus.  They're shiny.
struct slidy_button_state_t
{
  float lerp;
  int top;
  int left;
  ui_Widget * button_list;
  int        button_count;
} SlidyButtonState;

//--------------------------------------------------------------------------------------------
int mnu_initWidgetsList(ui_Widget wlist[], int wmax, const char * text[])
{
  int i, cnt;
  cnt = 0;
  for(i=0; i<wmax && 0x00 != text[i][0]; i++, cnt++)
  {
    wlist[i].id = i;
    wlist[i].width  = 200;
    wlist[i].height = 30;
    wlist[i].x  = mnu_buttonLeft;
    wlist[i].y  = mnu_buttonTop + (i * 35);
    wlist[i].text = text[i];
    wlist[i].img = NULL;
  };

  return cnt;
};

//--------------------------------------------------------------------------------------------
static void initSlidyButtons(float lerp, ui_Widget * buttons, int count)
{
  SlidyButtonState.lerp         = lerp;
  SlidyButtonState.button_list  = buttons;
  SlidyButtonState.button_count = count;
}

//--------------------------------------------------------------------------------------------
static void mnu_updateSlidyButtons(float deltaTime)
{
  SlidyButtonState.lerp += (deltaTime * 1.5f);
}

//--------------------------------------------------------------------------------------------
static void mnu_drawSlidyButtons()
{
  int i;
  ui_Widget wtmp;

  for (i = 0; i<SlidyButtonState.button_count;  i++)
  {
    float dx;
    
    dx = - (360 - i * 35)  * SlidyButtonState.lerp;

    mnu_slideButton(&wtmp, &SlidyButtonState.button_list[i], dx, 0); 
    ui_doButton(&wtmp);
  }
}

//--------------------------------------------------------------------------------------------
// initMenus()
// Loads resources for the menus, and figures out where things should
// be positioned.  If we ever allow changing resolution on the fly, this
// function will have to be updated/called more than once.
//

int initMenus()
{
  int i;

  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.uifont_ttf);
  mnu_Font = fnt_loadFont(CStringTmp1, CData.uifont_points2);
  if (NULL==mnu_Font)
  {
    log_error("Could not load the menu font!\n");
    return 0;
  }

  // Figure out where to draw the buttons
  mnu_buttonLeft = 40;
  mnu_buttonTop = displaySurface->h - 20;
  for (i = 0; mnu_mainMenuButtons[i][0] != 0; i++)
  {
    mnu_buttonTop -= 35;
  }

  // Figure out where to draw the copyright text
  mnu_copyrightLeft = 0;
  mnu_copyrightLeft = 0;
  fnt_getTextBoxSize(mnu_Font, mnu_copyrightText, 20, &mnu_copyrightLeft, &mnu_copyrightTop);
  // Draw the copyright text to the right of the buttons
  mnu_copyrightLeft = 280;
  // And relative to the bottom of the screen
  mnu_copyrightTop = displaySurface->h - mnu_copyrightTop - 20;

  // Figure out where to draw the options text
  mnu_optionsTextLeft = 0;
  mnu_optionsTextLeft = 0;
  fnt_getTextBoxSize(mnu_Font, mnu_optionsText, 20, &mnu_optionsTextLeft, &mnu_optionsTextTop);
  // Draw the copyright text to the right of the buttons
  mnu_optionsTextLeft = 280;
  // And relative to the bottom of the screen
  mnu_optionsTextTop = displaySurface->h - mnu_optionsTextTop - 20;

  return 1;
}

//--------------------------------------------------------------------------------------------
int mnu_doMain(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright;
  static float lerp;
  static int menuChoice = 0;

  int result = 0;

  switch (menuState)
  {
  case MM_Begin:
    // set up menu variables
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_main_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_TEXTURE);
    menuChoice = 0;
    menuState = MM_Entering;


    mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_mainMenuButtons);
    initSlidyButtons(1.0f, mnu_widgetList, mnu_widgetCount);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    // let this fall through into MM_Entering

  case MM_Entering:
    // do buttons sliding in animation, and background fading in
    // background

    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Copyright" text
    ui_drawTextBox(&wCopyright, 20);

    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(-deltaTime);

    // Let lerp wind down relative to the time elapsed
    if (SlidyButtonState.lerp <= 0.0f)
    {
      menuState = MM_Running;
    }

    break;

  case MM_Running:
    // Do normal run
    // Background
    glColor4f(1, 1, 1, 1);
    ui_drawImage(&wBackground);

    // "Copyright" text
    ui_drawTextBox(&wCopyright, 20);

    // Buttons
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      // begin single player stuff
      menuChoice = 1;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      // begin multi player stuff
      menuChoice = 2;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
      // go to options menu
      menuChoice = 3;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[3]))
    {
      // quit game
      menuChoice = 4;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[4]))
    {
      // quit game
      menuChoice = 5;
    }

    if (menuChoice != 0)
    {
      menuState = MM_Leaving;
      mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_mainMenuButtons);
      initSlidyButtons(0.0f, mnu_widgetList, mnu_widgetCount);
    }
    break;

  case MM_Leaving:
    // Do buttons sliding out and background fading
    // Do the same stuff as in MM_Entering, but backwards
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Copyright" text
    ui_drawTextBox(&wCopyright, 20);

    // Buttons
    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(deltaTime);
    if (SlidyButtonState.lerp >= 1.0f)
    {
      menuState = MM_Finish;
    }
    break;

  case MM_Finish:
    // Free the background texture; don't need to hold onto it
    GLTexture_Release(&background);
    menuState = MM_Begin; // Make sure this all resets next time mnu_doMain is called

    // Set the next menu to load
    result = menuChoice;
    break;

  };

  return result;
}

//--------------------------------------------------------------------------------------------
int mnu_doSinglePlayer(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright;
  static int menuChoice;
  int result = 0;

  switch (menuState)
  {
  case MM_Begin:
    // Load resources for this menu
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_advent_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);
    menuChoice = 0;

    menuState = MM_Entering;

    mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_singlePlayerButtons);
    initSlidyButtons(1.0f, mnu_widgetList, mnu_widgetCount);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    // Let this fall through

  case MM_Entering:
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);

    // Draw the background image
    ui_drawImage(&wBackground);

    // "Copyright" text
    fnt_drawTextBox(mnu_Font, mnu_copyrightText, mnu_copyrightLeft, mnu_copyrightTop, 0, 0, 20);

    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(-deltaTime);

    if (SlidyButtonState.lerp <= 0.0f)
      menuState = MM_Running;

    break;

  case MM_Running:

    // Draw the background image
    ui_drawImage(&wBackground);

    // "Copyright" text
    fnt_drawTextBox(mnu_Font, mnu_copyrightText, mnu_copyrightLeft, mnu_copyrightTop, 0, 0, 20);

    // Buttons
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      menuChoice = 1;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      menuChoice = 2;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
      menuChoice = 3;
    }

    if (menuChoice != 0)
    {
      menuState = MM_Leaving;
      mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_singlePlayerButtons);
      initSlidyButtons(0.0f, mnu_widgetList, mnu_widgetCount);
    }

    break;

  case MM_Leaving:
    // Do buttons sliding out and background fading
    // Do the same stuff as in MM_Entering, but backwards
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Copyright" text
    ui_drawTextBox(&wCopyright, 20);

    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(deltaTime);

    if (SlidyButtonState.lerp >= 1.0f)
    {
      menuState = MM_Finish;
    }
    break;

  case MM_Finish:
    // Release the background texture
    GLTexture_Release(&background);

    // Set the next menu to load
    result = menuChoice;

    // And make sure that if we come back to this menu, it resets
    // properly
    menuState = MM_Begin;
  }

  return result;
}

//--------------------------------------------------------------------------------------------
// TODO: I totally fudged the layout of this menu by adding an offset for when
// the game isn't in 640x480.  Needs to be fixed.
int mnu_doChooseModule(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static int startIndex;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright, wtmp;
  static int validModules[MAXMODULE];
  static int numValidModules;

  static int moduleMenuOffsetX;
  static int moduleMenuOffsetY;

  int result = 0;
  int i, j, x, y;
  char txtBuffer[128];

  switch (menuState)
  {
  case MM_Begin:
    // Load font & background
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_sleepy_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    startIndex = 0;
    mnu_selectedModule = -1;
    modsummaryval      = -2;

    // Find the module's that we want to allow loading for.  If mnu_startNewPlayer
    // is true, we want ones that don't allow imports (e.g. starter modules).
    // Otherwise, we want modules that allow imports
    memset(validModules, 0, sizeof(int) * MAXMODULE);
    numValidModules = 0;
    for (i = 0;i < globalnummodule; i++)
    {
      if(!mnu_startNewPlayer && modimportamount[i]>0)
      {
        validModules[numValidModules] = i;
        numValidModules++;
      }
      else if (modimportamount[i] == 0 && modmaxplayers[i] == 1)
      {
        validModules[numValidModules] = i;
        numValidModules++;
      };
    }

    // Figure out at what offset we want to draw the module menu.
    moduleMenuOffsetX = (displaySurface->w - 640) / 2;
    moduleMenuOffsetY = (displaySurface->h - 480) / 2;

    // navigation buttons
    ui_initWidget(&mnu_widgetList[0], 0, mnu_Font, "<-", NULL, moduleMenuOffsetX + 20, moduleMenuOffsetY + 74, 30, 30);
    ui_initWidget(&mnu_widgetList[1], 1, mnu_Font, "->", NULL, moduleMenuOffsetX + 590, moduleMenuOffsetY + 74, 30, 30);
    ui_initWidget(&mnu_widgetList[2], 2, mnu_Font, "Select Module", NULL, moduleMenuOffsetX + 327, moduleMenuOffsetY + 173, 200, 30);
    ui_initWidget(&mnu_widgetList[3], 3, mnu_Font, "Back", NULL, moduleMenuOffsetX + 327, moduleMenuOffsetY + 208, 200, 30);

    // Module "windows"
    ui_initWidget(&mnu_widgetList[4], 4, mnu_Font, NULL, NULL, moduleMenuOffsetX, moduleMenuOffsetY, 138, 138);
    ui_initWidget(&mnu_widgetList[5], 5, mnu_Font, NULL, NULL, moduleMenuOffsetX, moduleMenuOffsetY, 138, 138);
    ui_initWidget(&mnu_widgetList[6], 6, mnu_Font, NULL, NULL, moduleMenuOffsetX, moduleMenuOffsetY, 138, 138);

    // Module description
    ui_initWidget(&mnu_widgetList[7], UI_Nothing, mnu_Font, NULL, NULL, moduleMenuOffsetX + 21, moduleMenuOffsetY + 173, 291, 230);


    menuState = MM_Entering;

    x = (displaySurface->w / 2) - (background.imgW / 2);
    y = displaySurface->h - background.imgH;

    // fall through...

  case MM_Entering:
    menuState = MM_Running;

    // fall through for now...

  case MM_Running:
    // Draw the background
    glColor4f(1, 1, 1, 1);
    ui_drawImage(&wBackground);

    // Fudged offset here.. DAMN!  Doesn't work, as the mouse tracking gets skewed
    // I guess I'll do it the uglier way
    //glTranslatef(moduleMenuOffsetX, moduleMenuOffsetY, 0);


    // Draw the arrows to pick modules
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      startIndex--;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      startIndex++;

      if (startIndex + 3 >= numValidModules)
      {
        startIndex = numValidModules - 3;
      }
    }

    // Clamp startIndex to 0
    startIndex = MAX(0, startIndex);

    // Draw buttons for the modules that can be selected
    x = 93;
    y = 20;
    for (i = startIndex,j=0; i < (startIndex + 3) && i < numValidModules; i++,j++)
    {
      mnu_widgetList[4+j].img = &TxTitleImage[validModules[i]];
      mnu_slideButton(&wtmp, &mnu_widgetList[4+j], x, y);
      if (BUTTON_UP == ui_doImageButton(&wtmp))
      {
        mnu_selectedModule = i;
      }

      x += 138 + 20; // Width of the button, and the spacing between buttons
    }

    // Draw an unused button as the backdrop for the text for now
    ui_drawButton(&mnu_widgetList[7]);

    // And draw the next & back buttons
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
      // go to the next menu with this module selected
      mnu_selectedModule = validModules[mnu_selectedModule];
      menuState = MM_Leaving;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[3]))
    {
      // Signal mnu_Run to go back to the previous menu
      mnu_selectedModule = -1;
      menuState = MM_Leaving;
    }

    // Draw the text description of the selected module
    if (mnu_selectedModule > -1)
    {
      y = 173 + 5;
      x = 21 + 5;
      glColor4f(1, 1, 1, 1);
      fnt_drawText(mnu_Font, moduleMenuOffsetX + x, moduleMenuOffsetY + y,
                   modlongname[validModules[mnu_selectedModule]]);
      y += 20;

      snprintf(txtBuffer, sizeof(txtBuffer), "Difficulty: %s", modrank[validModules[mnu_selectedModule]]);
      fnt_drawText(mnu_Font, moduleMenuOffsetX + x, moduleMenuOffsetY + y, txtBuffer);
      y += 20;

      if (modmaxplayers[validModules[mnu_selectedModule]] > 1)
      {
        if (modminplayers[validModules[mnu_selectedModule]] == modmaxplayers[validModules[mnu_selectedModule]])
        {
          snprintf(txtBuffer, sizeof(txtBuffer), "%d Players", modminplayers[validModules[mnu_selectedModule]]);
        }
        else
        {
          snprintf(txtBuffer, sizeof(txtBuffer), "%d - %d Players", modminplayers[validModules[mnu_selectedModule]], modmaxplayers[validModules[mnu_selectedModule]]);
        }
      }
      else
      {
	    if(modimportamount[validModules[mnu_selectedModule]] == 0) snprintf(txtBuffer, sizeof(txtBuffer), "Starter Module");
		else snprintf(txtBuffer, sizeof(txtBuffer), "Special Module");
      }
      fnt_drawText(mnu_Font, moduleMenuOffsetX + x, moduleMenuOffsetY + y, txtBuffer);
      y += 20;

      // And finally, the summary
      snprintf(txtBuffer, sizeof(txtBuffer), "%s/%s/%s/%s", CData.modules_dir, modloadname[validModules[mnu_selectedModule]], CData.gamedat_dir, CData.mnu_file);
      if(validModules[mnu_selectedModule] != modsummaryval)
      {
        if(get_module_summary(txtBuffer)) modsummaryval = validModules[mnu_selectedModule];
      };

      for (i = 0;i < SUMMARYLINES;i++)
      {
        fnt_drawText(mnu_Font, moduleMenuOffsetX + x, moduleMenuOffsetY + y, modsummary[i]);
        y += 20;
      }
    }

    break;

  case MM_Leaving:
    menuState = MM_Finish;
    // fall through for now

  case MM_Finish:
    GLTexture_Release(&background);

    menuState = MM_Begin;

    if (mnu_selectedModule == -1)
    {
      result = -1;
    }
    else
    {
      // Save the name of the module that we've picked
      strncpy(pickedmodule, modloadname[mnu_selectedModule], 64);

      // If the module allows imports, return 1.  Else, return 2
      if (modimportamount[mnu_selectedModule] > 0)
      {
        importvalid = btrue;
        importamount = modimportamount[mnu_selectedModule];
        result = 1;
      }
      else
      {
        importvalid = bfalse;
        result = 2;
      }

      exportvalid = modallowexport[mnu_selectedModule];
      playeramount = modmaxplayers[mnu_selectedModule];

      respawnvalid = bfalse;
      respawnanytime = bfalse;
      if (modrespawnvalid[mnu_selectedModule]) respawnvalid = btrue;
      if (modrespawnvalid[mnu_selectedModule] == ANYTIME) respawnanytime = btrue;

    }
    break;

  }

  return result;
}

//--------------------------------------------------------------------------------------------
static bool_t mnu_checkSelectedPlayer(int player)
{
  int i;
  if(player<0 || player > numloadplayer) return bfalse;

  for(i=0; i<MAXPLAYER && i<mnu_selectedPlayerCount; i++)
  {
    if(mnu_selectedPlayer[i] == player) return btrue;
  }

  return bfalse;
};

//--------------------------------------------------------------------------------------------
static bool_t addSelectedPlayer(int player)
{
  if(player<0 || player > numloadplayer || mnu_selectedPlayerCount>=MAXPLAYER) return bfalse;
  if( mnu_checkSelectedPlayer(player) ) return bfalse;

  mnu_selectedPlayer[mnu_selectedPlayerCount] = player;
  mnu_selectedInput[mnu_selectedPlayerCount]  = INPUTNONE;
  mnu_selectedPlayerCount++;

  return btrue;
};

//--------------------------------------------------------------------------------------------
static bool_t mnu_removeSelectedPlayer(int player)
{
  int i;
  bool_t found = bfalse;

  if(player<0 || player > numloadplayer || mnu_selectedPlayerCount<=0) return bfalse;


  for(i=0; i<MAXPLAYER && i<mnu_selectedPlayerCount; i++)
  {
    if(mnu_selectedPlayer[i] == player)
    {
      found = btrue;
      break;
    }
  }

  if(found)
  {
    i++;
    for( /* nothing */; i<MAXPLAYER && i<mnu_selectedPlayerCount; i++)
    {
      mnu_selectedPlayer[i-1] = mnu_selectedPlayer[i];
    }

    mnu_selectedPlayerCount--;
  };

  return found;
};

//--------------------------------------------------------------------------------------------
static bool_t mnu_addSelectedPlayerInput(int player, Uint32 input)
{
  int i;

  for(i=0; i<MAXPLAYER && i<mnu_selectedPlayerCount; i++)
  {
    if(mnu_selectedPlayer[i] == player)
    {
      mnu_selectedInput[i] |= input;
      return btrue;
    }
  }

  return bfalse;
};

//--------------------------------------------------------------------------------------------
static bool_t mnu_removeSelectedPlayerInput(int player, Uint32 input)
{
  int i;

  for(i=0; i<MAXPLAYER && i<mnu_selectedPlayerCount; i++)
  {
    if(mnu_selectedPlayer[i] == player)
    {
      mnu_selectedInput[i] &= ~input;
      return btrue;
    }
  }

  return bfalse;
};

//--------------------------------------------------------------------------------------------
int mnu_doChoosePlayer(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright, wtmp;
  int result = 0;
  static int numVertical, numHorizontal;
  int i, j, k, m, loadplayer, x, y;
  int player;
  char srcDir[64], destDir[64];
  static GLTexture TxInput[4];
  static Uint32 BitsInput[4];

  switch (menuState)
  {
  case MM_Begin:
    mnu_selectedPlayerCount = 0;
    mnu_selectedPlayer[0] = 0;

    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_sleepy_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);

    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.keybicon_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &TxInput[0], CStringTmp1, INVALID_KEY);
    BitsInput[0] = INPUTKEY;

    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.mousicon_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &TxInput[1], CStringTmp1, INVALID_KEY);
    BitsInput[1] = INPUTMOUSE;

    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.joyaicon_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &TxInput[2], CStringTmp1, INVALID_KEY);
    BitsInput[2] = INPUTJOYA;

    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.joybicon_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &TxInput[3], CStringTmp1, INVALID_KEY);
    BitsInput[3] = INPUTJOYB;

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    // load information for all the players that could be imported
    check_player_import("players");

    // set the configuration
    numVertical = 6;
    numHorizontal = 2;

    ui_initWidget(&mnu_widgetList[0], 0, mnu_Font, "Select Module", NULL, 40, 350, 200, 30);
    ui_initWidget(&mnu_widgetList[1], 1, mnu_Font, "Back", NULL, 40, 385, 200, 30);

    // initialize the selction buttons
    x = 20;
    m = 2;
    for (i = 0; i < numHorizontal; i++)
    {
      y = 20;
      for (j = 0; j < numVertical; j++)
      {
        ui_initWidget(&mnu_widgetList[m], m, mnu_Font, NULL, NULL, x, y, 175, 42);
        ui_widgetAddMask(&mnu_widgetList[m], UI_BITS_CLICKED);
        m++;

        for(k=0; k<4; k++, m++)
        {
          ui_initWidget(&mnu_widgetList[m], m, mnu_Font, NULL, &TxInput[k], x + 175 + k*42, y, 42, 42);
          ui_widgetAddMask(&mnu_widgetList[m], UI_BITS_CLICKED);
        };

        y += 47;
      }
      x += 180;
    };

    menuState = MM_Entering;
    // fall through

  case MM_Entering:
    menuState = MM_Running;
    // fall through

  case MM_Running:
    // Figure out how many players we can show without scrolling

    // Draw the background
    ui_drawImage(&wBackground);

    // Draw the player selection buttons
    // I'm being tricky, and drawing two buttons right next to each other
    // for each player: one with the icon, and another with the name.  I'm
    // given those buttons the same ID, so they'll act like the same button.

    player = 0;
    m = 2;
    for (i = 0; i < numHorizontal && player < numloadplayer; i++)
    {
      for (j = 0; j < numVertical && player < numloadplayer; j++, player++)
      {
        mnu_widgetList[m].img  = &TxIcon[player];
        mnu_widgetList[m].text = loadplayername[player];

        if (BUTTON_NOCHANGE != ui_doImageButtonWithText(&mnu_widgetList[m]))
        {
          if(0 != (mnu_widgetList[m].state & UI_BITS_CLICKED) && !mnu_checkSelectedPlayer(player))
          {
            // button has become clicked
            addSelectedPlayer(player);
          }
          else if(0 == (mnu_widgetList[m].state & UI_BITS_CLICKED) && mnu_checkSelectedPlayer(player))
          {
            // button has become unclicked
            mnu_removeSelectedPlayer(player);
          };
        };
        m++;

        for(k=0; k<4; k++, m++)
        {
          if (BUTTON_NOCHANGE != ui_doImageButton(&mnu_widgetList[m]))
          {
            if(0 != (mnu_widgetList[m].state & UI_BITS_CLICKED) )
            {
              // button has become clicked
              mnu_addSelectedPlayerInput(player, BitsInput[k]);
            }
            else if(0 == (mnu_widgetList[m].state & UI_BITS_CLICKED) && mnu_checkSelectedPlayer(player))
            {
              // button has become unclicked
              mnu_removeSelectedPlayerInput(player, BitsInput[k]);
            };
          };
        };
      }
    };



    // Buttons for going ahead
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      menuState = MM_Leaving;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      mnu_selectedPlayerCount = 0;
      menuState = MM_Leaving;
    }

    break;

  case MM_Leaving:
    menuState = MM_Finish;
    // fall through

  case MM_Finish:
    GLTexture_Release(&background);
    menuState = MM_Begin;

    if (0 == mnu_selectedPlayerCount)
    {
      result = -1;
    }
    else
    {
      empty_import_directory();
      fs_createDirectory("import");

      numimport = 0;
      for(numimport=0; numimport<mnu_selectedPlayerCount; numimport++)
      {
        // Build the import directory
        // I'm just allowing 1 player for now...

        loadplayer = mnu_selectedPlayer[numimport];
        localcontrol[numimport] = mnu_selectedInput[numimport];
        localslot[numimport]    = 9 * numimport;

        // Copy the character to the import directory
        snprintf(srcDir,  sizeof(srcDir),  "players/%s", loadplayerdir[loadplayer]);
        snprintf(destDir, sizeof(destDir), "import/temp%04d.obj", localslot[numimport]);
        fs_copyDirectory(srcDir, destDir);

        // Copy all of the character's items to the import directory
        for (i=0; i<8; i++)
        {
          snprintf(srcDir, sizeof(srcDir), "players/%s/%d.obj", loadplayerdir[loadplayer], i);
          snprintf(destDir, sizeof(destDir), "import/temp%04d.obj", localslot[numimport] + i + 1);

          fs_copyDirectory(srcDir, destDir);
        }
      };

      result = 1;
    }

    break;

  }

  return result;
}


//--------------------------------------------------------------------------------------------
//TODO: This needs to be finished
int mnu_doOptions(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright;
  static float lerp;
  static int menuChoice = 0;

  int result = 0;

  switch (menuState)
  {
  case MM_Begin:
    // set up menu variables
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_gnome_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_optionsText, NULL, mnu_optionsTextLeft, mnu_optionsTextTop, 0, 0);

    menuChoice = 0;
    menuState = MM_Entering;

    mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_optionsButtons);
    initSlidyButtons(1.0f, mnu_widgetList, mnu_widgetCount);



    // let this fall through into MM_Entering

  case MM_Entering:
    // do buttons sliding in animation, and background fading in
    // background
    
    //Draw the background
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Copyright" text
    ui_drawTextBox(&wCopyright, 20);

    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(-deltaTime);

    // Let lerp wind down relative to the time elapsed
    if (SlidyButtonState.lerp <= 0.0f)
    {
      menuState = MM_Running;
    }

    break;

  case MM_Running:
    // Do normal run
    // Background
    glColor4f(1, 1, 1, 1);
    ui_drawImage(&wBackground);

    // "Options" text
    ui_drawTextBox(&wCopyright, 20);

    // Buttons
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      //audio options
      menuChoice = 1;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      //input options
      menuChoice = 2;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
      //video options
      menuChoice = 3;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[3]))
    {
      //back to main menu
      menuChoice = 4;
    }

    if (menuChoice != 0)
    {
      menuState = MM_Leaving;
      mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_optionsButtons);
      initSlidyButtons(0.0f, mnu_widgetList, mnu_widgetCount);
    }
    break;

  case MM_Leaving:
    // Do buttons sliding out and background fading
    // Do the same stuff as in MM_Entering, but backwards
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Options" text
    ui_drawTextBox(&wCopyright, 20);

    // Buttons
    mnu_drawSlidyButtons();
    mnu_updateSlidyButtons(deltaTime);
    if (SlidyButtonState.lerp >= 1.0f)
    {
      menuState = MM_Finish;
    }
    break;

  case MM_Finish:
    // Free the background texture; don't need to hold onto it
    GLTexture_Release(&background);
    menuState = MM_Begin; // Make sure this all resets next time mnu_doMain is called

    // Set the next menu to load
    result = menuChoice;
    break;

  }
  return result;
}

//--------------------------------------------------------------------------------------------
int mnu_doAudioOptions(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright;
  static float lerp;
  static int menuChoice = 0;
  static char Cmaxsoundchannel[128];
  static char Cbuffersize[128];
  static char Csoundvolume[128];
  static char Cmusicvolume[128];

  int result = 0;

  switch (menuState)
  {
  case MM_Begin:
    // set up menu variables
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_gnome_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    ui_initWidget(&mnu_widgetList[0], 0, mnu_Font, mnu_audioOptionsText[0], NULL, mnu_buttonLeft + 150, displaySurface->h - 270, 100, 30);
    ui_initWidget(&mnu_widgetList[1], 1, mnu_Font, mnu_audioOptionsText[1], NULL, mnu_buttonLeft + 150, displaySurface->h - 235, 100, 30);
    ui_initWidget(&mnu_widgetList[2], 2, mnu_Font, mnu_audioOptionsText[2], NULL, mnu_buttonLeft + 150, displaySurface->h - 165, 100, 30);
    ui_initWidget(&mnu_widgetList[3], 3, mnu_Font, mnu_audioOptionsText[3], NULL, mnu_buttonLeft + 150, displaySurface->h - 130, 100, 30);
    ui_initWidget(&mnu_widgetList[4], 4, mnu_Font, mnu_audioOptionsText[4], NULL, mnu_buttonLeft + 450, displaySurface->h - 200, 100, 30);
    ui_initWidget(&mnu_widgetList[5], 5, mnu_Font, mnu_audioOptionsText[5], NULL, mnu_buttonLeft + 450, displaySurface->h - 165, 100, 30);
    ui_initWidget(&mnu_widgetList[6], 6, mnu_Font, mnu_audioOptionsText[6], NULL, mnu_buttonLeft, displaySurface->h - 60, 200, 30);


    menuChoice = 0;
    menuState = MM_Entering;
    // let this fall through into MM_Entering

  case MM_Entering:
    // do buttons sliding in animation, and background fading in
    // background
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);

    //Draw the background
    ui_drawImage(&wBackground);

    //Load the current settings
    if (CData.soundvalid) mnu_widgetList[0].text = "On";
    else mnu_widgetList[0].text = "Off";

    snprintf(Csoundvolume, sizeof(Csoundvolume), "%i", CData.soundvolume);
    mnu_widgetList[1].text = Csoundvolume;

    if (CData.musicvalid) mnu_widgetList[2].text = "On";
    else mnu_widgetList[2].text = "Off";

    snprintf(Cmusicvolume, sizeof(Cmusicvolume), "%i", CData.musicvolume);
    mnu_widgetList[3].text = Cmusicvolume;

    snprintf(Cmaxsoundchannel, sizeof(Cmaxsoundchannel), "%i", CData.maxsoundchannel);
    mnu_widgetList[4].text = Cmaxsoundchannel;

    snprintf(Cbuffersize, sizeof(Cbuffersize), "%i", CData.buffersize);
    mnu_widgetList[5].text = Cbuffersize;

    //Fall trough
    menuState = MM_Running;
    break;

  case MM_Running:
    // Do normal run
    // Background
    glColor4f(1, 1, 1, 1);
    ui_drawImage(&wBackground);

    fnt_drawTextBox(mnu_Font, "Sound:", mnu_buttonLeft, displaySurface->h - 270, 0, 0, 20);
    // Buttons
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      if (CData.soundvalid)
      {
        CData.soundvalid = bfalse;
        mnu_widgetList[0].text = "Off";
      }
      else
      {
        CData.soundvalid = btrue;
        mnu_widgetList[0].text = "On";
      }
    }

    fnt_drawTextBox(mnu_Font, "Sound Volume:", mnu_buttonLeft, displaySurface->h - 235, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {
      CData.soundvolume += 5;
      CData.soundvolume %= 100;

      snprintf(Csoundvolume, sizeof(Csoundvolume), "%i", CData.soundvolume);
      mnu_widgetList[1].text = Csoundvolume;
    }

    fnt_drawTextBox(mnu_Font, "Music:", mnu_buttonLeft, displaySurface->h - 165, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
      if (CData.musicvalid)
      {
        CData.musicvalid = bfalse;
        mnu_widgetList[2].text = "Off";
      }
      else
      {
        CData.musicvalid = btrue;
        mnu_widgetList[2].text = "On";
      }
    }

    fnt_drawTextBox(mnu_Font, "Music Volume:", mnu_buttonLeft, displaySurface->h - 130, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[3]))
    {
      CData.musicvolume += 5;
      CData.musicvolume %= 100;

      snprintf(Cmusicvolume, sizeof(Cmusicvolume), "%i", CData.musicvolume);
      mnu_widgetList[3].text = Cmusicvolume;
    }

    fnt_drawTextBox(mnu_Font, "Sound Channels:", mnu_buttonLeft + 300, displaySurface->h - 200, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[4]))
    {
      switch (CData.maxsoundchannel)
      {
      case 32:
        CData.maxsoundchannel = 8;
        break;

      case 8:
        CData.maxsoundchannel = 16;
        break;

      case 16:
        CData.maxsoundchannel = 24;
        break;

      case 24:
        CData.maxsoundchannel = 32;
        break;

      default:
        CData.maxsoundchannel = 16;
        break;

      }

      snprintf(Cmaxsoundchannel, sizeof(Cmaxsoundchannel), "%i", CData.maxsoundchannel);
      mnu_widgetList[4].text = Cmaxsoundchannel;
    }

    fnt_drawTextBox(mnu_Font, "Buffer Size:", mnu_buttonLeft + 300, displaySurface->h - 165, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[5]))
    {
      switch (CData.buffersize)
      {
      case 8192:
        CData.buffersize = 512;
        break;

      case 512:
        CData.buffersize = 1024;
        break;

      case 1024:
        CData.buffersize = 2048;
        break;

      case 2048:
        CData.buffersize = 4096;
        break;

      case 4096:
        CData.buffersize = 8192;
        break;

      default:
        CData.buffersize = 2048;
        break;

      }
      snprintf(Cbuffersize, sizeof(Cbuffersize), "%i", CData.buffersize);
      mnu_widgetList[5].text = Cbuffersize;
    }

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[6]))
    {
      //save settings and go back
      mnu_saveSettings();
      if (CData.musicvalid) play_music(0, 0, -1);
      else if (mixeron) Mix_PauseMusic();
      if (!CData.musicvalid && !CData.soundvalid)
      {
        Mix_CloseAudio();
        mixeron = bfalse;
      }
      menuState = MM_Leaving;
    }

    break;

  case MM_Leaving:
    // Do buttons sliding out and background fading
    // Do the same stuff as in MM_Entering, but backwards
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    //Fall trough
    menuState = MM_Finish;
    break;

  case MM_Finish:
    // Free the background texture; don't need to hold onto it
    GLTexture_Release(&background);
    menuState = MM_Begin; // Make sure this all resets next time mnu_doMain is called

    // Set the next menu to load
    result = 1;
    break;

  }
  return result;
}

//--------------------------------------------------------------------------------------------
int mnu_doVideoOptions(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static GLTexture background;
  static ui_Widget wBackground, wCopyright;
  static float lerp;
  static int menuChoice = 0;
  int result = 0;
  static char Cmaxmessage[128];
  static char Cscrd[128];
  static char Cscrz[128];

  int dmode_min,  dmode, imode_min = 0, cnt;

  switch (menuState)
  {
  case MM_Begin:
    // set up menu variables
    snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s/%s", CData.basicdat_dir, CData.mnu_dir, CData.menu_gnome_bitmap);
    GLTexture_Load(GL_TEXTURE_2D, &background, CStringTmp1, INVALID_KEY);

    ui_initWidget(&wBackground, UI_Nothing, ui_getFont(), NULL, &background, (displaySurface->w - background.imgW), 0, 0, 0);
    ui_initWidget(&wCopyright,  UI_Nothing, ui_getFont(), mnu_copyrightText, NULL, mnu_copyrightLeft, mnu_copyrightTop, 0, 0);

    ui_initWidget(&mnu_widgetList[ 0],  0, mnu_Font, mnu_videoOptionsText[ 0], NULL, mnu_buttonLeft + 150, displaySurface->h - 215, 100, 30);
    ui_initWidget(&mnu_widgetList[ 1],  1, mnu_Font, mnu_videoOptionsText[ 1], NULL, mnu_buttonLeft + 150, displaySurface->h - 180, 100, 30);
    ui_initWidget(&mnu_widgetList[ 2],  2, mnu_Font, mnu_videoOptionsText[ 2], NULL, mnu_buttonLeft + 150, displaySurface->h - 145, 100, 30);
    ui_initWidget(&mnu_widgetList[ 3],  3, mnu_Font, mnu_videoOptionsText[ 3], NULL, mnu_buttonLeft + 150, displaySurface->h - 110, 100, 30);
    ui_initWidget(&mnu_widgetList[ 4],  4, mnu_Font, mnu_videoOptionsText[ 4], NULL, mnu_buttonLeft + 150, displaySurface->h - 250, 100, 30);
    ui_initWidget(&mnu_widgetList[ 5],  5, mnu_Font, mnu_videoOptionsText[ 5], NULL, mnu_buttonLeft + 150, displaySurface->h - 285, 130, 30);
    ui_initWidget(&mnu_widgetList[ 6],  6, mnu_Font, mnu_videoOptionsText[ 6], NULL, mnu_buttonLeft + 150, displaySurface->h - 320, 100, 30);
    ui_initWidget(&mnu_widgetList[ 7],  7, mnu_Font, mnu_videoOptionsText[ 7], NULL, mnu_buttonLeft + 450, displaySurface->h - 320, 100, 30);
    ui_initWidget(&mnu_widgetList[ 8],  8, mnu_Font, mnu_videoOptionsText[ 8], NULL, mnu_buttonLeft + 450, displaySurface->h - 285, 100, 30);
    ui_initWidget(&mnu_widgetList[ 9],  9, mnu_Font, mnu_videoOptionsText[ 9], NULL, mnu_buttonLeft + 450, displaySurface->h - 250, 100, 30);
    ui_initWidget(&mnu_widgetList[10], 10, mnu_Font, mnu_videoOptionsText[10], NULL, mnu_buttonLeft + 450, displaySurface->h - 215, 100, 30);
    ui_initWidget(&mnu_widgetList[11], 11, mnu_Font, mnu_videoOptionsText[11], NULL, mnu_buttonLeft + 450, displaySurface->h - 145,  75, 30);
    ui_initWidget(&mnu_widgetList[12], 12, mnu_Font, mnu_videoOptionsText[12], NULL, mnu_buttonLeft + 450, displaySurface->h - 110, 125, 30);
    ui_initWidget(&mnu_widgetList[13], 13, mnu_Font, mnu_videoOptionsText[13], NULL, mnu_buttonLeft +   0, displaySurface->h -  60, 200, 30);
    menuChoice = 0;

    //scan the video_mode_list to find the mode closest to our own
    dmode_min = MAX(ABS(CData.scrx - video_mode_list[0]->w), ABS(CData.scry - video_mode_list[0]->h));
    imode_min = 0;
    for (cnt = 1; NULL != video_mode_list[cnt]; ++cnt)
    {
      dmode = MAX(ABS(CData.scrx - video_mode_list[cnt]->w), ABS(CData.scry - video_mode_list[cnt]->h));

      if (dmode < dmode_min)
      {
        dmode_min = dmode;
        imode_min = cnt;
      };
    };

    video_mode_chaged = bfalse;
    mnu_display_mode_index = imode_min;
    if (dmode_min != 0)
    {
      CData.scrx = video_mode_list[imode_min]->w;
      CData.scry = video_mode_list[imode_min]->h;
      video_mode_chaged = btrue;
    };

    menuState = MM_Entering;  // let this fall through into MM_Entering

  case MM_Entering:
    // do buttons sliding in animation, and background fading in
    // background
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);

    //Draw the background
    ui_drawImage(&wBackground);

    //Load all the current video settings
    if (CData.antialiasing) mnu_widgetList[0].text = "On";
    else mnu_widgetList[0].text = "Off";

	switch(CData.particletype)
	{
	case PART_NORMAL:
		mnu_widgetList[1].text = "Normal";
	break;

	case PART_SMOOTH:
		mnu_widgetList[1].text = "Smooth";
	break;

	case PART_FAST:
		mnu_widgetList[1].text = "Fast";
	break;

	default:
		CData.particletype = PART_NORMAL;
		mnu_widgetList[1].text = "Normal";
	break;
	}

    if (CData.texturefilter == TX_UNFILTERED)
    {
      mnu_widgetList[5].text = "Unfiltered";
    }
    else if (CData.texturefilter == TX_LINEAR)
    {
      mnu_widgetList[5].text = "Linear";
    }
    else if (CData.texturefilter == TX_MIPMAP)
    {
      mnu_widgetList[5].text = "Mipmap";
    }
    else if (CData.texturefilter == TX_BILINEAR)
    {
      mnu_widgetList[5].text = "Bilinear";
    }
    else if (CData.texturefilter == TX_TRILINEAR_1)
    {
      mnu_widgetList[5].text = "Tlinear 1";
    }
    else if (CData.texturefilter == TX_TRILINEAR_2)
    {
      mnu_widgetList[5].text = "Tlinear 2";
    }
    else if (CData.texturefilter >= TX_ANISOTROPIC)
    {
      snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%s%d", "AIso ", userAnisotropy),
      mnu_widgetList[5].text = mnu_filternamebuffer;
    }

    if(CData.autoturncamera == btrue) mnu_widgetList[2].text = "On";
	else if(CData.autoturncamera == bfalse) mnu_widgetList[2].text = "Off";
	else if(CData.autoturncamera == 255) mnu_widgetList[2].text = "Quick";

    if (CData.fullscreen) mnu_widgetList[3].text = "True";
    else mnu_widgetList[3].text = "False";

    if (CData.refon)
    {
      mnu_widgetList[4].text = "Low";
      if (CData.reffadeor == 0)
      {
        mnu_widgetList[4].text = "Medium";
        if (CData.zreflect) mnu_widgetList[4].text = "High";
      }
    }
    else mnu_widgetList[4].text = "Off";


    snprintf(Cmaxmessage, sizeof(Cmaxmessage), "%i", CData.maxmessage);
    if (CData.maxmessage > MAXMESSAGE || CData.maxmessage < 0) CData.maxmessage = MAXMESSAGE - 1;
    if (CData.maxmessage == 0) snprintf(Cmaxmessage, sizeof(Cmaxmessage), "None");       //Set to default
    mnu_widgetList[11].text = Cmaxmessage;

    if (CData.shaon)
    {
      mnu_widgetList[6].text = "Normal";
      if (!CData.shasprite) mnu_widgetList[6].text = "Best";
    }
    else mnu_widgetList[6].text = "Off";

    if (CData.scrz != 32 && CData.scrz != 16 && CData.scrz != 24)
    {
      CData.scrz = 16;       //Set to default
    }
    snprintf(Cscrz, sizeof(Cscrz), "%i", CData.scrz);    //Convert the integer to a char we can use
    mnu_widgetList[7].text = Cscrz;

    if (CData.fogallowed) mnu_widgetList[8].text = "Enable";
    else mnu_widgetList[8].text = "Disable";

    if (CData.dither)
    {
      mnu_widgetList[9].text = "Okay";
      if (CData.overlayvalid && CData.backgroundvalid)
      {
        mnu_widgetList[9].text = "Good";
        if (GL_NICEST==CData.perspective) mnu_widgetList[9].text = "Superb";
      }
      else              //Set to defaults
      {
        CData.perspective     = GL_FASTEST;
        CData.backgroundvalid = bfalse;
        CData.overlayvalid    = bfalse;
      }
    }
    else               //Set to defaults
    {
      CData.perspective      = GL_FASTEST;
      CData.backgroundvalid  = bfalse;
      CData.overlayvalid     = bfalse;
	    CData.dither           = bfalse;
      mnu_widgetList[9].text = "Off";
    }

    if (CData.twolayerwateron) mnu_widgetList[10].text = "On";
    else mnu_widgetList[10].text = "Off";


    if (NULL == video_mode_list[mnu_display_mode_index])
    {
      video_mode_chaged = btrue;
      mnu_display_mode_index = 0;
      CData.scrx = video_mode_list[mnu_display_mode_index]->w;
      CData.scry = video_mode_list[mnu_display_mode_index]->h;
    };

    snprintf(mnu_display_mode_buffer, sizeof(mnu_display_mode_buffer), "%dx%d", CData.scrx, CData.scry);
    mnu_widgetList[12].text = mnu_display_mode_buffer;

    menuState = MM_Running;
    break;

  case MM_Running:
    // Do normal run
    // Background
    glColor4f(1, 1, 1, 1);
    ui_drawImage(&wBackground);

    //Antialiasing Button
    fnt_drawTextBox(mnu_Font, "Antialiasing:", mnu_buttonLeft, displaySurface->h - 215, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      if (CData.antialiasing)
      {
        mnu_widgetList[0].text = "Off";
        CData.antialiasing = bfalse;
      }
      else
      {
        mnu_widgetList[0].text = "On";
        CData.antialiasing = btrue;
      }
    }

    //Color depth
    fnt_drawTextBox(mnu_Font, "Particle Effects:", mnu_buttonLeft, displaySurface->h - 180, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[1]))
    {

      switch(CData.particletype)
      {

      case PART_FAST:
        mnu_widgetList[1].text = "Normal";
        CData.particletype = PART_NORMAL;
        break;

      case PART_NORMAL:
        mnu_widgetList[1].text = "Smooth";
        CData.particletype = PART_SMOOTH;
        break;

      case PART_SMOOTH:
        mnu_widgetList[1].text = "Fast";
        CData.particletype = PART_FAST;
        break;

      default:
        mnu_widgetList[1].text = "Normal";
        CData.particletype = PART_NORMAL;
        break;
      }
    }

    //Autoturn camera
    fnt_drawTextBox(mnu_Font, "Autoturn Camera:", mnu_buttonLeft, displaySurface->h - 145, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[2]))
    {
		  if(CData.autoturncamera == 255)
		  {
			  mnu_widgetList[2].text = "Off";
			  CData.autoturncamera = bfalse;
		  }
		  else if(!CData.autoturncamera)
		  {
			  mnu_widgetList[2].text = "On";
			  CData.autoturncamera = btrue;
		  }
		  else
		  {
			  mnu_widgetList[2].text = "Quick";
			  CData.autoturncamera = 255;
		  }
    }

    //Fullscreen
    fnt_drawTextBox(mnu_Font, "Fullscreen:", mnu_buttonLeft, displaySurface->h - 110, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[3]))
    {
      if (CData.fullscreen)
      {
        mnu_widgetList[3].text = "False";
        CData.fullscreen = bfalse;
      }
      else
      {
        mnu_widgetList[3].text = "True";
        CData.fullscreen = btrue;
      }
    }

    //Reflection
    fnt_drawTextBox(mnu_Font, "Reflections:", mnu_buttonLeft, displaySurface->h - 250, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[4]))
    {
      if (CData.refon && CData.reffadeor == 0 && CData.zreflect)
      {
        CData.refon = bfalse;
        CData.reffadeor = bfalse;
        CData.zreflect = bfalse;
        mnu_widgetList[4].text = "Off";
      }
      else
      {
        if (CData.refon && CData.reffadeor == 255 && !CData.zreflect)
        {
          mnu_widgetList[4].text = "Medium";
          CData.reffadeor = 0;
          CData.zreflect = bfalse;
        }
        else
        {
          if (CData.refon && CData.reffadeor == 0 && !CData.zreflect)
          {
            mnu_widgetList[4].text = "High";
            CData.zreflect = btrue;
          }
          else
          {
            CData.refon = btrue;
            CData.reffadeor = 255;    //Just in case so we dont get stuck at "Low"
            mnu_widgetList[4].text = "Low";
            CData.zreflect = bfalse;
          }
        }
      }
    }

    //Texture Filtering
    fnt_drawTextBox(mnu_Font, "Texture Filtering:", mnu_buttonLeft, displaySurface->h - 285, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[5]))
    {
      if (CData.texturefilter == TX_UNFILTERED)
      {
        CData.texturefilter++;
        mnu_widgetList[5].text = "Linear";
      }
      else if (CData.texturefilter == TX_LINEAR)
      {
        CData.texturefilter++;
        mnu_widgetList[5].text = "Mipmap";
      }
      else if (CData.texturefilter == TX_MIPMAP)
      {
        CData.texturefilter++;
        mnu_widgetList[5].text = "Bilinear";
      }
      else if (CData.texturefilter == TX_BILINEAR)
      {
        CData.texturefilter++;
        mnu_widgetList[5].text = "Tlinear 1";
      }
      else if (CData.texturefilter == TX_TRILINEAR_1)
      {
        CData.texturefilter++;
        mnu_widgetList[5].text = "Tlinear 2";
      }
      else if (CData.texturefilter >= TX_TRILINEAR_2 && CData.texturefilter < TX_ANISOTROPIC + log2Anisotropy - 1)
      {
        CData.texturefilter++;
        userAnisotropy = 1 << (CData.texturefilter - TX_ANISOTROPIC + 1);
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%s%d", "AIso ", userAnisotropy),
        mnu_widgetList[5].text = mnu_filternamebuffer;
      }
      else
      {
        CData.texturefilter = TX_UNFILTERED;
        mnu_widgetList[5].text = "Unfiltered";
      }
    }

    //Shadows
    fnt_drawTextBox(mnu_Font, "Shadows:", mnu_buttonLeft, displaySurface->h - 320, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[6]))
    {
      if (CData.shaon && !CData.shasprite)
      {
        CData.shaon = bfalse;
        CData.shasprite = bfalse;        //Just in case
        mnu_widgetList[6].text = "Off";
      }
      else
      {
        if (CData.shaon && CData.shasprite)
        {
          mnu_widgetList[6].text = "Best";
          CData.shasprite = bfalse;
        }
        else
        {
          CData.shaon = btrue;
          CData.shasprite = btrue;
          mnu_widgetList[6].text = "Normal";
        }
      }
    }

    //Z bit
    fnt_drawTextBox(mnu_Font, "Z Bit:", mnu_buttonLeft + 300, displaySurface->h - 320, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[7]))
    {
      switch (CData.scrz)
      {
      case 32:
        mnu_widgetList[7].text = "16";
        CData.scrz = 16;
        break;

      case 16:
        mnu_widgetList[7].text = "24";
        CData.scrz = 24;
        break;

      case 24:
        mnu_widgetList[7].text = "32";
        CData.scrz = 32;
        break;

      default:
        mnu_widgetList[7].text = "16";
        CData.scrz = 16;
        break;

      }
    }

    //Fog
    fnt_drawTextBox(mnu_Font, "Fog Effects:", mnu_buttonLeft + 300, displaySurface->h - 285, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[8]))
    {
      if (CData.fogallowed)
      {
        CData.fogallowed = bfalse;
        mnu_widgetList[8].text = "Disable";
      }
      else
      {
        CData.fogallowed = btrue;
        mnu_widgetList[8].text = "Enable";
      }
    }

    //Perspective correction, overlay/background effects and phong mapping
    fnt_drawTextBox(mnu_Font, "3D Effects:", mnu_buttonLeft + 300, displaySurface->h - 250, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[9]))
    {
      if (CData.dither && (GL_FASTEST==CData.perspective) && CData.overlayvalid && CData.backgroundvalid)
      {
        CData.dither           = bfalse;
        CData.perspective      = GL_FASTEST;
        CData.overlayvalid     = bfalse;
        CData.backgroundvalid  = bfalse;
        mnu_widgetList[9].text = "Off";
      }
      else
      {
        if (!CData.dither)
        {
          mnu_widgetList[9].text = "Okay";
          CData.dither = btrue;
        }
        else
        {
          if ((GL_NICEST==CData.perspective) && CData.overlayvalid && CData.backgroundvalid)
          {
            mnu_widgetList[9].text = "Superb";
            CData.perspective = GL_NICEST;
          }
          else
          {
            CData.overlayvalid = btrue;
            CData.backgroundvalid = btrue;
            mnu_widgetList[9].text = "Good";
          }
        }
      }
    }

    //Water Quality
    fnt_drawTextBox(mnu_Font, "Pretty Water:", mnu_buttonLeft + 300, displaySurface->h - 215, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[10]))
    {
      if (CData.twolayerwateron)
      {
        mnu_widgetList[10].text = "Off";
        CData.twolayerwateron = bfalse;
      }
      else
      {
        mnu_widgetList[10].text = "On";
        CData.twolayerwateron = btrue;
      }
    }

    //Text messages
    fnt_drawTextBox(mnu_Font, "Max  Messages:", mnu_buttonLeft + 300, displaySurface->h - 145, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[11]))
    {
      if (CData.maxmessage != MAXMESSAGE)
      {
        CData.maxmessage++;
        CData.messageon = btrue;
        snprintf(Cmaxmessage, sizeof(Cmaxmessage), "%i", CData.maxmessage);   //Convert integer to a char we can use
      }
      else
      {
        CData.maxmessage = 0;
        CData.messageon = bfalse;
        snprintf(Cmaxmessage, sizeof(Cmaxmessage), "None");
      }
      mnu_widgetList[11].text = Cmaxmessage;
    }

    //Screen Resolution
    fnt_drawTextBox(mnu_Font, "Resolution:", mnu_buttonLeft + 300, displaySurface->h - 110, 0, 0, 20);
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[12]))
    {

      if (NULL == video_mode_list[mnu_display_mode_index])
      {
        video_mode_chaged = btrue;
        mnu_display_mode_index = 0;
      }
      else
      {
        video_mode_chaged = btrue;
        mnu_display_mode_index++;
        if (NULL == video_mode_list[mnu_display_mode_index])
        {
          mnu_display_mode_index = 0;
        }
      };

      CData.scrx = video_mode_list[mnu_display_mode_index]->w;
      CData.scry = video_mode_list[mnu_display_mode_index]->h;
      snprintf(mnu_display_mode_buffer, sizeof(mnu_display_mode_buffer), "%dx%d", CData.scrx, CData.scry);
      mnu_widgetList[12].text = mnu_display_mode_buffer;
    };

    //Save settings button
    if (BUTTON_UP == ui_doButton(&mnu_widgetList[13]))
    {
      menuChoice = 1;
      mnu_saveSettings();

      //Reload some of the graphics
      load_graphics();
    }

    if (menuChoice != 0)
    {
      menuState = MM_Leaving;
      mnu_widgetCount = mnu_initWidgetsList(mnu_widgetList, MAXWIDGET, mnu_videoOptionsText);
      initSlidyButtons(0.0f, mnu_widgetList, mnu_widgetCount);
    }
    break;

  case MM_Leaving:
    // Do buttons sliding out and background fading
    // Do the same stuff as in MM_Entering, but backwards
    glColor4f(1, 1, 1, 1 - SlidyButtonState.lerp);
    ui_drawImage(&wBackground);

    // "Options" text
    fnt_drawTextBox(mnu_Font, mnu_optionsText, mnu_optionsTextLeft, mnu_optionsTextTop, 0, 0, 20);

    //Fall trough
    menuState = MM_Finish;
    break;

  case MM_Finish:
    // Free the background texture; don't need to hold onto it
    GLTexture_Release(&background);
    menuState = MM_Begin; // Make sure this all resets next time mnu_doMain is called

    // Set the next menu to load
    result = menuChoice;
    break;
  }
  return result;
}


//--------------------------------------------------------------------------------------------
int mnu_doShowResults(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static Font *font;
  int x, y, i, result;
  
  switch (menuState)
  {
  case MM_Begin:
        font = ui_getFont();

    menuState = MM_Entering;

  case MM_Entering:
      menuState = MM_Running;

      ui_initWidget(&mnu_widgetList[0], UI_Nothing, mnu_Font, NULL, NULL, 30, 30, CData.scrx - 60, CData.scry - 60);


  case MM_Running:

    ui_drawButton(&mnu_widgetList[0]);

    x = 35;
    y = 35;
    glColor4f(1, 1, 1, 1);

    fnt_drawTextFormatted(font, x, y, "Module selected: %s", modlongname[mnu_selectedModule]);
    y += 35;

    if (importvalid)
    {
      for(i=0; i<mnu_selectedPlayerCount; i++)
      {
        fnt_drawTextFormatted(font, x, y, "Player selected: %s", loadplayername[mnu_selectedPlayer[i]]);
        y += 35;
      };
    }
    else
    {
      fnt_drawText(font, x, y, "Starting a new player.");
    }

    menuState = MM_Leaving;


  case MM_Leaving:
      menuState = MM_Finish;


  case MM_Finish:
    result = 1;

    menuState = MM_Begin;
  }

  return result;
}

//--------------------------------------------------------------------------------------------
int mnu_doNotImplemented(float deltaTime)
{
  static MenuStates menuState = MM_Begin;
  static char notImplementedMessage[] = "Not implemented yet!  Check back soon!";
  int result = 0;

  int x, y;
  int w, h;
  
  switch (menuState)
  {
  case MM_Begin:
    fnt_getTextSize(ui_getFont(), notImplementedMessage, &w, &h);
    w += 50; // add some space on the sides

    x = (displaySurface->w - w) / 2;
    y = (displaySurface->h -34) / 2;

    ui_initWidget(&mnu_widgetList[0], UI_Nothing, mnu_Font, notImplementedMessage, NULL, x, y, w, 30);

    menuState = MM_Entering;

  case MM_Entering:

      menuState = MM_Running;  


  case MM_Running:

    if (BUTTON_UP == ui_doButton(&mnu_widgetList[0]))
    {
      menuState = MM_Leaving; 
    }

    break;


  case MM_Leaving:
      menuState = MM_Finish;


  case MM_Finish:
    result = 1;

    menuState = MM_Begin;
  }

  return result;
}





//--------------------------------------------------------------------------------------------
int mnu_RunIngame(float deltaTime)
{
  static int whichMenu = mnu_Inventory;
  static int lastMenu  = mnu_Inventory;
  int result = 0;

  switch (whichMenu)
  {
  case mnu_Inventory:
    result = mnu_doNotImplemented(deltaTime);
    if (result != 0)
    {
      lastMenu = mnu_Inventory;
      if (result == 1) return -1;
    }
    break;

  default:
  case mnu_NotImplemented:
    result = mnu_doNotImplemented(deltaTime);
    if (result != 0)
    {
      whichMenu = lastMenu;
    }
  }

  return 0;
}


//--------------------------------------------------------------------------------------------
int mnu_Run(float deltaTime)
{
  static int whichMenu = mnu_Main;
  static int lastMenu = mnu_Main;
  int result = 0;

  switch (whichMenu)
  {
  case mnu_Main:
    result = mnu_doMain(deltaTime);
    if (result != 0)
    {
      lastMenu = mnu_Main;
      if (result == 1)      { whichMenu = mnu_ChooseModule; mnu_startNewPlayer = btrue; }
      else if (result == 2) { whichMenu = mnu_ChooseModule; mnu_startNewPlayer = bfalse; }
      else if (result == 3) whichMenu = mnu_Network;
      else if (result == 4) whichMenu = mnu_Options;
      else if (result == 5) return -1; // need to request a quit somehow
    }
    break;

  case mnu_SinglePlayer:
    result = mnu_doSinglePlayer(deltaTime);
    if (result != 0)
    {
      lastMenu = mnu_SinglePlayer;
      if (result == 1)
      {
        whichMenu = mnu_ChooseModule;
        mnu_startNewPlayer = btrue;
      }
      else if (result == 2)
      {
        whichMenu = mnu_ChoosePlayer;
        mnu_startNewPlayer = bfalse;
      }
      else if (result == 3) whichMenu = mnu_Main;
      else whichMenu = mnu_NewPlayer;
    }
    break;

  case mnu_ChooseModule:
    result = mnu_doChooseModule(deltaTime);
    if (result == -1) whichMenu = lastMenu;
    else if (result == 1) whichMenu = mnu_ChoosePlayer;
    else if (result == 2) whichMenu = mnu_TestResults;
    break;

  case mnu_ChoosePlayer:
    result = mnu_doChoosePlayer(deltaTime);
    if (result == -1)     whichMenu = lastMenu;
    else if (result == 1) whichMenu = mnu_TestResults;
    break;

  case mnu_Options:
    result = mnu_doOptions(deltaTime);
    if (result != 0)
    {
      if (result == -1)   whichMenu = lastMenu;
      else if (result == 1) whichMenu = mnu_AudioOptions;
      else if (result == 2) whichMenu = mnu_InputOptions;
      else if (result == 3) whichMenu = mnu_VideoOptions;
      else if (result == 4) whichMenu = mnu_Main;
    }
    break;

  case mnu_AudioOptions:
    result = mnu_doAudioOptions(deltaTime);
    if (result != 0)
    {
      whichMenu = mnu_Options;
    }
    break;

  case mnu_VideoOptions:
    result = mnu_doVideoOptions(deltaTime);
    if (result != 0)
    {
      whichMenu = mnu_Options;
    }
    break;

  case mnu_TestResults:
    result = mnu_doShowResults(deltaTime);
    if (result != 0)
    {
      whichMenu = mnu_Main;
      return 1;
    }
    break;

  default:
  case mnu_NotImplemented:
    result = mnu_doNotImplemented(deltaTime);
    if (result != 0)
    {
      whichMenu = lastMenu;
    }
  }

  return 0;
}

//--------------------------------------------------------------------------------------------
void mnu_frameStep()
{

}

//--------------------------------------------------------------------------------------------
void mnu_saveSettings()
{
  //This function saves all current game settings to setup.txt
  FILE* setupfile;
  STRING write;
  char *TxtTmp;

  setupfile = fs_fileOpen(PRI_NONE, NULL, CData.setup_file, "w");
  if (setupfile)
  {
    /*GRAPHIC PART*/
    fputs("{GRAPHIC}\n", setupfile);
    snprintf(write, sizeof(write), "[MAX_NUMBER_VERTICES] : \"%i\"\n", CData.maxtotalmeshvertices / 1024);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[COLOR_DEPTH] : \"%i\"\n", CData.scrd);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[Z_DEPTH] : \"%i\"\n", CData.scrz);
    fputs(write, setupfile);
    if (CData.fullscreen) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[FULLSCREEN] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.zreflect) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[Z_REFLECTION] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[SCREENSIZE_X] : \"%i\"\n", CData.scrx);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[SCREENSIZE_Y] : \"%i\"\n", CData.scry);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[MAX_TEXT_MESSAGE] : \"%i\"\n", CData.maxmessage);
    fputs(write, setupfile);
    if (CData.staton) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[STATUS_BAR] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (GL_NICEST==CData.perspective) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[PERSPECTIVE_CORRECT] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);

    if (CData.texturefilter >= TX_ANISOTROPIC && maxAnisotropy > 0.0f)
    {
      int anisotropy = MIN(maxAnisotropy, userAnisotropy);
      snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d ANISOTROPIC %d", CData.texturefilter, anisotropy);
    }
    else if (CData.texturefilter >= TX_ANISOTROPIC)
    {
      CData.texturefilter = TX_TRILINEAR_2;
      snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d TRILINEAR 2", CData.texturefilter);
    }
    else
    {
      switch (CData.texturefilter)
      {
      case TX_UNFILTERED:
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d UNFILTERED", CData.texturefilter);
        break;

      case TX_MIPMAP:
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d MIPMAP", CData.texturefilter);
        break;

      case TX_TRILINEAR_1:
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d TRILINEAR 1", CData.texturefilter);
        break;

      case TX_TRILINEAR_2:
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d TRILINEAR 2", CData.texturefilter);
        break;

      default:
      case TX_LINEAR:
        CData.texturefilter = TX_LINEAR;
        snprintf(mnu_filternamebuffer, sizeof(mnu_filternamebuffer), "%d LINEAR", CData.texturefilter);
        break;

      }
    }

    snprintf(write, sizeof(write), "[TEXTURE_FILTERING] : \"%s\"\n", mnu_filternamebuffer);
    fputs(write, setupfile);
    if (CData.shading == GL_SMOOTH) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[GOURAUD_SHADING] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.antialiasing) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[ANTIALIASING] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.dither) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[DITHERING] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.refon) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[REFLECTION] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.shaon) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[SHADOWS] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.shasprite) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[SHADOW_AS_SPRITE] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.phongon) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[PHONG] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.fogallowed) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[FOG] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.reffadeor == 0) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[FLOOR_REFLECTION_FADEOUT] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.twolayerwateron) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[MULTI_LAYER_WATER] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.overlayvalid) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[OVERLAY] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.backgroundvalid) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[BACKGROUND] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
	if(CData.particletype == PART_SMOOTH) TxtTmp = "SMOOTH"; 
	else if(CData.particletype == PART_NORMAL) TxtTmp = "NORMAL";
	else if(CData.particletype == PART_FAST) TxtTmp = "FAST";
    snprintf(write, sizeof(write), "[PARTICLE_EFFECTS] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);

    /*SOUND PART*/
    snprintf(write, sizeof(write), "\n{SOUND}\n");
    fputs(write, setupfile);
    if (CData.musicvalid) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[MUSIC] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[MUSIC_VOLUME] : \"%i\"\n", CData.musicvolume);
    fputs(write, setupfile);
    if (CData.soundvalid) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[SOUND] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[SOUND_VOLUME] : \"%i\"\n", CData.soundvolume);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[OUTPUT_BUFFER_SIZE] : \"%i\"\n", CData.buffersize);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[MAX_SOUND_CHANNEL] : \"%i\"\n", CData.maxsoundchannel);
    fputs(write, setupfile);

    /*CAMERA PART*/
    snprintf(write, sizeof(write), "\n{CONTROL}\n");
    fputs(write, setupfile);
    switch (CData.autoturncamera)
    {
    case 255: TxtTmp = "GOOD";
      break;

    case btrue: TxtTmp = "TRUE";
      break;

    case bfalse: TxtTmp = "FALSE";
      break;

    }
    snprintf(write, sizeof(write), "[AUTOTURN_CAMERA] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);

    /*NETWORK PART*/
    snprintf(write, sizeof(write), "\n{NETWORK}\n");
    fputs(write, setupfile);
    if (CData.networkon) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[NETWORK_ON] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    TxtTmp = CData.nethostname;
    snprintf(write, sizeof(write), "[HOST_NAME] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    TxtTmp = CData.netmessagename;
    snprintf(write, sizeof(write), "[MULTIPLAYER_NAME] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    snprintf(write, sizeof(write), "[LAG_TOLERANCE] : \"%i\"\n", CData.lag);
    fputs(write, setupfile);

    /*DEBUG PART*/
    snprintf(write, sizeof(write), "\n{DEBUG}\n");
    fputs(write, setupfile);
    if (CData.fpson) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[DISPLAY_FPS] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.HideMouse) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[HIDE_MOUSE] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (SDL_GRAB_ON==CData.GrabMouse) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[GRAB_MOUSE] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);
    if (CData.DevMode) TxtTmp = "TRUE"; else TxtTmp = "FALSE";
    snprintf(write, sizeof(write), "[DEVELOPER_MODE] : \"%s\"\n", TxtTmp);
    fputs(write, setupfile);

    //Close it up
    fs_fileClose(setupfile);
  }
  else
  {
    log_warning("Cannot open setup.txt to write new settings into.\n");
  }
}

//--------------------------------------------------------------------------------------------
//int mnu_doModel(float deltaTime)
//{
//  // Advance the model's animation
//  m_modelLerp += (float)deltaTime / kAnimateSpeed;
//  if (m_modelLerp >= 1.0f)
//  {
//   m_modelFrame = m_modelNextFrame;
//   m_modelNextFrame++;
//
//   // Roll back to the first frame if we run out of them
//   if (m_modelNextFrame >= m_model->numFrames())
//   {
//    m_modelNextFrame = 0;
//   }
//
//   m_modelLerp -= 1.0f;
//  }
//
//  float width, height;
//  width  = (float)CData.scrx;
//  height = (float)CData.scry;
//
//    glPushAttrib(GL_ENABLE_BIT|GL_DEPTH_BUFFER_BIT|GL_TRANSFORM_BIT|GL_VIEWPORT_BIT|GL_COLOR_BUFFER_BIT);
//  glMatrixMode(GL_PROJECTION);                              /* GL_TRANSFORM_BIT */
//  glLoadIdentity();                                         /* GL_TRANSFORM_BIT */
//  gluPerspective(60.0, width/height, 1, 2048);              /* GL_TRANSFORM_BIT */
//
//  glMatrixMode(GL_MODELVIEW);                               /* GL_TRANSFORM_BIT */
//  glLoadIdentity();                                         /* GL_TRANSFORM_BIT */
//  glTranslatef(20, -20, -75);                               /* GL_TRANSFORM_BIT */
//  glRotatef(-90, 1, 0, 0);                                  /* GL_TRANSFORM_BIT */
//  glRotatef(mnu_modelAngle, 0, 0, 1);                         /* GL_TRANSFORM_BIT */
//
//  glEnable(GL_TEXTURE_2D);
//  glEnable(GL_DEPTH_TEST);
//
//    GLTexture_Bind( &m_modelTxr, CData.texturefilter );
//  m_model->drawBlendedFrames(m_modelFrame, m_modelNextFrame, m_modelLerp);
// }
//};

/* Old Menu Code */
#if 0
//--------------------------------------------------------------------------------------------
//void draw_trimx(int x, int y, int length)
//{
//  // ZZ> This function draws a horizontal trim bar
//  GLfloat txWidth, txHeight, txLength;
//
//  if ( INVALID_TEXTURE != GLTexture_GetTextureID(&TxTrim) )
//  {
//    /*while( length > 0 )
//    {
//    trimrect.right = length;
//    if(length > TRIMX)  trimrect.right = TRIMX;
//    trimrect.bottom = 4;
//    lpDDSBack->BltFast(x, y, lpDDSTrimX, &trimrect, DDBLTFAST_NOCOLORKEY);
//    length-=TRIMX;
//    x+=TRIMX;
//    }*/
//
//    /* Calculate the texture width, height, and length */
//    txWidth = ( GLfloat )( GLTexture_GetImageWidth( &TxTrim )/GLTexture_GetTextureWidth( &TxTrim ) );
//    txHeight = ( GLfloat )( GLTexture_GetImageHeight( &TxTrim )/GLTexture_GetTextureHeight( &TxTrim ) );
//    txLength = ( GLfloat )( length/GLTexture_GetImageWidth( &TxTrim ) );
//
//
//    /* Bind our texture */
//    GLTexture_Bind( &TxTrim, CData.texturefilter );
//
//    /* Draw the trim */
//    glColor4f( 1, 1, 1, 1 );
//    glBegin( GL_QUADS );
//    glTexCoord2f( 0, 1 ); glVertex2f( x, CData.scry - y );
//    glTexCoord2f( 0, 1 - txHeight ); glVertex2f( x, CData.scry - y - GLTexture_GetImageHeight( &TxTrim ) );
//    glTexCoord2f( txWidth*txLength, 1 - txHeight ); glVertex2f( x + length, CData.scry - y - GLTexture_GetImageHeight( &TxTrim ) );
//    glTexCoord2f( txWidth*txLength, 1 ); glVertex2f( x + length, CData.scry - y );
//    glEnd();
//  }
//}
//
////--------------------------------------------------------------------------------------------
//void draw_trimy(int x, int y, int length)
//{
//  // ZZ> This function draws a vertical trim bar
//  GLfloat txWidth, txHeight, txLength;
//
//  if ( INVALID_TEXTURE != GLTexture_GetTextureID(&TxTrim) )
//  {
//    /*while(length > 0)
//    {
//    trimrect.bottom = length;
//    if(length > TRIMY)  trimrect.bottom = TRIMY;
//    trimrect.right = 4;
//    lpDDSBack->BltFast(x, y, lpDDSTrimY, &trimrect, DDBLTFAST_NOCOLORKEY);
//    length-=TRIMY;
//    y+=TRIMY;
//    }*/
//
//    /* Calculate the texture width, height, and length */
//    txWidth = ( GLfloat )( GLTexture_GetImageWidth( &TxTrim )/GLTexture_GetTextureWidth( &TxTrim ) );
//    txHeight = ( GLfloat )( GLTexture_GetImageHeight( &TxTrim )/GLTexture_GetTextureHeight( &TxTrim ) );
//    txLength = ( GLfloat )( length/GLTexture_GetImageHeight( &TxTrim ) );
//
//    /* Bind our texture */
//    GLTexture_Bind( &TxTrim, CData.texturefilter );
//
//    /* Draw the trim */
//    glColor4f( 1, 1, 1, 1 );
//    glBegin( GL_QUADS );
//    glTexCoord2f( 0, 1 ); glVertex2f( x, CData.scry - y );
//    glTexCoord2f( 0, 1 - txHeight*txLength ); glVertex2f( x, CData.scry - y - length );
//    glTexCoord2f( txWidth, 1 - txHeight*txLength ); glVertex2f( x + GLTexture_GetImageWidth( &TxTrim ), CData.scry - y - length );
//    glTexCoord2f( txWidth, 1 ); glVertex2f( x + GLTexture_GetImageWidth( &TxTrim ), CData.scry - y );
//    glEnd();
//  }
//}
//
////--------------------------------------------------------------------------------------------
//void draw_trim_box(int left, int top, int right, int bottom)
//{
//  // ZZ> This function draws a trim rectangle
//  float l,t,r,b;
//  l=((float)left)/CData.scrx;
//  r=((float)right)/CData.scrx;
//  t=((float)top)/CData.scry;
//  b=((float)bottom)/CData.scry;
//
//  Begin2DMode();
//
//  draw_trimx(left, top, right-left);
//  draw_trimx(left, bottom-4, right-left);
//  draw_trimy(left, top, bottom-top);
//  draw_trimy(right-4, top, bottom-top);
//
//  End2DMode();
//}
//
////--------------------------------------------------------------------------------------------
//void draw_trim_box_opening(int left, int top, int right, int bottom, float amount)
//{
//  // ZZ> This function draws a trim rectangle, scaled around its center
//  int x = (left + right)>>1;
//  int y = (top + bottom)>>1;
//  left   = (x * (1.0-amount)) + (left * amount);
//  right  = (x * (1.0-amount)) + (right * amount);
//  top    = (y * (1.0-amount)) + (top * amount);
//  bottom = (y * (1.0-amount)) + (bottom * amount);
//  draw_trim_box(left, top, right, bottom);
//}

////--------------------------------------------------------------------------------------------
//void mnu_service_select()
//{
//    // ZZ> This function lets the user choose a network service to use
//    char text[256];
//    int x, y;
//    float open;
//    int cnt;
//    int stillchoosing;
//
//
//    networkservice = NONETWORK;
//    if(numservice > 0)
//    {
//        // Open a big window
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            draw_trim_box_opening(0, 0, 320, fontyspacing*(numservice+4), open);
//            request_pageflip();
//            open += .030;
//        }
//        // Tell the user which ones we found ( in setup_network )
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box(0, 0, CData.scrx, CData.scry);
//            draw_trim_box(0, 0, 320, fontyspacing*(numservice+4));
//            y = 8;
//            sprintf(text, "Network options...");
//            draw_string(text, 14, y);
//            y += fontyspacing;
//            cnt = 0;
//            while(cnt < numservice)
//            {
//                sprintf(text, "%s", netservicename[cnt]);
//                draw_string(text, 50, y);
//                y += fontyspacing;
//                cnt++;
//            }
//            sprintf(text, "No Network");
//            draw_string(text, 50, y);
//            do_cursor();
//            x = cursorx - 50;
//            y = (cursory - 8 - fontyspacing);
//            if(x > 0 && x < 300 && y >= 0)
//            {
//                y = y/fontyspacing;
//                if(y <= numservice)
//                {
//                    if(mousebutton[0] || mousebutton[1])
//                    {
//                        stillchoosing = bfalse;
//                        networkservice = y;
//                    }
//                }
//            }
//            request_pageflip();
//        }
//    }
////    turn_on_service(networkservice);
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_start_or_join()
//{
//    // ZZ> This function lets the user start or join a game for a network game
//    char text[256];
//    int x, y;
//    float open;
//    int stillchoosing;
//
//
//    // Open another window
//    if(CData.networkon)
//    {
//        open = 0;
//        while(open < 1.0)
//        {
//   //clear_surface(lpDDSBack);
//   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//   glLoadIdentity();
//   draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//   draw_trim_box_opening(0, 0, 280, 102, open);
//   request_pageflip();
//   open += .030;
//  }
//        // Give the user some options
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//   //clear_surface(lpDDSBack);
//   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//   glLoadIdentity();
//   draw_trim_box(0, 0, CData.scrx, CData.scry);
//   draw_trim_box(0, 0, 280, 102);
//
//   // Draw the menu text
//   y = 8;
//   sprintf(text, "Game options...");
//   draw_string(text, 14, y);
//   y += fontyspacing;
//   sprintf(text, "New Game");
//   draw_string(text, 50, y);
//   y += fontyspacing;
//   sprintf(text, "Join Game");
//   draw_string(text, 50, y);
//   y += fontyspacing;
//   sprintf(text, "Quit Game");
//   draw_string(text, 50, y);
//
//   do_cursor();
//
////   sprintf(text, "Cursor position: %03d, %03d", cursorx, cursory);
////   draw_string(text, 14, 400);
//
//   x = cursorx - 50;
//   // The adjustments to y here were figured out empirically; I still
//   // don't understand the reasoning behind it.  I don't think the text
//   // draws where it says it's going to.
//   y = (cursory - 21 - fontyspacing);
//
//
//
//            if(x > 0 && x < 280 && y >= 0)
//            {
//                y = y/fontyspacing;
//                if(y < 3)
//                {
//                    if(mousebutton[0] || mousebutton[1])
//                    {
//                        if(y == 0)
//                        {
//                            if(sv_hostGame())
//                            {
//                                hostactive = btrue;
//                                nextmenu = MENUD;
//                                stillchoosing = bfalse;
//                            }
//                        }
//                        if(y == 1 && networkservice != NONETWORK)
//                        {
//                            nextmenu = MENUC;
//                            stillchoosing = bfalse;
//                        }
//                        if(y == 2)
//                        {
//                            nextmenu = MENUB;
//                            menuactive = bfalse;
//                            stillchoosing = bfalse;
//                            gameActive = bfalse;
//                        }
//                    }
//                }
//            }
//            request_pageflip();
//        }
//    }
//    else
//    {
//        hostactive = btrue;
//        nextmenu = MENUD;
//    }
//}
//
////--------------------------------------------------------------------------------------------
//void draw_module_tag(int module, int y)
//{
//    // ZZ> This function draws a module tag
//    char text[256];
//    draw_trim_box(0, y, 136, y+136);
//    draw_trim_box(132, y, CData.scrx, y+136);
//    if(module < globalnummodule)
//    {
//        draw_titleimage(module, 4, y+4);
//        y+=6;
//        sprintf(text, "%s", modlongname[module]);  draw_string(text, 150, y);  y+=fontyspacing;
//        sprintf(text, "%s", modrank[module]);  draw_string(text, 150, y);  y+=fontyspacing;
//        if(modmaxplayers[module] > 1)
//        {
//            if(modminplayers[module]==modmaxplayers[module])
//            {
//                sprintf(text, "%d players", modminplayers[module]);
//            }
//            else
//            {
//                sprintf(text, "%d-%d players", modminplayers[module], modmaxplayers[module]);
//            }
//        }
//        else
//        {
//            sprintf(text, "1 player");
//        }
//        draw_string(text, 150, y);  y+=fontyspacing;
//        if(modimportamount[module] == 0 && !modallowexport[module])
//        {
//            sprintf(text, "No Import/Export");  draw_string(text, 150, y);  y+=fontyspacing;
//        }
//        else
//        {
//            if(modimportamount[module] == 0)
//            {
//                sprintf(text, "No Import");  draw_string(text, 150, y);  y+=fontyspacing;
//            }
//            if(!modallowexport[module])
//            {
//                sprintf(text, "No Export");  draw_string(text, 150, y);  y+=fontyspacing;
//            }
//        }
//        if(!modrespawnvalid[module])
//        {
//            sprintf(text, "No Respawn");  draw_string(text, 150, y);  y+=fontyspacing;
//        }
//        if(modrtscontrol[module])
//        {
//            sprintf(text, "RTS");  draw_string(text, 150, y);  y+=fontyspacing;
//        }
//        if(modrtscontrol[module] == ALLSELECT)
//        {
//            sprintf(text, "Diaboo RTS");  draw_string(text, 150, y);  y+=fontyspacing;
//        }
//    }
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_pick_player(int module)
//{
//    // ZZ> This function handles the display for picking players to import
//    int x, y;
//    float open;
//    int cnt, tnc, start, numshow;
//    int stillchoosing;
//    int import;
//    Uint8 control, sparkle;
//    char fromdir[128];
//    char todir[128];
// int clientFilesSent = 0;
// int hostFilesSent = 0;
// int pending;
//
//    // Set the important flags
//    respawnvalid = bfalse;
//    respawnanytime = bfalse;
//    if(modrespawnvalid[module])  respawnvalid = btrue;
//    if(modrespawnvalid[module]==ANYTIME)  respawnanytime = btrue;
//    rtscontrol = bfalse;
//    if(modrtscontrol[module] != bfalse)
//    {
//        rtscontrol = btrue;
//        allselect = bfalse;
//        if(modrtscontrol[module] == ALLSELECT)
//            allselect = btrue;
//    }
//    exportvalid = modallowexport[module];
//    importvalid = (modimportamount[module] > 0);
//    importamount = modimportamount[module];
//    playeramount = modmaxplayers[module];
//    fs_createDirectory("import");  // Just in case...
//
//
//    start = 0;
//    if(importvalid)
//    {
//        // Figure out which characters are available
//        check_player_import("players");
//        numshow = (CData.scry-80-fontyspacing-fontyspacing)>>5;
//
//
//        // Open some windows
//        y = fontyspacing + 8;
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            draw_trim_box_opening(0, 0, CData.scrx, 40, open);
//            draw_trim_box_opening(0, CData.scry-40, CData.scrx, CData.scry, open);
//            request_pageflip();
//            open += .030;
//        }
//
//
//        wldframe = 0;  // For sparkle
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//            // Draw the windows
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box(0, 0, CData.scrx, CData.scry);
//            draw_trim_box(0, 40, CData.scrx, CData.scry-40);
//
//            // Draw the Up/Down buttons
//            if(start == 0)
//            {
//                // Show the instructions
//                x = (CData.scrx-270)>>1;
//                draw_string("Setup controls", x, 10);
//            }
//            else
//            {
//                x = (CData.scrx-40)>>1;
//                draw_string("Up", x, 10);
//            }
//            x = (CData.scrx-80)>>1;
//            draw_string("Down", x, CData.scry-fontyspacing-20);
//
//
//            // Draw each import character
//            y = 40+fontyspacing;
//            cnt = 0;
//            while(cnt < numshow && cnt + start < numloadplayer)
//            {
//                sparkle = NOSPARKLE;
//                if(keybplayer == (cnt+start))
//                {
//                    draw_one_icon(keybicon, 32, y, NOSPARKLE);
//                    sparkle = 0;  // White
//                }
//                else
//                    draw_one_icon(nullicon, 32, y, NOSPARKLE);
//                if(mousplayer == (cnt+start))
//                {
//                    draw_one_icon(mousicon, 64, y, NOSPARKLE);
//                    sparkle = 0;  // White
//                }
//                else
//                    draw_one_icon(nullicon, 64, y, NOSPARKLE);
//                if(joyaplayer == (cnt+start) && joyaon)
//                {
//                    draw_one_icon(joyaicon, 128, y, NOSPARKLE);
//                    sparkle = 0;  // White
//                }
//                else
//                    draw_one_icon(nullicon, 128, y, NOSPARKLE);
//                if(joybplayer == (cnt+start) && joybon)
//                {
//                    draw_one_icon(joybicon, 160, y, NOSPARKLE);
//                    sparkle = 0;  // White
//                }
//                else
//                    draw_one_icon(nullicon, 160, y, NOSPARKLE);
//                draw_one_icon((cnt+start), 96, y, sparkle);
//                draw_string(loadplayername[cnt+start], 200, y+6);
//                y+=32;
//                cnt++;
//            }
//            wldframe++;  // For sparkle
//
//
//            // Handle other stuff...
//            do_cursor();
//            if(pending_click)
//            {
//         pending_click=bfalse;
//                if(cursory < 40 && start > 0)
//                {
//                    // Up button
//                    start--;
//                }
//                if(cursory >= (CData.scry-40) && (start + numshow) < numloadplayer)
//                {
//                    // Down button
//                    start++;
//                }
//            }
//            if(mousebutton[0])
//            {
//                x = (cursorx - 32) >> 5;
//                y = (cursory - 44) >> 5;
//                if(y >= 0 && y < numshow)
//                {
//                    y += start;
//                    // Assign the controls
//                    if(y < numloadplayer)  // !!!BAD!!! do scroll
//                    {
//                        if(x == 0)  keybplayer = y;
//                        if(x == 1)  mousplayer = y;
//                        if(x == 3)  joyaplayer = y;
//                        if(x == 4)  joybplayer = y;
//                    }
//                }
//            }
//            if(mousebutton[1])
//            {
//                // Done picking
//                stillchoosing = bfalse;
//            }
//            request_pageflip();
//        }
//        wldframe = 0;  // For sparkle
//
//
//  // Tell the user we're loading
//  y = fontyspacing + 8;
//  open = 0;
//  while(open < 1.0)
//  {
//   //clear_surface(lpDDSBack);
//   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//   glLoadIdentity();
//   draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//   request_pageflip();
//   open += .030;
//  }
//
//  //clear_surface(lpDDSBack);
//  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//  glLoadIdentity();
//  draw_trim_box(0, 0, CData.scrx, CData.scry);
//  draw_string("Copying the imports...", y, y);
//  request_pageflip();
//
//
//        // Now build the import directory...
//        empty_import_directory();
//        cnt = 0;
//        numimport = 0;
//        while(cnt < numloadplayer)
//        {
//            if((cnt == keybplayer && keyon)   ||
//               (cnt == mousplayer && mouseon) ||
//               (cnt == joyaplayer && joyaon)  ||
//               (cnt == joybplayer && joybon))
//            {
//                // This character has been selected
//                control = INPUTNONE;
//                if(cnt == keybplayer)  control |= INPUTKEY;
//                if(cnt == mousplayer)  control |= INPUTMOUSE;
//                if(cnt == joyaplayer)  control |= INPUTJOYA;
//                if(cnt == joybplayer)  control |= INPUTJOYB;
//                localcontrol[numimport] = control;
////                localslot[numimport] = (numimport+(localmachine*4))*9;
//    localslot[numimport] = (numimport + localmachine) * 9;
//
//
//                // Copy the character to the import directory
//                sprintf(fromdir, "players/%s", loadplayerdir[cnt]);
//                sprintf(todir, "import/temp%04d.obj", localslot[numimport]);
//
//    // This will do a local copy if I'm already on the host machine, other
//    // wise the directory gets sent across the network to the host
//    net_copyDirectoryToHost(fromdir, todir);
//
//    // Copy all of the character's items to the import directory
//    tnc = 0;
//    while(tnc < 8)
//    {
//     sprintf(fromdir, "players/%s/%d.obj", loadplayerdir[cnt], tnc);
//     sprintf(todir, "import/temp%04d.obj", localslot[numimport]+tnc+1);
//
//     net_copyDirectoryToHost(fromdir, todir);
//     tnc++;
//    }
//
//                numimport++;
//            }
//            cnt++;
//        }
//
//  // Have clients wait until all files have been sent to the host
//  clientFilesSent = net_pendingFileTransfers();
//  if(CData.networkon && !hostactive)
//  {
//   pending = net_pendingFileTransfers();
//
//   // Let the host know how many files you're sending it
//   net_startNewPacket();
//   packet_addUnsignedShort(NET_NUM_FILES_TO_SEND);
//   packet_addUnsignedShort((Uint16)pending);
//   net_sendPacketToHostGuaranteed();
//
//   while(pending)
//   {
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//    glLoadIdentity();
//
//    draw_trim_box(0, 0, CData.scrx, CData.scry);
//    y = fontyspacing + 8;
//
//    sprintf(todir, "Sending file %d of %d...", clientFilesSent - pending, clientFilesSent);
//    draw_string(todir, fontyspacing + 8, y);
//    request_pageflip();
//
//    // do this to let SDL do it's window events stuff, so that windows doesn't think
//    // the game has hung while transferring files
//    do_cursor();
//
//    net_updateFileTransfers();
//
//    pending = net_pendingFileTransfers();
//   }
//
//   // Tell the host I'm done sending files
//   net_startNewPacket();
//   packet_addUnsignedShort(NET_DONE_SENDING_FILES);
//   net_sendPacketToHostGuaranteed();
//  }
//
//  if(CData.networkon)
//  {
//   if(hostactive)
//   {
//    // Host waits for all files from all remotes
//    numfile = 0;
//    numfileexpected = 0;
//    numplayerrespond = 1;
//    while(numplayerrespond < numplayer)
//    {
//     //clear_surface(lpDDSBack);
//     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//     glLoadIdentity();
//     draw_trim_box(0, 0, CData.scrx, CData.scry);
//     y = fontyspacing + 8;
//     draw_string("Incoming files...", fontyspacing+8, y);  y+=fontyspacing;
//     sprintf(todir, "File %d/%d", numfile, numfileexpected);
//     draw_string(todir, fontyspacing+20, y); y+=fontyspacing;
//     sprintf(todir, "Play %d/%d", numplayerrespond, numplayer);
//     draw_string(todir, fontyspacing+20, y);
//     request_pageflip();
//
//     listen_for_packets();
//
//     do_cursor();
//
//     if(SDLKEYDOWN(SDLK_ESCAPE))
//     {
//      gameActive = bfalse;
//      menuactive = bfalse;
//      close_session();
//      break;

//     }
//    }
//
//
//                // Say you're done
//                //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//                draw_trim_box(0, 0, CData.scrx, CData.scry);
//                y = fontyspacing + 8;
//                draw_string("Sending files to remotes...", fontyspacing+8, y);  y+=fontyspacing;
//                request_pageflip();
//
//
//                // Host sends import directory to all remotes, deletes extras
//                numfilesent = 0;
//                import = 0;
//                cnt = 0;
//    if(numplayer > 1)
//    {
//     while(cnt < MAXIMPORT)
//     {
//      sprintf(todir, "import/temp%04d.obj", cnt);
//      strncpy(fromdir, todir, 128);
//      if(fs_fileIsDirectory(fromdir))
//      {
//       // Only do directories that actually exist
//       if((cnt % 9)==0) import++;
//       if(import > importamount)
//       {
//        // Too many directories
//        fs_removeDirectoryAndContents(fromdir);
//       }
//       else
//       {
//        // Ship it out
//        net_copyDirectoryToAllPlayers(fromdir, todir);
//       }
//      }
//      cnt++;
//     }
//
//     hostFilesSent = net_pendingFileTransfers();
//     pending = hostFilesSent;
//
//     // Let the client know how many are coming
//     net_startNewPacket();
//     packet_addUnsignedShort(NET_NUM_FILES_TO_SEND);
//     packet_addUnsignedShort((Uint16)pending);
//     net_sendPacketToAllPlayersGuaranteed();
//
//     while(pending > 0)
//     {
//      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//      glLoadIdentity();
//
//      draw_trim_box(0, 0, CData.scrx, CData.scry);
//      y = fontyspacing + 8;
//
//      sprintf(todir, "Sending file %d of %d...", hostFilesSent - pending, hostFilesSent);
//      draw_string(todir, fontyspacing + 8, y);
//      request_pageflip();
//
//      // do this to let SDL do it's window events stuff, so that windows doesn't think
//      // the game has hung while transferring files
//      do_cursor();
//
//      net_updateFileTransfers();
//
//      pending = net_pendingFileTransfers();
//     }
//
//     // Tell the players I'm done sending files
//     net_startNewPacket();
//     packet_addUnsignedShort(NET_DONE_SENDING_FILES);
//     net_sendPacketToAllPlayersGuaranteed();
//    }
//            }
//            else
//            {
//    // Remotes wait for all files in import directory
//    log_info("mnu_pick_player: Waiting for files to come from the host...\n");
//    numfile = 0;
//    numfileexpected = 0;
//    numplayerrespond = 0;
//    while(numplayerrespond < 1)
//    {
//     //clear_surface(lpDDSBack);
//     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//     glLoadIdentity();
//     draw_trim_box(0, 0, CData.scrx, CData.scry);
//     y = fontyspacing + 8;
//     draw_string("Incoming files from host...", fontyspacing+8, y);  y+=fontyspacing;
//     sprintf(todir, "File %d/%d", numfile, numfileexpected);
//     draw_string(todir, fontyspacing+20, y);
//     request_pageflip();
//
//     listen_for_packets();
//     do_cursor();
//
//     if(SDLKEYDOWN(SDLK_ESCAPE))
//     {
//      gameActive = bfalse;
//      menuactive = bfalse;
//      break;

//      close_session();
//     }
//                }
//            }
//        }
//    }
//    nextmenu = MENUG;
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_module_loading(int module)
//{
//    // ZZ> This function handles the display for when a module is loading
//    char text[256];
//    int y;
//    float open;
//    int cnt;
//
//
//    // Open some windows
//    y = fontyspacing + 8;
//    open = 0;
//    while(open < 1.0)
//    {
//        //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//        draw_trim_box_opening(0, y, 136, y+136, open);
//        draw_trim_box_opening(132, y, CData.scrx, y+136, open);
//        draw_trim_box_opening(0, y+132, CData.scrx, CData.scry, open);
//        request_pageflip();
//        open += .030;
//    }
//
//
//    // Put the stuff in the windows
//    //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//    y = 0;
//    sprintf(text, "Loading...  Wait!!!");  draw_string(text, 0, y);  y+=fontyspacing;
//    y+=8;
//    draw_module_tag(module, y);
//    draw_trim_box(0, y+132, CData.scrx, CData.scry);
//
//
//    // Show the summary
//    sprintf(text, "%s/%s/%s/%s", CData.modules_dir, modloadname[module], CData.gamedat_dir, CData.mnu_file);
//    get_module_summary(text);
//    y = fontyspacing+152;
//    cnt = 0;
//    while(cnt < SUMMARYLINES)
//    {
//        sprintf(text, "%s", modsummary[cnt]);  draw_string(text, 14, y);  y+=fontyspacing;
//        cnt++;
//    }
//    request_pageflip();
//    nextmenu = MENUB;
//    menuactive = bfalse;
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_join_multiplayer()
//{
// // JF> This function attempts to join the multiplayer game hosted
// //     by whatever server is named in the HOST_NAME part of setup.txt
// char text[256];
// float open;
//
// if(CData.networkon)
// {
//  // Do the little opening menu animation
//  open = 0;
//  while(open < 1.0)
//  {
//   //clear_surface(lpDDSBack);
//   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//   glLoadIdentity();
//   draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//   request_pageflip();
//   open += .030;
//  }
//
//  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//  glLoadIdentity();
//  draw_trim_box(0, 0, CData.scrx, CData.scry);
//
//  strncpy(text, "Attempting to join game at:", 256);
//  draw_string(text, (CData.scrx>>1)-240, (CData.scry>>1)-fontyspacing);
//
//  strncpy(text, CData.nethostname, 256);
//  draw_string(text, (CData.scrx>>1)-240, (CData.scry>>1));
//  request_pageflip();
//
//  if(cl_joinGame(CData.nethostname))
//  {
//   nextmenu = MENUE;
//  } else
//  {
//   nextmenu = MENUB;
//  }
// }
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_choose_host()
//{
//    // ZZ> This function lets the player choose a host
//    char text[256];
//    int x, y;
//    float open;
//    int cnt;
//    int stillchoosing;
//
//
//    if(CData.networkon)
//    {
//        // Bring up a helper window
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            request_pageflip();
//            open += .030;
//        }
//        //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//        draw_trim_box(0, 0, CData.scrx, CData.scry);
//        sprintf(text, "Press Enter if");
//        draw_string(text, (CData.scrx>>1)-120, (CData.scry>>1)-fontyspacing);
//        sprintf(text, "nothing happens");
//        draw_string(text, (CData.scrx>>1)-120, (CData.scry>>1));
//        request_pageflip();
//
//
//
//        // Find available games
////        find_open_sessions();       // !!!BAD!!!  Do this every now and then
//
//        // Open a big window
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            draw_trim_box_opening(0, 0, 320, fontyspacing*(numsession+4), open);
//            request_pageflip();
//            open += .030;
//        }
//
//        // Tell the user which ones we found
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box(0, 0, CData.scrx, CData.scry);
//            draw_trim_box(0, 0, 320, fontyspacing*(numsession+4));
//            y = 8;
//            sprintf(text, "Open hosts...");
//            draw_string(text, 14, y);
//            y += fontyspacing;
//            cnt = 0;
//            while(cnt < numsession)
//            {
//                sprintf(text, "%s", netsessionname[cnt]);
//                draw_string(text, 50, y);
//                y += fontyspacing;
//                cnt++;
//            }
//            sprintf(text, "Go Back...");
//            draw_string(text, 50, y);
//            do_cursor();
//            x = cursorx - 50;
//            y = (cursory - 8 - fontyspacing);
//            if(x > 0 && x < 300 && y >= 0)
//            {
//                y = y/fontyspacing;
//                if(y <= numsession)
//                {
//                    if(mousebutton[0] || mousebutton[1])
//                    {
//                        //if(y == numsession)
//                        //{
//                        //    nextmenu = MENUB;
//                        //    stillchoosing = bfalse;
//                        //}
//                        //else
//                        {
//                            if(cl_joinGame("solace2.csusm.edu"))
//                            {
//                                nextmenu = MENUE;
//                                stillchoosing = bfalse;
//                            }
//                        }
//                    }
//                }
//            }
//            request_pageflip();
//        }
//    }
//    else
//    {
//        // This should never happen
//        nextmenu = MENUB;
//    }
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_choose_module()
//{
//    // ZZ> This function lets the host choose a module
//    int numtag;
//    char text[256];
//    int x, y, ystt;
//    float open;
//    int cnt;
//    int module;
//    int stillchoosing;
//    if(hostactive)
//    {
//        // Figure out how many tags to display
//        numtag = (CData.scry-4-40)/132;
//        ystt = (CData.scry-(numtag*132)-4)>>1;
//
//
//        // Open the tag windows
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            y = ystt;
//            cnt = 0;
//            while(cnt < numtag)
//            {
//                draw_trim_box_opening(0, y, 136, y+136, open);
//                draw_trim_box_opening(132, y, CData.scrx, y+136, open);
//                y+=132;
//                cnt++;
//            }
//            request_pageflip();
//            open += .030;
//        }
//
//
//
//
//        // Let the user pick a module
//        module = 0;
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//            // Draw the tags
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box(0, 0, CData.scrx, CData.scry);
//            y = ystt;
//            cnt = 0;
//            while(cnt < numtag)
//            {
//                draw_module_tag(module+cnt, y);
//                y+=132;
//                cnt++;
//            }
//
//            // Draw the Up/Down buttons
//            sprintf(text, "Up");
//            x = (CData.scrx-40)>>1;
//            draw_string(text, x, 10);
//            sprintf(text, "Down");
//            x = (CData.scrx-80)>>1;
//            draw_string(text, x, CData.scry-fontyspacing-20);
//
//
//            // Handle the mouse
//            do_cursor();
//            y = (cursory - ystt)/132;
//            if(pending_click)
//       {
//         pending_click=bfalse;
//                if(cursory < ystt && module > 0)
//    {
//                    // Up button
//                    module--;
//    }
//                if(y >= numtag && module + numtag < globalnummodule)
//    {
//                    // Down button
//                    module++;
//    }
//  if(cursory > ystt && y > -1 && y < numtag)
//    {
//      y = module + y;
//      if((mousebutton[0] || mousebutton[1]) && y < globalnummodule)
//        {
//   // Set start infow
//   playersready = 1;
//   seed = time(0);
//   pickedindex = y;
//   sprintf(pickedmodule, "%s", modloadname[y]);
//   readytostart = btrue;
//   stillchoosing = bfalse;
//        }
//    }
//       }
//            // Check for quitters
//            if(SDLKEYDOWN(SDLK_ESCAPE) && networkservice == NONETWORK)
//            {
//                nextmenu = MENUB;
//                menuactive = bfalse;
//                stillchoosing = bfalse;
//                gameActive = bfalse;
//            }
//            request_pageflip();
//        }
//    }
//    nextmenu = MENUE;
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_boot_players()
//{
//    // ZZ> This function shows all the active players and lets the host kick 'em out
//    //     !!!BAD!!!  Let the host boot players
//    char text[256];
//    int x, y, starttime, time;
//    float open;
//    int cnt, player;
//    int stillchoosing;
//
//
//    numplayer = 1;
//    if(CData.networkon)
//    {
//        // Find players
//        sv_letPlayersJoin();
//
//        // Open a big window
//        open = 0;
//        while(open < 1.0)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//            draw_trim_box_opening(0, 0, 320, fontyspacing*(numplayer+4), open);
//            request_pageflip();
//            open += .030;
//        }
//
//        // Tell the user which ones we found
//        starttime = SDL_GetTicks();
//        stillchoosing = btrue;
//        while(stillchoosing)
//        {
//   time = SDL_GetTicks();
//   if((time-starttime) > NETREFRESH)
//   {
//    sv_letPlayersJoin();
//    starttime = time;
//   }
//
//   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//   glLoadIdentity();
//   draw_trim_box(0, 0, CData.scrx, CData.scry);
//   draw_trim_box(0, 0, 320, fontyspacing*(numplayer+4));
//
//   if(hostactive)
//   {
//    y = 8;
//    sprintf(text, "Active machines...");
//    draw_string(text, 14, y);
//    y += fontyspacing;
//
//    cnt = 0;
//    while(cnt < numplayer)
//    {
//     sprintf(text, "%s", netplayername[cnt]);
//     draw_string(text, 50, y);
//     y += fontyspacing;
//     cnt++;
//    }
//
//    sprintf(text, "Start Game");
//                draw_string(text, 50, y);
//   } else
//   {
//    strncpy(text, "Connected to host:", 256);
//    draw_string(text, 14, 8);
//    draw_string(CData.nethostname, 14, 8 + fontyspacing);
//    listen_for_packets();  // This happens implicitly for the host in sv_letPlayersJoin
//   }
//
//            do_cursor();
//            x = cursorx - 50;
//   // Again, y adjustments were figured out empirically in mnu_start_or_join
//            y = (cursory - 21 - fontyspacing);
//
//            if(SDLKEYDOWN(SDLK_ESCAPE)) // !!!BAD!!!
//            {
//                nextmenu = MENUB;
//                menuactive = bfalse;
//                stillchoosing = bfalse;
//                gameActive = bfalse;
//            }
//            if(x > 0 && x < 300 && y >= 0 && (mousebutton[0] || mousebutton[1]) && hostactive)
//            {
//                // Let the host do things
//                y = y/fontyspacing;
//                if(y < numplayer && hostactive)
//                {
//                    // Boot players
//                }
//                if(y == numplayer && readytostart)
//                {
//                    // Start the modules
//                    stillchoosing = bfalse;
//                }
//            }
//            if(readytostart && !hostactive)
//            {
//                // Remotes automatically start
//                stillchoosing = bfalse;
//            }
//            request_pageflip();
//        }
//    }
//    if(CData.networkon && hostactive)
//    {
//        // Let the host coordinate start
//        stop_players_from_joining();
//        sv_letPlayersJoin();
//        cnt = 0;
//        readytostart = bfalse;
//        if(numplayer == 1)
//        {
//            // Don't need to bother, since the host is alone
//            readytostart = btrue;
//        }
//        while(!readytostart)
//        {
//            //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//            draw_trim_box(0, 0, CData.scrx, CData.scry);
//            y = 8;
//            sprintf(text, "Waiting for replies...");
//            draw_string(text, 14, y);
//            y += fontyspacing;
//            do_cursor();
//            if(SDLKEYDOWN(SDLK_ESCAPE)) // !!!BAD!!!
//            {
//                nextmenu = MENUB;
//                menuactive = bfalse;
//                stillchoosing = bfalse;
//                gameActive = bfalse;
//                readytostart = btrue;
//            }
//            if((cnt&63)==0)
//            {
//                sprintf(text, "  Lell...");
//                draw_string(text, 14, y);
//                player = 0;
//                while(player < numplayer-1)
//                {
//                    net_startNewPacket();
//                    packet_addUnsignedShort(TO_REMOTE_MODULE);
//                    packet_addUnsignedInt(seed);
//                    packet_addUnsignedByte(player+1);
//                    packet_addString(pickedmodule);
////                    send_packet_to_all_players();
//                    net_sendPacketToOnePlayerGuaranteed(player);
//                    player++;
//                }
//            }
//            listen_for_packets();
//            cnt++;
//            request_pageflip();
//        }
//    }
//
//
//    nextmenu=MENUF;
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_end_text()
//{
//    // ZZ> This function gives the player the ending text
//    float open;
//    int stillchoosing;
////    SDL_Event ev;
//
//
//    // Open the text window
//    open = 0;
//    while(open < 1.0)
//    {
//        //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//        draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//        request_pageflip();
//        open += .030;
//    }
//
//
//
//    // Wait for input
//    stillchoosing = btrue;
//    while(stillchoosing)
//    {
//        // Show the text
//        //clear_surface(lpDDSBack);
//    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//       glLoadIdentity();
//        draw_trim_box(0, 0, CData.scrx, CData.scry);
//        draw_wrap_string(endtext, 14, 8, CData.scrx-40);
//
//
//
//        // Handle the mouse
//        do_cursor();
//        if(pending_click || SDLKEYDOWN(SDLK_ESCAPE))
//        {
//     pending_click = bfalse;
//            stillchoosing = bfalse;
//        }
//        request_pageflip();
//    }
//    nextmenu = MENUB;
//}
//
////--------------------------------------------------------------------------------------------
//void mnu_initial_text()
//{
//    // ZZ> This function gives the player the initial title screen
//    float open;
//    char text[1024];
//    int stillchoosing;
//
//
//    //fprintf(stderr,"DIAG: In mnu_initial_text()\n");
//    //draw_trim_box(0, 0, CData.scrx, CData.scry);//draw_trim_box(60, 60, 320, 200); // JUST TEST BOX
//
//    // Open the text window
//    open = 0;
//    while(open < 1.0)
//    {
//        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//     glLoadIdentity();
//
//        // clear_surface(lpDDSBack); PORT!
//        draw_trim_box_opening(0, 0, CData.scrx, CData.scry, open);
//        request_pageflip();
//        open += .030;
//    }
//
// /*fprintf(stderr,"waiting to read a scanf\n");
//    scanf("%s",text);
//    exit(0);*/
//
//    // Wait for input
//    stillchoosing = btrue;
//    while(stillchoosing)
//    {
//        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
//     glLoadIdentity();
//
//        // Show the text
//        // clear_surface(lpDDSBack); PORT!
//        draw_trim_box(0, 0, CData.scrx, CData.scry);
//        sprintf(text, "Egoboo v2.22");
//        draw_string(text, (CData.scrx>>1)-200, ((CData.scry>>1)-30));
//        sprintf(text, "http://egoboo.sourceforge.net");
//        draw_string(text, (CData.scrx>>1)-200, ((CData.scry>>1)));
//        sprintf(text, "See controls.txt to configure input");
//        draw_string(text, (CData.scrx>>1)-200, ((CData.scry>>1)+30));
//
//  // get input
//  read_input(NULL);
//
//        // Handle the mouse
//        do_cursor();
//  if ( pending_click || SDLKEYDOWN(SDLK_ESCAPE) )
//  {
//          pending_click = bfalse;
//   stillchoosing = bfalse;
//  }
//        request_pageflip();
//    }
//    nextmenu = MENUA;
//}
//
////--------------------------------------------------------------------------------------------
//void fiddle_with_menu()
//{
//    // ZZ> This function gives a nice little menu to play around in.
//
//    menuactive = btrue;
//    readytostart = bfalse;
//    playersready = 0;
//    localmachine = 0;
//    rtslocalteam = 0;
//    numfile = 0;
//    numfilesent = 0;
//    numfileexpected = 0;
//    while(menuactive)
//    {
//        switch(nextmenu)
//        {
//            case MENUA:
//                // MENUA...  Let the user choose a network service
//                //printf("MENUA\n");
//                if(menuaneeded)
//                {
//                    mnu_service_select();
//                    menuaneeded = bfalse;
//                }
//                nextmenu = MENUB;
//                break;

//            case MENUB:
//                // MENUB...  Let the user start or join
//                //printf("MENUB\n");
//                mnu_start_or_join();
//                break;

//            case MENUC:
//                // MENUC...  Choose an open game to join
//                //printf("MENUC\n");
//                //mnu_choose_host();
//    mnu_join_multiplayer();
//                break;

//            case MENUD:
//                // MENUD...  Choose a module to run
//                //printf("MENUD\n");
//                mnu_choose_module();
//                break;

//            case MENUE:
//                // MENUE...  Wait for all the players
//                //printf("MENUE\n");
//                mnu_boot_players();
//                break;

//            case MENUF:
//                // MENUF...  Let the players choose characters
//                //printf("MENUF\n");
//                mnu_pick_player(pickedindex);
//                break;

//            case MENUG:
//                // MENUG...  Let the user read while it loads
//                //printf("MENUG\n");
//                mnu_module_loading(pickedindex);
//                break;

//            case MENUH:
//                // MENUH...  Show the end text
//                //printf("MENUH\n");
//                mnu_end_text();
//                break;

//            case MENUI:
//                // MENUI...  Show the initial text
//                //printf("MENUI\n");
//                mnu_initial_text();
//                break;

//        }
//    }
//    //printf("Left menu system\n");
//}
//
////--------------------------------------------------------------------------------------------
//void release_menu_trim()
//{
// // ZZ> This function frees the menu trim memory
// //GLTexture_Release( &TxTrimX );  //RELEASE(lpDDSTrimX);
// //GLTexture_Release( &TxTrimY );  //RELEASE(lpDDSTrimY);
// GLTexture_Release( &TxBlip );  //RELEASE(lpDDSBlip);
// GLTexture_Release( &TxTrim );
//
//}
//
////--------------------------------------------------------------------------------------------
//void release_menu()
//{
// // ZZ> This function releases all the menu images
// GLTexture_Release( &TxFont );  //RELEASE(lpDDSFont);
//    release_all_titleimages();
//    release_all_icons();
//
//}
#endif
