/* Egoboo - char.c
*/

/*
This file is part of Egoboo.

Egoboo is free software: you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Egoboo is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with Egoboo.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "mathstuff.h"
#include "Network.h"
#include "Client.h"
#include "Server.h"
#include "Log.h"
#include "egoboo.h"
#include <assert.h>

//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------
void flash_character_height(Uint16 character, Uint8 valuelow, Sint16 low,
                            Uint8 valuehigh, Sint16 high)
{
  // ZZ> This function sets a character's lighting depending on vertex height...
  //     Can make feet dark and head light...
  int cnt;
  Uint16 frame;
  Sint16 z;


  frame = chrframe[character];
  cnt = 0;
  while (cnt < madtransvertices[chrmodel[character]])
  {
    z = madvrtz[frame][cnt];
    if (z < low)
    {
      chrvrta[character][cnt] = valuelow;
    }
    else
    {
      if (z > high)
      {
        chrvrta[character][cnt] = valuehigh;
      }
      else
      {
        chrvrta[character][cnt] = (valuehigh * (z - low) / (high - low)) +
          (valuelow * (high - z) / (high - low));
      }
    }
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void flash_character(Uint16 character, Uint8 value)
{
  // ZZ> This function sets a character's lighting
  int cnt;

  cnt = 0;
  while (cnt < madtransvertices[chrmodel[character]])
  {
    chrvrta[character][cnt] = value;
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void flash_select(float dUpdate)
{
  /*   // ZZ> This function makes the selected characters blink
  int cnt;
  Uint8 value;

  if((wldframe&31)==0 && !allselect)
  {
  value = ((wldframe&32)<<3) - ((wldframe&32)>>5);
  cnt = 0;
  while(cnt < numrtsselect)
  {
  flash_character(rtsselect[cnt], value);
  cnt++;
  }
  }*/
}

//--------------------------------------------------------------------------------------------
void add_to_dolist(int cnt)
{
  // This function puts a character in the list
  int fan;


  if (!chrindolist[cnt])
  {
    fan = chronwhichfan[cnt];
    if (meshinrenderlist[fan])
    {
      //chrlightspek[cnt] = meshvrtl[meshvrtstart[fan]];
      dolist[numdolist] = cnt;
      chrindolist[cnt] = btrue;
      numdolist++;


      // Do flashing
      if ((allframe&chrflashand[cnt]) == 0 && chrflashand[cnt] != DONTFLASH)
      {
        flash_character(cnt, 255);
      }
      // Do blacking
      if ((allframe&SEEKURSEAND) == 0 && localseekurse && chriskursed[cnt])
      {
        flash_character(cnt, 0);
      }
    }
    else
    {
      // Double check for large/special objects
      if (capalwaysdraw[chrmodel[cnt]])
      {
        dolist[numdolist] = cnt;
        chrindolist[cnt] = btrue;
        numdolist++;
      }
    }
    // Add its weapons too
    if (chrholdingwhich[cnt][0] != MAXCHR)
      add_to_dolist(chrholdingwhich[cnt][0]);
    if (chrholdingwhich[cnt][1] != MAXCHR)
      add_to_dolist(chrholdingwhich[cnt][1]);
  }
}

//--------------------------------------------------------------------------------------------
void order_dolist(void)
{
  // ZZ> This function orders the dolist based on distance from camera,
  //     which is needed for reflections to properly clip themselves.
  //     Order from closest to farthest
  int cnt, tnc, character, order;
  int dist[MAXCHR];
  Uint16 olddolist[MAXCHR];


  // Figure the distance of each
  cnt = 0;
  while (cnt < numdolist)
  {
    character = dolist[cnt];  olddolist[cnt] = character;
    if (chrlight[character] != 255 || chralpha[character] != 255)
    {
      // This makes stuff inside an invisible character visible...
      // A key inside a Jellcube, for example
      dist[cnt] = 0x7fffffff;
    }
    else
    {
      dist[cnt] = ABS(chrxpos[character] - camx) + ABS(chrypos[character] - camy);
    }
    cnt++;
  }


  // Put em in the right order
  cnt = 0;
  while (cnt < numdolist)
  {
    character = olddolist[cnt];
    order = 0;  // Assume this character is closest
    tnc = 0;
    while (tnc < numdolist)
    {
      // For each one closer, increment the order
      order += (dist[cnt] > dist[tnc]);
      order += (dist[cnt] == dist[tnc]) && (cnt < tnc);
      tnc++;
    }
    dolist[order] = character;
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void make_dolist(void)
{
  // ZZ> This function finds the characters that need to be drawn and puts them in the list
  int cnt, character;


  // Remove everyone from the dolist
  cnt = 0;
  while (cnt < numdolist)
  {
    character = dolist[cnt];
    chrindolist[character] = bfalse;
    cnt++;
  }
  numdolist = 0;


  // Now fill it up again
  cnt = 0;
  while (cnt < MAXCHR)
  {
    if (chron[cnt] && (!chrinpack[cnt]))
    {
      // Add the character
      add_to_dolist(cnt);
    }
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void keep_weapons_with_holders()
{
  // ZZ> This function keeps weapons near their holders
  int cnt, character;


  // !!!BAD!!!  May need to do 3 levels of attachment...
  cnt = 0;
  while (cnt < MAXCHR)
  {
    if (chron[cnt])
    {
      character = chrattachedto[cnt];
      if (character == MAXCHR)
      {
        // Keep inventory with character
        if (!chrinpack[cnt])
        {
          character = chrnextinpack[cnt];
          while (character != MAXCHR)
          {
            chrxpos[character] = chrxpos[cnt];
            chrypos[character] = chrypos[cnt];
            chrzpos[character] = chrzpos[cnt];
            // Copy olds to make SendMessageNear work
            chroldx[character] = chrxpos[cnt];
            chroldy[character] = chrypos[cnt];
            chroldz[character] = chrzpos[cnt];
            character = chrnextinpack[character];
          }
        }
      }
      else
      {
        // Keep in hand weapons with character
        if (chrmatrixvalid[character] && chrmatrixvalid[cnt])
        {
          chrxpos[cnt] = chrmatrix[cnt]_CNV(3, 0);
          chrypos[cnt] = chrmatrix[cnt]_CNV(3, 1);
          chrzpos[cnt] = chrmatrix[cnt]_CNV(3, 2);
        }
        else
        {
          chrxpos[cnt] = chrxpos[character];
          chrypos[cnt] = chrypos[character];
          chrzpos[cnt] = chrzpos[character];
        }
        chrturnleftright[cnt] = chrturnleftright[character];
        // Copy this stuff ONLY if it's a weapon, not for mounts
        if (chrtransferblend[character] && chrisitem[cnt])
        {
          if (chralpha[character] != 255)
          {
            chralpha[cnt] = chralpha[character];
          }
          if (chrlight[character] != 255)
          {
            chrlight[cnt] = chrlight[character];
          }
        }
      }
    }
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void make_turntosin(void)
{
  // ZZ> This function makes the lookup table for chrturn...
  int cnt;

  cnt = 0;
  while (cnt < 16384)
  {
    turntosin[cnt] = sin((TWO_PI * cnt) / (float)TRIGTABLE_SIZE);
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void make_one_character_matrix(Uint16 cnt)
{
  // ZZ> This function sets one character's matrix
  Uint16 tnc;
  chrmatrixvalid[cnt] = btrue;
  if (chroverlay[cnt])
  {
    // Overlays are kept with their target...
    tnc = chraitarget[cnt];
    chrxpos[cnt] = chrxpos[tnc];
    chrypos[cnt] = chrypos[tnc];
    chrzpos[cnt] = chrzpos[tnc];
    chrmatrix[cnt]_CNV(0, 0) = chrmatrix[tnc]_CNV(0, 0);
    chrmatrix[cnt]_CNV(0, 1) = chrmatrix[tnc]_CNV(0, 1);
    chrmatrix[cnt]_CNV(0, 2) = chrmatrix[tnc]_CNV(0, 2);
    chrmatrix[cnt]_CNV(0, 3) = chrmatrix[tnc]_CNV(0, 3);
    chrmatrix[cnt]_CNV(1, 0) = chrmatrix[tnc]_CNV(1, 0);
    chrmatrix[cnt]_CNV(1, 1) = chrmatrix[tnc]_CNV(1, 1);
    chrmatrix[cnt]_CNV(1, 2) = chrmatrix[tnc]_CNV(1, 2);
    chrmatrix[cnt]_CNV(1, 3) = chrmatrix[tnc]_CNV(1, 3);
    chrmatrix[cnt]_CNV(2, 0) = chrmatrix[tnc]_CNV(2, 0);
    chrmatrix[cnt]_CNV(2, 1) = chrmatrix[tnc]_CNV(2, 1);
    chrmatrix[cnt]_CNV(2, 2) = chrmatrix[tnc]_CNV(2, 2);
    chrmatrix[cnt]_CNV(2, 3) = chrmatrix[tnc]_CNV(2, 3);
    chrmatrix[cnt]_CNV(3, 0) = chrmatrix[tnc]_CNV(3, 0);
    chrmatrix[cnt]_CNV(3, 1) = chrmatrix[tnc]_CNV(3, 1);
    chrmatrix[cnt]_CNV(3, 2) = chrmatrix[tnc]_CNV(3, 2);
    chrmatrix[cnt]_CNV(3, 3) = chrmatrix[tnc]_CNV(3, 3);
  }
  else
  {
    chrmatrix[cnt] = ScaleXYZRotateXYZTranslate(chrscale[cnt], chrscale[cnt], chrscale[cnt],
      chrturnleftright[cnt] >> 2,
      ((Uint16)(chrturnmapud[cnt] + 32768)) >> 2,
      ((Uint16)(chrturnmaplr[cnt] + 32768)) >> 2,
      chrxpos[cnt], chrypos[cnt], chrzpos[cnt]);
  }
}

//--------------------------------------------------------------------------------------------
void free_one_character(Uint16 character)
{
  // ZZ> This function sticks a character back on the free character stack
  int cnt;

  freechrlist[numfreechr] = character;
  numfreechr++;
  // Remove from stat list
  if (chrstaton[character])
  {
    chrstaton[character] = bfalse;
    cnt = 0;
    while (cnt < numstat)
    {
      if (statlist[cnt] == character)
      {
        cnt++;
        while (cnt < numstat)
        {
          statlist[cnt-1] = statlist[cnt];
          cnt++;
        }
        numstat--;
      }
      cnt++;
    }
  }
  // Make sure everyone knows it died
  if (chralive[character] && !capinvictus[chrmodel[character]])
  {
    teammorale[chrbaseteam[character]]--;
  }
  cnt = 0;
  while (cnt < MAXCHR)
  {
    if (chron[cnt])
    {
      if (chraitarget[cnt] == character)
      {
        chralert[cnt] |= ALERTIFTARGETKILLED;
        chraitarget[cnt] = cnt;
      }
      if (teamleader[chrteam[cnt]] == character)
      {
        chralert[cnt] |= ALERTIFLEADERKILLED;
      }
    }
    cnt++;
  }
  if (teamleader[chrteam[character]] == character)
  {
    teamleader[chrteam[character]] = NOLEADER;
  }
  chron[character] = bfalse;
  chralive[character] = bfalse;
  chrinpack[character] = bfalse;
}

//--------------------------------------------------------------------------------------------
void free_inventory(Uint16 character)
{
  // ZZ> This function frees every item in the character's inventory
  int cnt, next;

  cnt = chrnextinpack[character];
  while (cnt < MAXCHR)
  {
    next = chrnextinpack[cnt];
    free_one_character(cnt);
    cnt = next;
  }
}

//--------------------------------------------------------------------------------------------
void attach_particle_to_character(Uint16 particle, Uint16 character, int grip)
{
  // ZZ> This function sets one particle's position to be attached to a character.
  //     It will kill the particle if the character is no longer around
  Uint16 vertex, model, frame, lastframe;
  Uint8 lip;
  float pointx;
  float pointy;
  float pointz;
  int temp;


  // Check validity of attachment
  if (!chron[character] || chrinpack[character])
  {
    prtgopoof[character] = btrue;
    return;
  }


  // Do we have a matrix???
  if (chrmatrixvalid[character])//meshinrenderlist[chronwhichfan[character]])
  {
    // Transform the weapon grip from model to world space
    model = chrmodel[character];
    frame = chrframe[character];
    lastframe = chrlastframe[character];
    lip = chrlip[character] >> 6;
    if (grip == SPAWNORIGIN)
    {
      prtxpos[particle] = chrmatrix[character]_CNV(3, 0);
      prtypos[particle] = chrmatrix[character]_CNV(3, 1);
      prtzpos[particle] = chrmatrix[character]_CNV(3, 2);
      return;
    }
    vertex = madvertices[model] - grip;


    // Calculate grip point locations with linear interpolation and other silly things
    switch (lip)
    {
    case 0:  // 25% this frame
      temp = madvrtx[lastframe][vertex];
      temp = temp + temp + temp + madvrtx[frame][vertex] >> 2;
      pointx = temp;///chrscale[cnt];
      temp = madvrty[lastframe][vertex];
      temp = temp + temp + temp + madvrty[frame][vertex] >> 2;
      pointy = temp;///chrscale[cnt];
      temp = madvrtz[lastframe][vertex];
      temp = temp + temp + temp + madvrtz[frame][vertex] >> 2;
      pointz = temp;///chrscale[cnt];
      break;

    case 1:  // 50% this frame
      pointx = (madvrtx[frame][vertex] + madvrtx[lastframe][vertex] >> 1);///chrscale[cnt];
      pointy = (madvrty[frame][vertex] + madvrty[lastframe][vertex] >> 1);///chrscale[cnt];
      pointz = (madvrtz[frame][vertex] + madvrtz[lastframe][vertex] >> 1);///chrscale[cnt];
      break;

    case 2:  // 75% this frame
      temp = madvrtx[frame][vertex];
      temp = temp + temp + temp + madvrtx[lastframe][vertex] >> 2;
      pointx = temp;///chrscale[cnt];
      temp = madvrty[frame][vertex];
      temp = temp + temp + temp + madvrty[lastframe][vertex] >> 2;
      pointy = temp;///chrscale[cnt];
      temp = madvrtz[frame][vertex];
      temp = temp + temp + temp + madvrtz[lastframe][vertex] >> 2;
      pointz = temp;///chrscale[cnt];
      break;

    case 3:  // 100% this frame...  This is the legible one
      pointx = madvrtx[frame][vertex];///chrscale[cnt];
      pointy = madvrty[frame][vertex];///chrscale[cnt];
      pointz = madvrtz[frame][vertex];///chrscale[cnt];
      break;

    }





    // Do the transform
    prtxpos[particle] = (pointx * chrmatrix[character]_CNV(0, 0) +
      pointy * chrmatrix[character]_CNV(1, 0) +
      pointz * chrmatrix[character]_CNV(2, 0));
    prtypos[particle] = (pointx * chrmatrix[character]_CNV(0, 1) +
      pointy * chrmatrix[character]_CNV(1, 1) +
      pointz * chrmatrix[character]_CNV(2, 1));
    prtzpos[particle] = (pointx * chrmatrix[character]_CNV(0, 2) +
      pointy * chrmatrix[character]_CNV(1, 2) +
      pointz * chrmatrix[character]_CNV(2, 2));
    prtxpos[particle] += chrmatrix[character]_CNV(3, 0);
    prtypos[particle] += chrmatrix[character]_CNV(3, 1);
    prtzpos[particle] += chrmatrix[character]_CNV(3, 2);
  }
  else
  {
    // No matrix, so just wing it...
    prtxpos[particle] = chrxpos[character];
    prtypos[particle] = chrypos[character];
    prtzpos[particle] = chrzpos[character];
  }
}

//--------------------------------------------------------------------------------------------
#define POINTS 4

void make_one_weapon_matrix(Uint16 cnt)
{
  // ZZ> This function sets one weapon's matrix, based on who it's attached to

  int tnc;
  Uint16 character, vertex, model, frame, lastframe;
  Uint8 lip;
  float pointx[POINTS];
  float pointy[POINTS];
  float pointz[POINTS];
  float nupointx[POINTS];
  float nupointy[POINTS];
  float nupointz[POINTS];
  int temp;


  // Transform the weapon grip from model to world space
  character = chrattachedto[cnt];
  model = chrmodel[character];
  frame = chrframe[character];
  lastframe = chrlastframe[character];
  lip = chrlip[character] >> 6;
  chrmatrixvalid[cnt] = btrue;


  // Calculate grip point locations with linear interpolation and other silly things
  switch (lip)
  {
  case 0:  // 25% this frame
    vertex = chrweapongrip[cnt][0];
    temp = madvrtx[lastframe][vertex];
    temp = temp + temp + temp + madvrtx[frame][vertex] >> 2;
    pointx[0] = temp;
    temp = madvrty[lastframe][vertex];
    temp = temp + temp + temp + madvrty[frame][vertex] >> 2;
    pointy[0] = temp;
    temp = madvrtz[lastframe][vertex];
    temp = temp + temp + temp + madvrtz[frame][vertex] >> 2;
    pointz[0] = temp;

    vertex = chrweapongrip[cnt][1];
    temp = madvrtx[lastframe][vertex];
    temp = temp + temp + temp + madvrtx[frame][vertex] >> 2;
    pointx[1] = temp;
    temp = madvrty[lastframe][vertex];
    temp = temp + temp + temp + madvrty[frame][vertex] >> 2;
    pointy[1] = temp;
    temp = madvrtz[lastframe][vertex];
    temp = temp + temp + temp + madvrtz[frame][vertex] >> 2;
    pointz[1] = temp;

    vertex = chrweapongrip[cnt][2];
    temp = madvrtx[lastframe][vertex];
    temp = temp + temp + temp + madvrtx[frame][vertex] >> 2;
    pointx[2] = temp;
    temp = madvrty[lastframe][vertex];
    temp = temp + temp + temp + madvrty[frame][vertex] >> 2;
    pointy[2] = temp;
    temp = madvrtz[lastframe][vertex];
    temp = temp + temp + temp + madvrtz[frame][vertex] >> 2;
    pointz[2] = temp;

    vertex = chrweapongrip[cnt][3];
    temp = madvrtx[lastframe][vertex];
    temp = temp + temp + temp + madvrtx[frame][vertex] >> 2;
    pointx[3] = temp;
    temp = madvrty[lastframe][vertex];
    temp = temp + temp + temp + madvrty[frame][vertex] >> 2;
    pointy[3] = temp;
    temp = madvrtz[lastframe][vertex];
    temp = temp + temp + temp + madvrtz[frame][vertex] >> 2;
    pointz[3] = temp;
    break;

  case 1:  // 50% this frame
    vertex = chrweapongrip[cnt][0];
    pointx[0] = (madvrtx[frame][vertex] + madvrtx[lastframe][vertex] >> 1);
    pointy[0] = (madvrty[frame][vertex] + madvrty[lastframe][vertex] >> 1);
    pointz[0] = (madvrtz[frame][vertex] + madvrtz[lastframe][vertex] >> 1);
    vertex = chrweapongrip[cnt][1];
    pointx[1] = (madvrtx[frame][vertex] + madvrtx[lastframe][vertex] >> 1);
    pointy[1] = (madvrty[frame][vertex] + madvrty[lastframe][vertex] >> 1);
    pointz[1] = (madvrtz[frame][vertex] + madvrtz[lastframe][vertex] >> 1);
    vertex = chrweapongrip[cnt][2];
    pointx[2] = (madvrtx[frame][vertex] + madvrtx[lastframe][vertex] >> 1);
    pointy[2] = (madvrty[frame][vertex] + madvrty[lastframe][vertex] >> 1);
    pointz[2] = (madvrtz[frame][vertex] + madvrtz[lastframe][vertex] >> 1);
    vertex = chrweapongrip[cnt][3];
    pointx[3] = (madvrtx[frame][vertex] + madvrtx[lastframe][vertex] >> 1);
    pointy[3] = (madvrty[frame][vertex] + madvrty[lastframe][vertex] >> 1);
    pointz[3] = (madvrtz[frame][vertex] + madvrtz[lastframe][vertex] >> 1);
    break;

  case 2:  // 75% this frame
    vertex = chrweapongrip[cnt][0];
    temp = madvrtx[frame][vertex];
    temp = temp + temp + temp + madvrtx[lastframe][vertex] >> 2;
    pointx[0] = temp;
    temp = madvrty[frame][vertex];
    temp = temp + temp + temp + madvrty[lastframe][vertex] >> 2;
    pointy[0] = temp;
    temp = madvrtz[frame][vertex];
    temp = temp + temp + temp + madvrtz[lastframe][vertex] >> 2;
    pointz[0] = temp;


    vertex = chrweapongrip[cnt][1];
    temp = madvrtx[frame][vertex];
    temp = temp + temp + temp + madvrtx[lastframe][vertex] >> 2;
    pointx[1] = temp;
    temp = madvrty[frame][vertex];
    temp = temp + temp + temp + madvrty[lastframe][vertex] >> 2;
    pointy[1] = temp;
    temp = madvrtz[frame][vertex];
    temp = temp + temp + temp + madvrtz[lastframe][vertex] >> 2;
    pointz[1] = temp;


    vertex = chrweapongrip[cnt][2];
    temp = madvrtx[frame][vertex];
    temp = temp + temp + temp + madvrtx[lastframe][vertex] >> 2;
    pointx[2] = temp;
    temp = madvrty[frame][vertex];
    temp = temp + temp + temp + madvrty[lastframe][vertex] >> 2;
    pointy[2] = temp;
    temp = madvrtz[frame][vertex];
    temp = temp + temp + temp + madvrtz[lastframe][vertex] >> 2;
    pointz[2] = temp;


    vertex = chrweapongrip[cnt][3];
    temp = madvrtx[frame][vertex];
    temp = temp + temp + temp + madvrtx[lastframe][vertex] >> 2;
    pointx[3] = temp;
    temp = madvrty[frame][vertex];
    temp = temp + temp + temp + madvrty[lastframe][vertex] >> 2;
    pointy[3] = temp;
    temp = madvrtz[frame][vertex];
    temp = temp + temp + temp + madvrtz[lastframe][vertex] >> 2;
    pointz[3] = temp;

    break;

  case 3:  // 100% this frame...  This is the legible one
    vertex = chrweapongrip[cnt][0];
    pointx[0] = madvrtx[frame][vertex];
    pointy[0] = madvrty[frame][vertex];
    pointz[0] = madvrtz[frame][vertex];
    vertex = chrweapongrip[cnt][1];
    pointx[1] = madvrtx[frame][vertex];
    pointy[1] = madvrty[frame][vertex];
    pointz[1] = madvrtz[frame][vertex];
    vertex = chrweapongrip[cnt][2];
    pointx[2] = madvrtx[frame][vertex];
    pointy[2] = madvrty[frame][vertex];
    pointz[2] = madvrtz[frame][vertex];
    vertex = chrweapongrip[cnt][3];
    pointx[3] = madvrtx[frame][vertex];
    pointy[3] = madvrty[frame][vertex];
    pointz[3] = madvrtz[frame][vertex];
    break;

  }





  tnc = 0;
  while (tnc < POINTS)
  {
    // Do the transform
    nupointx[tnc] = (pointx[tnc] * chrmatrix[character]_CNV(0, 0) +
      pointy[tnc] * chrmatrix[character]_CNV(1, 0) +
      pointz[tnc] * chrmatrix[character]_CNV(2, 0));
    nupointy[tnc] = (pointx[tnc] * chrmatrix[character]_CNV(0, 1) +
      pointy[tnc] * chrmatrix[character]_CNV(1, 1) +
      pointz[tnc] * chrmatrix[character]_CNV(2, 1));
    nupointz[tnc] = (pointx[tnc] * chrmatrix[character]_CNV(0, 2) +
      pointy[tnc] * chrmatrix[character]_CNV(1, 2) +
      pointz[tnc] * chrmatrix[character]_CNV(2, 2));

    nupointx[tnc] += chrmatrix[character]_CNV(3, 0);
    nupointy[tnc] += chrmatrix[character]_CNV(3, 1);
    nupointz[tnc] += chrmatrix[character]_CNV(3, 2);

    tnc++;
  }




  // Calculate weapon's matrix based on positions of grip points
  // chrscale is recomputed at time of attachment
  chrmatrix[cnt] = FourPoints(nupointx[0], nupointy[0], nupointz[0],
    nupointx[1], nupointy[1], nupointz[1],
    nupointx[2], nupointy[2], nupointz[2],
    nupointx[3], nupointy[3], nupointz[3],
    chrscale[cnt]);
}

//--------------------------------------------------------------------------------------------
void make_character_matrices()
{
  // ZZ> This function makes all of the character's matrices
  int cnt, tnc;

  // Forget about old matrices
  cnt = 0;
  while (cnt < MAXCHR)
  {
    chrmatrixvalid[cnt] = bfalse;
    cnt++;
  }


  // Do base characters
  tnc = 0;
  while (tnc < MAXCHR)
  {
    if (chrattachedto[tnc] == MAXCHR && chron[tnc]) // Skip weapons for now
    {
      make_one_character_matrix(tnc);
    }
    tnc++;
  }



  // Do first level of attachments
  tnc = 0;
  while (tnc < MAXCHR)
  {
    if (chrattachedto[tnc] != MAXCHR && chron[tnc])
    {
      if (chrattachedto[chrattachedto[tnc]] == MAXCHR)
      {
        make_one_weapon_matrix(tnc);
      }
    }
    tnc++;
  }


  // Do second level of attachments
  tnc = 0;
  while (tnc < MAXCHR)
  {
    if (chrattachedto[tnc] != MAXCHR && chron[tnc])
    {
      if (chrattachedto[chrattachedto[tnc]] != MAXCHR)
      {
        make_one_weapon_matrix(tnc);
      }
    }
    tnc++;
  }
}

//--------------------------------------------------------------------------------------------
int get_free_character()
{
  // ZZ> This function gets an unused character and returns its index
  Uint16 character;


  if (numfreechr == 0)
  {
    // Return MAXCHR if we can't find one
    return MAXCHR;
  }
  else
  {
    // Just grab the next one
    numfreechr--;
    character = freechrlist[numfreechr];
  }
  return character;
}

//--------------------------------------------------------------------------------------------
bool_t prt_find_target_in_block(int block_x, int block_y, float chrx, float chry, Uint16 facing,
                           bool_t onlyfriends, bool_t anyone, Uint8 team,
                           Uint16 donttarget, Uint16 oldtarget)
{
  // ZZ> This function helps find a target, returning btrue if it found a decent target
  int cnt;
  Uint16 angle;
  Uint16 charb;
  bool_t  enemies, returncode;
  Uint32 fanblock;
  int distance;

  returncode = bfalse;

  if(block_x<0 || block_x>(meshsizex>>2) || block_y<0 || block_y>(meshsizey>>2)) return returncode;

  // Current fanblock
  fanblock = block_x + meshblockstart[block_y];

  enemies = bfalse;
  if (!onlyfriends) enemies = btrue;

  charb = meshbumplistchr[fanblock];
  for (cnt=0; cnt < meshbumplistchrnum[fanblock]; cnt++, charb = chrbumpnext[charb])
  {
    if(!chron[charb] || !chralive[charb] || chrinvictus[charb] || chrinpack[charb] ) continue;

    if (charb!=donttarget && charb!=oldtarget)
    {
      if (anyone || (chrteam[charb] == team && onlyfriends) || (teamhatesteam[team][chrteam[charb]] && enemies))
      {
        distance = ABS(chrxpos[charb] - chrx) + ABS(chrypos[charb] - chry);
        if (distance < globestdistance)
        {
          angle = RAD_TO_TURN(atan2(chrypos[charb] - chry, chrxpos[charb] - chrx));
          angle = facing - angle;
          if (angle < globestangle || angle > (65535 - globestangle))
          {
            returncode = btrue;
            globesttarget = charb;
            globestdistance = distance;
            glouseangle = angle;
            if (angle  > 32767)
              globestangle = -angle;
            else
              globestangle = angle;
          }
        }
      }
    }
  }

  return returncode;
}

//--------------------------------------------------------------------------------------------
Uint16 prt_find_target(float prtx, float prty, Uint16 facing,
                   Uint16 targetangle, bool_t onlyfriends, bool_t anyone,
                   Uint8 team, Uint16 donttarget, Uint16 oldtarget)
{
  // This function finds the best target for the given parameters
  bool_t done, btmp;
  int block_x, block_y;

  block_x = ((int)prtx)>>9;
  block_y = ((int)prty)>>9;
  globestdistance = 9999;
  globestangle = targetangle;
  done = bfalse;

  btmp = prt_find_target_in_block(block_x+0, block_y+0, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  if (done) return globesttarget;

  btmp = prt_find_target_in_block(block_x+1, block_y+0, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x-1, block_y+0, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x+0, block_y+1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x+0, block_y-1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  if (done) return globesttarget;

  btmp = prt_find_target_in_block(block_x+1, block_y+1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x+1, block_y-1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x-1, block_y+1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  btmp = prt_find_target_in_block(block_x-1, block_y-1, prtx, prty, facing, onlyfriends, anyone, team, donttarget, oldtarget);
  done = done || btmp;
  if (done) return globesttarget;


  return MAXCHR;
}

//--------------------------------------------------------------------------------------------
void free_all_characters()
{
  // ZZ> This function resets the character allocation list
  nolocalplayers = btrue;
  numfreechr = 0;
  while (numfreechr < MAXCHR)
  {
    chron[numfreechr] = bfalse;
    chralive[numfreechr] = bfalse;
    chrinpack[numfreechr] = bfalse;
    chrnuminpack[numfreechr] = 0;
    chrnextinpack[numfreechr] = MAXCHR;
    chrstaton[numfreechr] = bfalse;
    chrmatrixvalid[numfreechr] = bfalse;
    freechrlist[numfreechr] = numfreechr;
    numfreechr++;
  }
  numpla = 0;
  numlocalpla = 0;
  numstat = 0;
}

//--------------------------------------------------------------------------------------------
Uint8 __chrhitawall(Uint16 character)
{
  // ZZ> This function returns nonzero if the character hit a wall that the
  //     character is not allowed to cross
  Uint8 passtl=0, passtr=0, passbr=0, passbl=0;
  float x, y, bs;
  Uint32 fan;


  y = chrypos[character];  x = chrxpos[character];  bs = chrbumpsize[character] * 0.5f;

  fan = mesh_get_fan(x-bs, y-bs);
  if(OFFEDGE != fan) passtl = meshfx[ fan ];

  fan = mesh_get_fan(x+bs, y-bs);
  if(OFFEDGE != fan) passtr = meshfx[ fan ];

  fan = mesh_get_fan(x+bs, y+bs);
  if(OFFEDGE != fan) passbr = meshfx[ fan ];

  fan = mesh_get_fan(x-bs, y+bs);
  if(OFFEDGE != fan) passbl = meshfx[ fan ];

  return (passtl | passtr | passbr | passbl) & chrstoppedby[character];
}

//--------------------------------------------------------------------------------------------
void reset_character_accel(Uint16 character)
{
  // ZZ> This function fixes a character's MAX acceleration
  Uint16 enchant;

  if (character != MAXCHR)
  {
    if (chron[character])
    {
      // Okay, remove all acceleration enchants
      enchant = chrfirstenchant[character];
      while (enchant < MAXENCHANT)
      {
        remove_enchant_value(enchant, ADDACCEL);
        enchant = encnextenchant[enchant];
      }
      // Set the starting value
      chrmaxaccel[character] = capmaxaccel[chrmodel[character]][(chrtexture[character] - madskinstart[chrmodel[character]]) % MAXSKIN];
      // Put the acceleration enchants back on
      enchant = chrfirstenchant[character];
      while (enchant < MAXENCHANT)
      {
        add_enchant_value(enchant, ADDACCEL, enceve[enchant]);
        enchant = encnextenchant[enchant];
      }
    }
  }
}

//--------------------------------------------------------------------------------------------
void detach_character_from_mount(Uint16 character, bool_t ignorekurse,
                                 bool_t doshop)
{
  // ZZ> This function drops an item
  Uint16 mount, hand, enchant, cnt, passage, owner=MAXCHR, price;
  bool_t inshop;
  int loc;


  // Make sure the character is valid
  if (character == MAXCHR)
    return;


  // Make sure the character is mounted
  mount = chrattachedto[character];
  if (mount >= MAXCHR)
    return;


  // Make sure both are still around
  if (!chron[character] || !chron[mount])
    return;


  // Don't allow living characters to drop kursed weapons
  if (!ignorekurse && chriskursed[character] && chralive[mount] && chrisitem[character])
  {
    chralert[character] |= ALERTIFNOTDROPPED;
    return;
  }


  // Figure out which hand it's in
  hand = 0;
  if (chrinwhichhand[character] == GRIPRIGHT)
  {
    hand = 1;
  }


  // Rip 'em apart
  chrattachedto[character] = MAXCHR;
  if (chrholdingwhich[mount][0] == character)
    chrholdingwhich[mount][0] = MAXCHR;
  if (chrholdingwhich[mount][1] == character)
    chrholdingwhich[mount][1] = MAXCHR;
  chrscale[character] = chrfat[character] * madscale[chrmodel[character]] * 4;


  // Run the falling animation...
  play_action(character, ACTIONJB + hand, bfalse);



  // Set the positions
  if (chrmatrixvalid[character])
  {
    chrxpos[character] = chrmatrix[character]_CNV(3, 0);
    chrypos[character] = chrmatrix[character]_CNV(3, 1);
    chrzpos[character] = chrmatrix[character]_CNV(3, 2);
  }
  else
  {
    chrxpos[character] = chrxpos[mount];
    chrypos[character] = chrypos[mount];
    chrzpos[character] = chrzpos[mount];
  }



  // Make sure it's not dropped in a wall...
  if(0 != __chrhitawall(character))
  {
    chrxpos[character] = chrxpos[mount];
    chrypos[character] = chrypos[mount];
  }


  // Check for shop passages
  inshop = bfalse;
  if (chrisitem[character] && numshoppassage != 0 && doshop)
  {
    cnt = 0;
    while (cnt < numshoppassage)
    {
      passage = shoppassage[cnt];
      loc = chrxpos[character];
      loc >>= 7;
      if (loc >= passtlx[passage] && loc <= passbrx[passage])
      {
        loc = chrypos[character];
        loc >>= 7;
        if (loc >= passtly[passage] && loc <= passbry[passage])
        {
          inshop = btrue;
          owner = shopowner[passage];
          cnt = numshoppassage;  // Finish loop
          if (owner == NOOWNER)
          {
            // The owner has died!!!
            inshop = bfalse;
          }
        }
      }
      cnt++;
    }
    if (inshop)
    {
      // Give the mount its money back, alert the shop owner
      price = capskincost[chrmodel[character]][(chrtexture[character] - madskinstart[chrmodel[character]]) % MAXSKIN];
      if (capisstackable[chrmodel[character]])
      {
        price *= chrammo[character];
      }
      chrmoney[mount] += price;
      chrmoney[owner] -= price;
      if (chrmoney[owner] < 0)  chrmoney[owner] = 0;
      if (chrmoney[mount] > MAXMONEY)  chrmoney[mount] = MAXMONEY;
      chralert[owner] |= ALERTIFORDERED;
      chrorder[owner] = price;  // Tell owner how much...
      chrcounter[owner] = 0;  // 0 for buying an item
    }
  }

  // Make sure it works right
  chrhitready[character] = btrue;
  chralert[character]   |= ALERTIFDROPPED;
  if (inshop)
  {
    // Drop straight down to avoid theft
    chrxvel[character] = 0;
    chryvel[character] = 0;
  }
  else
  {
    chrxvel[character] = chrxvel[mount];
    chryvel[character] = chryvel[mount];
  }
  chrzvel[character] = DROPZVEL;


  // Turn looping off
  chrloopaction[character] = bfalse;


  // Reset the team if it is a mount
  if (chrismount[mount])
  {
    chrteam[mount] = chrbaseteam[mount];
    chralert[mount] |= ALERTIFDROPPED;
  }
  chrteam[character] = chrbaseteam[character];
  chralert[character] |= ALERTIFDROPPED;


  // Reset transparency
  if (chrisitem[character] && chrtransferblend[mount])
  {
    // Okay, reset transparency
    enchant = chrfirstenchant[character];
    while (enchant < MAXENCHANT)
    {
      unset_enchant_value(enchant, SETALPHABLEND);
      unset_enchant_value(enchant, SETLIGHTBLEND);
      enchant = encnextenchant[enchant];
    }
    chralpha[character] = capalpha[chrmodel[character]];
    chrlight[character] = caplight[chrmodel[character]];
    enchant = chrfirstenchant[character];
    while (enchant < MAXENCHANT)
    {
      set_enchant_value(enchant, SETALPHABLEND, enceve[enchant]);
      set_enchant_value(enchant, SETLIGHTBLEND, enceve[enchant]);
      enchant = encnextenchant[enchant];
    }
  }


  // Set twist
  chrturnmaplr[character] = 32768;
  chrturnmapud[character] = 32768;
}

//--------------------------------------------------------------------------------------------
void attach_character_to_mount(Uint16 character, Uint16 mount,
                               Uint16 grip)
{
  // ZZ> This function attaches one character to another ( the mount )
  //     at either the left or right grip
  int tnc, hand;


  // Make sure both are still around
  if (!chron[character] || !chron[mount] || chrinpack[character] || chrinpack[mount])
    return;

  // Figure out which hand this grip relates to
  hand = 1;
  if (grip == GRIPLEFT)
    hand = 0;


  // Make sure the the hand is valid
  if (!capgripvalid[chrmodel[mount]][hand])
    return;


  // Put 'em together
  chrinwhichhand[character] = grip;
  chrattachedto[character] = mount;
  chrholdingwhich[mount][hand] = character;
  tnc = madvertices[chrmodel[mount]] - grip;
  chrweapongrip[character][0] = tnc;
  chrweapongrip[character][1] = tnc + 1;
  chrweapongrip[character][2] = tnc + 2;
  chrweapongrip[character][3] = tnc + 3;
  chrscale[character] = chrfat[character] / (chrfat[mount] * 1280);
  chrinwater[character] = bfalse;
  chrjumptime[character] = JUMPDELAY * 4;


  // Run the held animation
  if (chrismount[mount] && grip == GRIPONLY)
  {
    // Riding mount
    play_action(character, ACTIONMI, btrue);
    chrloopaction[character] = btrue;
  }
  else
  {
    play_action(character, ACTIONMM + hand, bfalse);
    if (chrisitem[character])
    {
      // Item grab
      chrkeepaction[character] = btrue;
    }
  }




  // Set the team
  if (chrisitem[character])
  {
    chrteam[character] = chrteam[mount];
    // Set the alert
    chralert[character] |= ALERTIFGRABBED;
  }
  if (chrismount[mount])
  {
    chrteam[mount] = chrteam[character];
    // Set the alert
    if (!chrisitem[mount])
    {
      chralert[mount] |= ALERTIFGRABBED;
    }
  }


  // It's not gonna hit the floor
  chrhitready[character] = bfalse;
}

//--------------------------------------------------------------------------------------------
Uint16 stack_in_pack(Uint16 item, Uint16 character)
{
  // ZZ> This function looks in the character's pack for an item similar
  //     to the one given.  If it finds one, it returns the similar item's
  //     index number, otherwise it returns MAXCHR.
  Uint16 inpack, id;
  bool_t allok;


  if (capisstackable[chrmodel[item]])
  {
    inpack = chrnextinpack[character];
    allok = bfalse;
    while (inpack != MAXCHR && !allok)
    {
      allok = btrue;
      if (chrmodel[inpack] != chrmodel[item])
      {
        if (!capisstackable[chrmodel[inpack]])
        {
          allok = bfalse;
        }

        if (chrammomax[inpack] != chrammomax[item])
        {
          allok = bfalse;
        }

        id = 0;
        while (id < MAXIDSZ && allok)
        {
          if (capidsz[chrmodel[inpack]][id] != capidsz[chrmodel[item]][id])
          {
            allok = bfalse;
          }
          id++;
        }
      }
      if (!allok)
      {
        inpack = chrnextinpack[inpack];
      }
    }

    if (allok)
    {
      return inpack;
    }
  }
  return MAXCHR;
}

//--------------------------------------------------------------------------------------------
void add_item_to_character_pack(Uint16 item, Uint16 character)
{
  // ZZ> This function puts one character inside the other's pack
  Uint16 oldfirstitem, newammo, stack;


  // Make sure everything is hunkydori
  if ((!chron[item]) || (!chron[character]) || chrinpack[item] || chrinpack[character] ||
    chrisitem[character])
    return;


  stack = stack_in_pack(item, character);
  if (stack != MAXCHR)
  {
    // We found a similar, stackable item in the pack
    if (chrnameknown[item] || chrnameknown[stack])
    {
      chrnameknown[item] = btrue;
      chrnameknown[stack] = btrue;
    }
    if (capusageknown[chrmodel[item]] || capusageknown[chrmodel[stack]])
    {
      capusageknown[chrmodel[item]] = btrue;
      capusageknown[chrmodel[stack]] = btrue;
    }
    newammo = chrammo[item] + chrammo[stack];
    if (newammo <= chrammomax[stack])
    {
      // All transfered, so kill the in hand item
      chrammo[stack] = newammo;
      if (chrattachedto[item] != MAXCHR)
      {
        detach_character_from_mount(item, btrue, bfalse);
      }
      free_one_character(item);
    }
    else
    {
      // Only some were transfered,
      chrammo[item] += chrammo[stack] - chrammomax[stack];
      chrammo[stack] = chrammomax[stack];
      chralert[character] |= ALERTIFTOOMUCHBAGGAGE;
    }
  }
  else
  {
    // Make sure we have room for another item
    if (chrnuminpack[character] >= MAXNUMINPACK)
    {
      chralert[character] |= ALERTIFTOOMUCHBAGGAGE;
      return;
    }


    // Take the item out of hand
    if (chrattachedto[item] != MAXCHR)
    {
      detach_character_from_mount(item, btrue, bfalse);
      chralert[item] &= (~ALERTIFDROPPED);
    }


    // Remove the item from play
    chrhitready[item] = bfalse;
    chrinpack[item] = btrue;


    // Insert the item into the pack as the first one
    oldfirstitem = chrnextinpack[character];
    chrnextinpack[character] = item;
    chrnextinpack[item] = oldfirstitem;
    chrnuminpack[character]++;
    if (capisequipment[chrmodel[item]])
    {
      // AtLastWaypoint doubles as PutAway
      chralert[item] |= ALERTIFATLASTWAYPOINT;
    }
  }
  return;
}

//--------------------------------------------------------------------------------------------
Uint16 get_item_from_character_pack(Uint16 character, Uint16 grip, bool_t ignorekurse)
{
  // ZZ> This function takes the last item in the character's pack and puts
  //     it into the designated hand.  It returns the item number or MAXCHR.
  Uint16 item, nexttolastitem;


  // Make sure everything is hunkydori
  if ((!chron[character]) || chrinpack[character] || chrisitem[character] || chrnextinpack[character] == MAXCHR)
    return MAXCHR;
  if (chrnuminpack[character] == 0)
    return MAXCHR;


  // Find the last item in the pack
  nexttolastitem = character;
  item = chrnextinpack[character];
  while (chrnextinpack[item] != MAXCHR)
  {
    nexttolastitem = item;
    item = chrnextinpack[item];
  }


  // Figure out what to do with it
  if (chriskursed[item] && chrisequipped[item] && !ignorekurse)
  {
    // Flag the last item as not removed
    chralert[item] |= ALERTIFNOTPUTAWAY;  // Doubles as IfNotTakenOut
    // Cycle it to the front
    chrnextinpack[item] = chrnextinpack[character];
    chrnextinpack[nexttolastitem] = MAXCHR;
    chrnextinpack[character] = item;
    if (character == nexttolastitem)
    {
      chrnextinpack[item] = MAXCHR;
    }
    return MAXCHR;
  }
  else
  {
    // Remove the last item from the pack
    chrinpack[item] = bfalse;
    chrisequipped[item] = bfalse;
    chrnextinpack[nexttolastitem] = MAXCHR;
    chrnuminpack[character]--;
    chrteam[item] = chrteam[character];


    // Attach the item to the character's hand
    attach_character_to_mount(item, character, grip);
    chralert[item] &= (~ALERTIFGRABBED);
    chralert[item] |= (ALERTIFTAKENOUT);
  }
  return item;
}

//--------------------------------------------------------------------------------------------
void drop_keys(Uint16 character)
{
  // ZZ> This function drops all keys ( [KEYA] to [KEYZ] ) that are in a character's
  //     inventory ( Not hands ).
  Uint16 item, lastitem, nextitem, direction, cosdir;
  int testa, testz;


  if (character < MAXCHR)
  {
    if (chron[character])
    {
      if (chrzpos[character] > -2) // Don't lose keys in pits...
      {
        // The IDSZs to find
        testa = MAKE_IDSZ("KEYA");  // [KEYA]
        testz = MAKE_IDSZ("KEYZ");  // [KEYZ]


        lastitem = character;
        item = chrnextinpack[character];
        while (item != MAXCHR)
        {
          nextitem = chrnextinpack[item];
          if (item != character) // Should never happen...
          {
            if ((capidsz[chrmodel[item]][IDSZPARENT] >= (Uint32) testa &&
              capidsz[chrmodel[item]][IDSZPARENT] <= (Uint32) testz) ||
              (capidsz[chrmodel[item]][IDSZTYPE] >= (Uint32) testa &&
              capidsz[chrmodel[item]][IDSZTYPE] <= (Uint32) testz))
            {
              // We found a key...
              chrinpack[item] = bfalse;
              chrisequipped[item] = bfalse;
              chrnextinpack[lastitem] = nextitem;
              chrnextinpack[item] = MAXCHR;
              chrnuminpack[character]--;
              chrattachedto[item] = MAXCHR;
              chrhitready[item] = btrue;
              chralert[item] |= ALERTIFDROPPED;


              direction = RANDIE;
              chrturnleftright[item] = direction + 32768;
              cosdir = direction + 16384;
              chrlevel[item] = chrlevel[character];
              chrxpos[item] = chrxpos[character];
              chrypos[item] = chrypos[character];
              chrzpos[item] = chrzpos[character];
              chrxvel[item] = turntosin[(cosdir>>2) & TRIGTABLE_MASK] * DROPXYVEL;
              chryvel[item] = turntosin[(direction>>2) & TRIGTABLE_MASK] * DROPXYVEL;
              chrzvel[item] = DROPZVEL;
              chrteam[item] = chrbaseteam[item];
            }
            else
            {
              lastitem = item;
            }
          }
          item = nextitem;
        }
      }
    }
  }
}

//--------------------------------------------------------------------------------------------
void drop_all_items(Uint16 character)
{
  // ZZ> This function drops all of a character's items
  Uint16 item, direction, cosdir, diradd;


  if (character < MAXCHR)
  {
    if (chron[character])
    {
      detach_character_from_mount(chrholdingwhich[character][0], btrue, bfalse);
      detach_character_from_mount(chrholdingwhich[character][1], btrue, bfalse);
      if (chrnuminpack[character] > 0)
      {
        direction = chrturnleftright[character] + 32768;
        diradd = 65535 / chrnuminpack[character];
        while (chrnuminpack[character] > 0)
        {
          item = get_item_from_character_pack(character, GRIPLEFT, bfalse);
          if (item < MAXCHR)
          {
            detach_character_from_mount(item, btrue, btrue);
            chrhitready[item] = btrue;
            chralert[item] |= ALERTIFDROPPED;
            chrxpos[item] = chrxpos[character];
            chrypos[item] = chrypos[character];
            chrzpos[item] = chrzpos[character];
            chrlevel[item] = chrlevel[character];
            chrturnleftright[item] = direction + 32768;
            cosdir = direction + 16384;
            chrxvel[item] = turntosin[(cosdir>>2) & TRIGTABLE_MASK] * DROPXYVEL;
            chryvel[item] = turntosin[(direction>>2) & TRIGTABLE_MASK] * DROPXYVEL;
            chrzvel[item] = DROPZVEL;
            chrteam[item] = chrbaseteam[item];
          }
          direction += diradd;
        }
      }
    }
  }
}

//--------------------------------------------------------------------------------------------
void character_grab_stuff(Uint16 chara, int grip, bool_t people)
{
  // ZZ> This function makes the character pick up an item if there's one around
  float xa, ya, za, xb, yb, zb, dist;
  Uint16 charb, hand;
  Uint16 vertex, model, frame, owner=MAXCHR, passage, cnt, price;
  float pointx, pointy, pointz;
  bool_t inshop;
  int loc;


  // Make life easier
  model = chrmodel[chara];
  hand = (grip - 4) >> 2;  // 0 is left, 1 is right


  // Make sure the character doesn't have something already, and that it has hands
  if (chrholdingwhich[chara][hand] != MAXCHR || !capgripvalid[model][hand])
    return;


  // Do we have a matrix???
  if (chrmatrixvalid[chara])//meshinrenderlist[chronwhichfan[chara]])
  {
    // Transform the weapon grip from model to world space
    frame = chrframe[chara];
    vertex = madvertices[model] - grip;


    // Calculate grip point locations
    pointx = madvrtx[frame][vertex];///chrscale[cnt];
    pointy = madvrty[frame][vertex];///chrscale[cnt];
    pointz = madvrtz[frame][vertex];///chrscale[cnt];


    // Do the transform
    xa = (pointx * chrmatrix[chara]_CNV(0, 0) +
      pointy * chrmatrix[chara]_CNV(1, 0) +
      pointz * chrmatrix[chara]_CNV(2, 0));
    ya = (pointx * chrmatrix[chara]_CNV(0, 1) +
      pointy * chrmatrix[chara]_CNV(1, 1) +
      pointz * chrmatrix[chara]_CNV(2, 1));
    za = (pointx * chrmatrix[chara]_CNV(0, 2) +
      pointy * chrmatrix[chara]_CNV(1, 2) +
      pointz * chrmatrix[chara]_CNV(2, 2));
    xa += chrmatrix[chara]_CNV(3, 0);
    ya += chrmatrix[chara]_CNV(3, 1);
    za += chrmatrix[chara]_CNV(3, 2);
  }
  else
  {
    // Just wing it
    xa = chrxpos[chara];
    ya = chrypos[chara];
    za = chrzpos[chara];
  }



  // Go through all characters to find the best match
  charb = 0;
  while (charb < MAXCHR)
  {
    if (chron[charb] && (!chrinpack[charb]) && chrweight[charb] < chrweight[chara] && chralive[charb] && chrattachedto[charb] == MAXCHR && ((!people && chrisitem[charb]) || (people && !chrisitem[charb])))
    {
      xb = chrxpos[charb];
      yb = chrypos[charb];
      zb = chrzpos[charb];
      // First check absolute value diamond
      xb = ABS(xa - xb);
      yb = ABS(ya - yb);
      zb = ABS(za - zb);
      dist = xb + yb;
      if (dist < GRABSIZE && zb < GRABSIZE)
      {
        // Don't grab your mount
        if (chrholdingwhich[charb][0] != chara && chrholdingwhich[charb][1] != chara)
        {
          // Check for shop
          inshop = bfalse;
          if (chrisitem[charb] && numshoppassage != 0)
          {
            cnt = 0;
            while (cnt < numshoppassage)
            {
              passage = shoppassage[cnt];
              loc = chrxpos[charb];
              loc >>= 7;
              if (loc >= passtlx[passage] && loc <= passbrx[passage])
              {
                loc = chrypos[charb];
                loc >>= 7;
                if (loc >= passtly[passage] && loc <= passbry[passage])
                {
                  inshop = btrue;
                  owner = shopowner[passage];
                  cnt = numshoppassage;  // Finish loop
                  if (owner == NOOWNER)
                  {
                    // The owner has died!!!
                    inshop = bfalse;
                  }
                }
              }
              cnt++;
            }
            if (inshop)
            {
              // Pay the shop owner, or don't allow grab...
              if (chrisitem[chara])
              {
                // Pets can shop for free =]
                inshop = bfalse;
              }
              else
              {
                chralert[owner] |= ALERTIFORDERED;
                price = capskincost[chrmodel[charb]][(chrtexture[charb] - madskinstart[chrmodel[charb]]) % MAXSKIN];
                if (capisstackable[chrmodel[charb]])
                {
                  price *= chrammo[charb];
                }
                chrorder[owner] = price;  // Tell owner how much...
                if (chrmoney[chara] >= price)
                {
                  // Okay to buy
                  chrcounter[owner] = 1;  // 1 for selling an item
                  chrmoney[chara] -= price;  // Skin 0 cost is price
                  chrmoney[owner] += price;
                  if (chrmoney[owner] > MAXMONEY)  chrmoney[owner] = MAXMONEY;
                  inshop = bfalse;
                }
                else
                {
                  // Don't allow purchase
                  chrcounter[owner] = 2;  // 2 for "you can't afford that"
                  inshop = btrue;
                }
              }
            }
          }


          if (!inshop)
          {
            // Stick 'em together and quit
            attach_character_to_mount(charb, chara, grip);
            charb = MAXCHR;
            if (people)
            {
              // Do a slam animation...  ( Be sure to drop!!! )
              play_action(chara, ACTIONMC + hand, bfalse);
            }
          }
          else
          {
            // Lift the item a little and quit...
            chrzvel[charb] = DROPZVEL;
            chrhitready[charb] = btrue;
            chralert[charb] |= ALERTIFDROPPED;
            charb = MAXCHR;
          }
        }
      }
    }
    charb++;
  }
}

//--------------------------------------------------------------------------------------------
void character_swipe(Uint16 cnt, Uint8 grip)
{
  // ZZ> This function spawns an attack particle
  int weapon, particle, spawngrip, thrown;
  Uint8 action;
  Uint16 tTmp;
  float dampen;
  float x, y, z, velocity;


  weapon = chrholdingwhich[cnt][grip];
  spawngrip = SPAWNLAST;
  action = chraction[cnt];
  // See if it's an unarmed attack...
  if (weapon == MAXCHR)
  {
    weapon = cnt;
    spawngrip = 4 + (grip << 2);  // 0 = GRIPLEFT, 1 = GRIPRIGHT
  }


  if (weapon != cnt && ((capisstackable[chrmodel[weapon]] && chrammo[weapon] > 1) || (action >= ACTIONFA && action <= ACTIONFD)))
  {
    // Throw the weapon if it's stacked or a hurl animation
    x = chrxpos[cnt];
    y = chrypos[cnt];
    z = chrzpos[cnt];
    thrown = spawn_one_character(x, y, z, chrmodel[weapon], chrteam[cnt], 0, chrturnleftright[cnt], chrname[weapon], MAXCHR);
    if (thrown < MAXCHR)
    {
      chriskursed[thrown] = bfalse;
      chrammo[thrown] = 1;
      chralert[thrown] |= ALERTIFTHROWN;
      velocity = chrstrength[cnt] / (chrweight[thrown] * THROWFIX);
      velocity += MINTHROWVELOCITY;
      if (velocity > MAXTHROWVELOCITY)
      {
        velocity = MAXTHROWVELOCITY;
      }
      tTmp = (0x7FFF + chrturnleftright[cnt]) >> 2;
      chrxvel[thrown] += turntosin[(tTmp+8192+TRIGTABLE_SHIFT) & TRIGTABLE_MASK] * velocity;
      chryvel[thrown] += turntosin[(tTmp+8192) & TRIGTABLE_MASK] * velocity;
      chrzvel[thrown] = DROPZVEL;
      if (chrammo[weapon] <= 1)
      {
        // Poof the item
        detach_character_from_mount(weapon, btrue, bfalse);
        free_one_character(weapon);
      }
      else
      {
        chrammo[weapon]--;
      }
    }
  }
  else
  {
    // Spawn an attack particle
    if (chrammomax[weapon] == 0 || chrammo[weapon] != 0)
    {
      if (chrammo[weapon] > 0 && !capisstackable[chrmodel[weapon]])
      {
        chrammo[weapon]--;  // Ammo usage
      }
      //HERE
      if (capattackprttype[chrmodel[weapon]] != -1)
      {
        particle = spawn_one_particle(chrxpos[weapon], chrypos[weapon], chrzpos[weapon], chrturnleftright[cnt], chrmodel[weapon], capattackprttype[chrmodel[weapon]], weapon, spawngrip, chrteam[cnt], cnt, 0, MAXCHR);
        if (particle != MAXPRT)
        {
          if (!capattackattached[chrmodel[weapon]])
          {
            // Detach the particle
            if (!pipstartontarget[prtpip[particle]] || prttarget[particle] == MAXCHR)
            {
              attach_particle_to_character(particle, weapon, spawngrip);
              // Correct Z spacing base, but nothing else...
              prtzpos[particle] += pipzspacingbase[prtpip[particle]];
            }
            prtattachedtocharacter[particle] = MAXCHR;
            // Don't spawn in walls
            if(0 != __prthitawall(particle))
            {
              prtxpos[particle] = chrxpos[weapon];
              prtypos[particle] = chrypos[weapon];
              if(0 != __prthitawall(particle))
              {
                prtxpos[particle] = chrxpos[cnt];
                prtypos[particle] = chrypos[cnt];
              }
            }
          }
          else
          {
            // Attached particles get a strength bonus for reeling...
            if(pipcauseknockback[prtpip[particle]]) dampen = (REELBASE + (chrstrength[cnt] / REEL))*3; //Extra knockback?
            else dampen = REELBASE + (chrstrength[cnt] / REEL);								//No, do normal
            prtxvel[particle] *= dampen;
            prtyvel[particle] *= dampen;
            prtzvel[particle] *= dampen;
          }

          // Initial particles get a strength bonus, which may be 0.00
          prtdamagebase[particle] += (chrstrength[cnt] * capstrengthdampen[chrmodel[weapon]]);
          // Initial particles get an enchantment bonus
          prtdamagebase[particle] += chrdamageboost[weapon];
          // Initial particles inherit damage type of weapon
          prtdamagetype[particle] = chrdamagetargettype[weapon];
        }
      }
    }
    else
    {
      chrammoknown[weapon] = btrue;
    }
  }
}

//--------------------------------------------------------------------------------------------
void move_characters(float dUpdate)
{
  // ZZ> This function handles character physics
  int cnt;
  Uint8 twist, actionready;
  Uint8 speed, framelip;
  float maxvel, chrvel2, dvx, dvy, dvmax;
  Uint16 action, weapon, mount, item;
  int distance, volume;
  bool_t allowedtoattack, watchtarget, grounded, dojumptimer;

  float horiz_friction, vert_friction;
  float loc_slippyfriction, loc_airfriction, loc_waterfriction, loc_noslipfriction;
  float loc_flydampen, loc_traction;
  float turnfactor;
  float ax,ay,az, vx,vy,vz, px,py,pz;
  float nx=0.0f,ny=0.0f,nz=1.0f;

  //loc_airfriction    = airfriction    / (airfriction    + dUpdate*(1.0f-airfriction   ));
  //loc_waterfriction  = waterfriction  / (waterfriction  + dUpdate*(1.0f-waterfriction ));
  //loc_slippyfriction = slippyfriction / (slippyfriction + dUpdate*(1.0f-slippyfriction));
  //loc_noslipfriction = noslipfriction / (noslipfriction + dUpdate*(1.0f-noslipfriction));

  loc_airfriction    = airfriction;
  loc_waterfriction  = waterfriction;
  loc_slippyfriction = slippyfriction;
  loc_noslipfriction = noslipfriction;
  loc_flydampen      = pow(FLYDAMPEN     , dUpdate);

  // Move every character
  for (cnt = 0; cnt < MAXCHR; cnt++)
  {
    if (!chron[cnt] || chrinpack[cnt]) continue;

    // make characters slide downhill
    if(OFFEDGE!=chronwhichfan[cnt])
    {
      twist = meshtwist[chronwhichfan[cnt]];
    }
    else
    {
      twist = 0x77;
    }

    // the pre-calculated normal to the tile under the character
    nx = normtwistx[twist];
    ny = normtwisty[twist];
    nz = normtwistz[twist];

    // TODO : replace with line(s) below
    turnfactor = 2.0f;
    // scale the turn rate by the dexterity. 
    // For zero dexterity, rate is half speed  
    // For maximum dexterity, rate is 1.5 normal rate
    //turnfactor = (3.0f * (float)chrdexterity[cnt] / (float)PERFECTSTAT + 1.0f) / 2.0f;

    ax = ay = az = 0.0f;
    vx = vy = vz = 0.0f;
    px = py = pz = 0.0f;

    grounded    = bfalse;
    valuegopoof = bfalse;

    // Down that ol' damage timer
    chrdamagetime[cnt] -= dUpdate;
    if(chrdamagetime[cnt]<0) chrdamagetime[cnt]= 0.0f;

    // Character's old location
    chroldx[cnt] = chrxpos[cnt];
    chroldy[cnt] = chrypos[cnt];
    chroldz[cnt] = chrzpos[cnt];
    chroldturn[cnt] = chrturnleftright[cnt];

    // Texture movement
    chruoffset[cnt] += chruoffvel[cnt] * dUpdate;
    chrvoffset[cnt] += chrvoffvel[cnt] * dUpdate;

    // calculate the Character's environment
    {
      float wt;
      float level, air_traction = (chrflyheight[cnt] == 0.0f) ? (1.0-airfriction) : airfriction;

      level = chrlevel[cnt];
      if(MAXCHR != chronwhichplatform[cnt])
      {
        level = MAX(level, chrlevel[chronwhichplatform[cnt]]);
      }

      wt = 0.0f;
      loc_traction   = 0;
      horiz_friction = 0;
      vert_friction  = 0;

      if( chrinwater[cnt] )
      {
        // we are partialy under water
        float buoy, lerp;
        
        if(chrweight[cnt] == 65535 || chrholdingweight[cnt] == 65535)
        {
          buoy = 0.0f;
        }
        else
        {
          float volume, weight;

          weight = chrweight[cnt] + chrholdingweight[cnt];
          volume = (chrbumpheight[cnt]+1) * ( (chrbumpsizebig[cnt]+1)*(chrbumpsizebig[cnt]+1) + (chrbumpsize[cnt]+1)*(chrbumpsize[cnt]+1)) * 0.5;

          // this adjusts the buoyancy so that the default adventurer gets a buoyancy of 0.3
          buoy = 0.3f * (weight/volume) * 1196.0f;
          if(buoy < 0.0f) buoy = 0.0f;
          if(buoy > 1.0f) buoy = 1.0f;
        };

        lerp = (float)(watersurfacelevel - chrzpos[cnt]) / (float)chrbumpheight[cnt];
        if(lerp > 1.0f) lerp = 1.0f;
        if(lerp < 0.0f) lerp = 0.0f;

        loc_traction   += waterfriction * lerp;
        horiz_friction += waterfriction * lerp;
        vert_friction  += waterfriction * lerp;
        az             -= buoy*gravity  * lerp;

        wt += lerp;
      }

      if(chrzpos[cnt] < level + PLATTOLERANCE)
      {
        // we are close to something
        bool_t is_slippy;
        float lerp = (level + PLATTOLERANCE - chrzpos[cnt]) / (float) PLATTOLERANCE;
        if(lerp > 1.0f) lerp = 1.0f;
        if(lerp < 0.0f) lerp = 0.0f;

        if(MAXCHR != chronwhichplatform[cnt])
        {
          is_slippy = bfalse;
        }
        else
        {
          is_slippy = (OFFEDGE!=chronwhichfan[cnt]) && HAS_BITS(meshfx[chronwhichfan[cnt]], MESHFXSLIPPY);
        }

        if( is_slippy )
        {
          loc_traction   += (1.0-slippyfriction)*lerp;
          horiz_friction += slippyfriction * lerp;
        }
        else
        {
          loc_traction   += (1.0-noslipfriction) * lerp;
          horiz_friction += noslipfriction * lerp;
        };
        vert_friction += loc_airfriction * lerp;

        wt += lerp;
      };

      if(wt < 1.0f)
      {
        // we are in clear air
        loc_traction   += (1.0f-wt)*air_traction;
        horiz_friction += (1.0f-wt)*loc_airfriction;
        vert_friction  += (1.0f-wt)*loc_airfriction;
      }
      else
      {
        loc_traction   /= wt;
        horiz_friction /= wt;
        vert_friction  /= wt;
      };

      grounded = (chrzpos[cnt] < chrlevel[cnt] + PLATTOLERANCE / 20.0f);
    }

    // do volontary movement
    if (chralive[cnt])
    {
      // Apply the latches
      if (MAXCHR == chrattachedto[cnt])
      {
        // Character latches for generalized movement
        dvx = chrlatchx[cnt];
        dvy = chrlatchy[cnt];

        // Reverse movements for daze
        if (chrdazetime[cnt] > 0)
        {
          dvx = -dvx;
          dvy = -dvy;
        }

        // Switch x and y for daze
        if (chrgrogtime[cnt] > 0)
        {
          dvmax = dvx;
          dvx = dvy;
          dvy = dvmax;
        }

        // Get direction from the DESIRED change in velocity
        if (chrturnmode[cnt] == TURNMODEWATCH)
        {
          if ((ABS(dvx) > WATCHMIN || ABS(dvy) > WATCHMIN))
          {
            chrturnleftright[cnt] = terp_dir(chrturnleftright[cnt], dvx, dvy, dUpdate * turnfactor);
          }
        }

        if (madframefx[chrframe[cnt]]&MADFXSTOP)
        {
          dvx = 0;
          dvy = 0;
        }

        // TODO : change to line(s) below
        maxvel = chrmaxaccel[cnt] / (1.0 - noslipfriction);
        // set a minimum speed of 6 to fix some stupid slow speeds
        //maxvel = 1.5f * MAX(MAX(3,chrrunspd[cnt]), MAX(chrwalkspd[cnt],chrsneakspd[cnt]));
        chrxtrgvel[cnt] = dvx*maxvel;
        chrytrgvel[cnt] = dvy*maxvel;
        chrztrgvel[cnt] = 0;

        if(chrmaxaccel[cnt] > 0.0f)
        {
          dvx = (chrxtrgvel[cnt]-chrxvel[cnt]);
          dvy = (chrytrgvel[cnt]-chryvel[cnt]);

          // TODO : change to line(s) below
          dvmax = chrmaxaccel[cnt];
          // Limit to max acceleration
          //if(maxvel==0.0)
          //{
          //  dvmax = 2.0f * chrmaxaccel[cnt];
          //}
          //else
          //{
          //  float ftmp;
          //  chrvel2 = chrxvel[cnt]*chrxvel[cnt] + chryvel[cnt]*chryvel[cnt];
          //  ftmp = MIN(1.0 , chrvel2/maxvel/maxvel);
          //  dvmax   = 2.0f * chrmaxaccel[cnt] * (1.0-ftmp);
          //};

          if (dvx < -dvmax) dvx = -dvmax;
          if (dvx >  dvmax) dvx =  dvmax;
          if (dvy < -dvmax) dvy = -dvmax;
          if (dvy >  dvmax) dvy =  dvmax;

          loc_traction *= 11.0f;                    // 11.0f corrects traction so that it gives full traction for non-slip floors in advent.mod
          loc_traction = MIN(1.0, loc_traction);

          ax += dvx * loc_traction * nz;
          ay += dvy * loc_traction * nz;
        };
      }

      // Apply chrlatchx[] and chrlatchy[]
      if (MAXCHR == chrattachedto[cnt])
      {
        // Face the target
        watchtarget = (chrturnmode[cnt] == TURNMODEWATCHTARGET);
        if (watchtarget)
        {
          if (cnt != chraitarget[cnt])
            chrturnleftright[cnt] = terp_dir(chrturnleftright[cnt], chrxpos[chraitarget[cnt]] - chrxpos[cnt], chrypos[chraitarget[cnt]] - chrypos[cnt], dUpdate * turnfactor);
        }

        // Get direction from ACTUAL change in velocity
        if (chrturnmode[cnt] == TURNMODEVELOCITY)
        {
          chrturnleftright[cnt] = terp_dir(chrturnleftright[cnt], chrxtrgvel[cnt], chrytrgvel[cnt], dUpdate * turnfactor);
        }

        // Otherwise make it spin
        else if (chrturnmode[cnt] == TURNMODESPIN)
        {
          chrturnleftright[cnt] += SPINRATE * dUpdate * turnfactor;
        }
      };

      // Character latches for generalized buttons
      if (LATCHBUTTONNONE != chrlatchbutton[cnt])
      {
        if ( HAS_BITS(chrlatchbutton[cnt], LATCHBUTTONJUMP) && chrjumptime[cnt] == 0.0f)
        {
          if (chrattachedto[cnt] != MAXCHR)
          {
            detach_character_from_mount(cnt, btrue, btrue);
            chrjumptime[cnt] = JUMPDELAY;
            vz += (chrflyheight[cnt] == 0) ? DISMOUNTZVEL : DISMOUNTZVELFLY;
            if (chrjumpnumberreset[cnt] != JUMPINFINITE && chrjumpnumber[cnt] > 0)
              chrjumpnumber[cnt]--;

            // Play the jump sound
            play_sound(chrxpos[cnt], chrypos[cnt], capwaveindex[chrmodel[cnt]][capwavejump[chrmodel[cnt]]]);
          }
          else if (chrjumpnumber[cnt] > 0 && (chrjumpready[cnt] || chrjumpnumberreset[cnt] > 1))
          {
            // Make the character jump
            if(chrinwater[cnt] && !grounded)
            {
              vz += WATERJUMP / 3.0f;
              chrjumptime[cnt] = JUMPDELAY / 3.0f;
            }
            else
            {
              vz += chrjump[cnt];
              chrjumptime[cnt] = JUMPDELAY;

              // Set to jump animation if not doing anything better
              if (chractionready[cnt])    play_action(cnt, ACTIONJA, btrue);

              // Play the jump sound (Boing!)
              play_sound(chrxpos[cnt], chrypos[cnt], capwaveindex[chrmodel[cnt]][capwavejump[chrmodel[cnt]]]);
            };

            chrhitready[cnt]  = btrue;
            chrjumpready[cnt] = bfalse;
            if (chrjumpnumberreset[cnt] != JUMPINFINITE) chrjumpnumber[cnt]--;
          }
        }

        if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONALTLEFT) && chractionready[cnt] && chrreloadtime[cnt] == 0)
        {
          chrreloadtime[cnt] = GRABDELAY;
          if (chrholdingwhich[cnt][0] == MAXCHR)
          {
            // Grab left
            play_action(cnt, ACTIONME, bfalse);
          }
          else
          {
            // Drop left
            play_action(cnt, ACTIONMA, bfalse);
          }
        }

        if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONALTRIGHT) && chractionready[cnt] && chrreloadtime[cnt] == 0)
        {
          chrreloadtime[cnt] = GRABDELAY;
          if (chrholdingwhich[cnt][1] == MAXCHR)
          {
            // Grab right
            play_action(cnt, ACTIONMF, bfalse);
          }
          else
          {
            // Drop right
            play_action(cnt, ACTIONMB, bfalse);
          }
        }
        if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONPACKLEFT) && chractionready[cnt] && chrreloadtime[cnt] == 0)
        {
          chrreloadtime[cnt] = PACKDELAY;
          item = chrholdingwhich[cnt][0];
          if (item != MAXCHR)
          {
            if ((chriskursed[item] || capistoobig[chrmodel[item]]) && !capisequipment[chrmodel[item]])
            {
              // The item couldn't be put away
              chralert[item] |= ALERTIFNOTPUTAWAY;
            }
            else
            {
              // Put the item into the pack
              add_item_to_character_pack(item, cnt);
            }
          }
          else
          {
            // Get a new one out and put it in hand
            get_item_from_character_pack(cnt, GRIPLEFT, bfalse);
          }
          // Make it take a little time
          play_action(cnt, ACTIONMG, bfalse);
        }

        if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONPACKRIGHT) && chractionready[cnt] && chrreloadtime[cnt] == 0)
        {
          chrreloadtime[cnt] = PACKDELAY;
          item = chrholdingwhich[cnt][1];
          if (item != MAXCHR)
          {
            if ((chriskursed[item] || capistoobig[chrmodel[item]]) && !capisequipment[chrmodel[item]])
            {
              // The item couldn't be put away
              chralert[item] |= ALERTIFNOTPUTAWAY;
            }
            else
            {
              // Put the item into the pack
              add_item_to_character_pack(item, cnt);
            }
          }
          else
          {
            // Get a new one out and put it in hand
            get_item_from_character_pack(cnt, GRIPRIGHT, bfalse);
          }
          // Make it take a little time
          play_action(cnt, ACTIONMG, bfalse);
        }

        if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONLEFT) && chrreloadtime[cnt] == 0)
        {
          // Which weapon?
          weapon = chrholdingwhich[cnt][0];
          if (weapon == MAXCHR)
          {
            // Unarmed means character itself is the weapon
            weapon = cnt;
          }
          action = capweaponaction[chrmodel[weapon]];


          // Can it do it?
          allowedtoattack = btrue;
          if (!madactionvalid[chrmodel[cnt]][action] || chrreloadtime[weapon] > 0 ||
            (capneedskillidtouse[chrmodel[weapon]] && !check_skills(cnt, capidsz[chrmodel[weapon]][IDSZSKILL])))
          {
            allowedtoattack = bfalse;
            if (chrreloadtime[weapon] == 0)
            {
              // This character can't use this weapon
              chrreloadtime[weapon] = 50;
              if (chrstaton[cnt])
              {
                // Tell the player that they can't use this weapon
                debug_message(1, "%s can't use this item...", chrname[cnt]);
              }
            }
          }

          if (action == ACTIONDA)
          {
            allowedtoattack = bfalse;
            if (chrreloadtime[weapon] == 0)
            {
              chralert[weapon] |= ALERTIFUSED;
            }
          }


          if (allowedtoattack)
          {
            // Rearing mount
            mount = chrattachedto[cnt];
            if (mount != MAXCHR)
            {
              allowedtoattack = capridercanattack[chrmodel[mount]];
              if (chrismount[mount] && chralive[mount] && !chrisplayer[mount] && chractionready[mount])
              {
                if ((action != ACTIONPA || !allowedtoattack) && chractionready[cnt])
                {
                  play_action(chrattachedto[cnt], ACTIONUA + (rand()&1), bfalse);
                  chralert[chrattachedto[cnt]] |= ALERTIFUSED;
                }
                else
                {
                  allowedtoattack = bfalse;
                }
              }
            }


            // Attack button
            if (allowedtoattack)
            {
              if (chractionready[cnt] && madactionvalid[chrmodel[cnt]][action])
              {
                // Check mana cost
                if (chrmana[cnt] >= chrmanacost[weapon] || chrcanchannel[cnt])
                {
                  cost_mana(cnt, chrmanacost[weapon], weapon);
                  // Check life healing
                  chrlife[cnt] += chrlifeheal[weapon];
                  if (chrlife[cnt] > chrlifemax[cnt])  chrlife[cnt] = chrlifemax[cnt];
                  actionready = bfalse;
                  if (action == ACTIONPA)
                    actionready = btrue;
                  action += rand() & 1;
                  play_action(cnt, action, actionready);
                  if (weapon != cnt)
                  {
                    // Make the weapon attack too
                    play_action(weapon, ACTIONMJ, bfalse);
                    chralert[weapon] |= ALERTIFUSED;
                  }
                  else
                  {
                    // Flag for unarmed attack
                    chralert[cnt] |= ALERTIFUSED;
                  }
                }
              }
            }
          }
        }
        else if ( HAS_BITS(chrlatchbutton[cnt],LATCHBUTTONRIGHT) && chrreloadtime[cnt] == 0)
        {
          // Which weapon?
          weapon = chrholdingwhich[cnt][1];
          if (weapon == MAXCHR)
          {
            // Unarmed means character itself is the weapon
            weapon = cnt;
          }
          action = capweaponaction[chrmodel[weapon]] + 2;


          // Can it do it?
          allowedtoattack = btrue;
          if (!madactionvalid[chrmodel[cnt]][action] || chrreloadtime[weapon] > 0 ||
            (capneedskillidtouse[chrmodel[weapon]] && !check_skills(cnt, capidsz[chrmodel[weapon]][IDSZSKILL])))
          {
            allowedtoattack = bfalse;
            if (chrreloadtime[weapon] == 0)
            {
              // This character can't use this weapon
              chrreloadtime[weapon] = 50;
              if (chrstaton[cnt])
              {
                // Tell the player that they can't use this weapon
                debug_message(1, "%s can't use this item...", chrname[cnt]);
              }
            }
          }
          if (action == ACTIONDC)
          {
            allowedtoattack = bfalse;
            if (chrreloadtime[weapon] == 0)
            {
              chralert[weapon] |= ALERTIFUSED;
            }
          }


          if (allowedtoattack)
          {
            // Rearing mount
            mount = chrattachedto[cnt];
            if (mount != MAXCHR)
            {
              allowedtoattack = capridercanattack[chrmodel[mount]];
              if (chrismount[mount] && chralive[mount] && !chrisplayer[mount] && chractionready[mount])
              {
                if ((action != ACTIONPC || !allowedtoattack) && chractionready[cnt])
                {
                  play_action(chrattachedto[cnt], ACTIONUC + (rand()&1), bfalse);
                  chralert[chrattachedto[cnt]] |= ALERTIFUSED;
                }
                else
                {
                  allowedtoattack = bfalse;
                }
              }
            }


            // Attack button
            if (allowedtoattack)
            {
              if (chractionready[cnt] && madactionvalid[chrmodel[cnt]][action])
              {
                // Check mana cost
                if (chrmana[cnt] >= chrmanacost[weapon] || chrcanchannel[cnt])
                {
                  cost_mana(cnt, chrmanacost[weapon], weapon);
                  // Check life healing
                  chrlife[cnt] += chrlifeheal[weapon];
                  if (chrlife[cnt] > chrlifemax[cnt])  chrlife[cnt] = chrlifemax[cnt];
                  actionready = bfalse;
                  if (action == ACTIONPC)
                    actionready = btrue;
                  action += rand() & 1;
                  play_action(cnt, action, actionready);
                  if (weapon != cnt)
                  {
                    // Make the weapon attack too
                    play_action(weapon, ACTIONMJ, bfalse);
                    chralert[weapon] |= ALERTIFUSED;
                  }
                  else
                  {
                    // Flag for unarmed attack
                    chralert[cnt] |= ALERTIFUSED;
                  }
                }
              }
            }
          }
        }
      }
    }



    // Integrate the z direction
    if(0.0f != chrflyheight[cnt])
    {
      float level = chrlevel[cnt];

      az += gravity;
      vz -= chrzvel[cnt] * (1.0f-loc_flydampen);

      if (level < 0) level = 0; // Don't fall in pits...
      pz +=  (level + chrflyheight[cnt] - chrzpos[cnt]) * (1.0-FLYDAMPEN);
    }
    else if(chrzpos[cnt] > chrlevel[cnt] + PLATTOLERANCE)
    {
      az += gravity;
    }
    else
    {
      float lerp_normal, lerp_tang;
      lerp_tang =  (chrlevel[cnt] + PLATTOLERANCE - chrzpos[cnt]) / (float)PLATTOLERANCE;
      if(lerp_tang > 1.0f) lerp_tang = 1.0f;
      if(lerp_tang < 0.0f) lerp_tang = 0.0f;

      // fix to make sure characters will hit the ground softly, but in a reasonable time
      lerp_normal = 1.0 - lerp_tang;
      if(lerp_normal > 1.0f) lerp_normal = 1.0f;
      if(lerp_normal < 0.2f) lerp_normal = 0.2f;

      // slippy hills make characters slide
      if(chrweight[cnt]!=65535 && wateriswater && !chrinwater[cnt] && OFFEDGE != chronwhichfan[cnt] && HAS_BITS(meshfx[chronwhichfan[cnt]], MESHFXSLIPPY))
      {
        ax -= nx * gravity * lerp_tang * hillslide;
        ay -= ny * gravity * lerp_tang * hillslide;
        az += nz * gravity * lerp_normal;
      }
      else
      {
        az += gravity * lerp_normal;
      };
    }

    // Apply friction for next time
    vx -= (1.0f-horiz_friction)*chrxvel[cnt];
    vy -= (1.0f-horiz_friction)*chryvel[cnt];
    vz -= (1.0f-vert_friction )*chrzvel[cnt];

    // Check for collisions with the mesh
    chrzpos[cnt] += chrzvel[cnt]*dUpdate + pz;
    chrzvel[cnt] += az*dUpdate + vz;
    pz = vz = 0;
    if(chrzpos[cnt] < chrlevel[cnt])
    {
      pz += chrlevel[cnt] + 0.001f - chrzpos[cnt];

      if(chrzvel[cnt] < -STOPBOUNCING)
      {
        vz -= chrzvel[cnt] * (1.0f + chrdampen[cnt]*chrdampen[cnt]);
      }
      else if(chrzvel[cnt] < STOPBOUNCING)
      {
        vz -= chrzvel[cnt];
      }

      if(chrhitready[cnt]) 
      {
        chralert[cnt] |= ALERTIFHITGROUND;
        chrhitready[cnt] = bfalse;
      };
    }
    chrzpos[cnt] += pz;
    chrzvel[cnt] += vz;

    // check wall collision in x
    chrxpos[cnt] += chrxvel[cnt]*dUpdate + px;
    chrxvel[cnt] += ax*dUpdate + vx;
    px = vx = 0;
    if(0 != __chrhitawall(cnt)) 
    { 
      px += chroldx[cnt]-chrxpos[cnt]; 
      vx -= chrxvel[cnt]*(1.0f+chrdampen[cnt]); 
    }
    chrxpos[cnt] += px;
    chrxvel[cnt] += vx;

    // check wall collision in x
    chrypos[cnt] += chryvel[cnt]*dUpdate + py;
    chryvel[cnt] += ay*dUpdate + vy;
    py = vy = 0;
    if(0 != __chrhitawall(cnt)) 
    { 
      py += chroldy[cnt]-chrypos[cnt]; 
      vy -= chryvel[cnt]*(1.0f+chrdampen[cnt]); 
    }
    chrypos[cnt] += py;
    chryvel[cnt] += vy;

    // reset the jump
    chrjumpready[cnt]  = grounded || chrinwater[cnt];
    if(chrjumptime[cnt]==0.0f)
    {
      if(grounded && chrjumpnumber[cnt]<chrjumpnumberreset[cnt])
      {
        chrjumpnumber[cnt] = chrjumpnumberreset[cnt];
        chrjumptime[cnt]   = JUMPDELAY;
      }
      else if (chrinwater[cnt] && chrjumpnumber[cnt]<1)
      {
        // "Swimming"
        chrjumpready[cnt]  = btrue;
        chrjumptime[cnt]   = JUMPDELAY / 3.0f;
        chrjumpnumber[cnt] = 1;
      }
    };

    // check to see if it can jump
    dojumptimer = btrue;
    if(grounded)
    {
      // only slippy, non-flat surfaces don't allow jumps
      if(OFFEDGE!=chronwhichfan[cnt] && HAS_BITS(meshfx[chronwhichfan[cnt]], MESHFXSLIPPY) )
      {
        if(!flattwist[twist])
        {
          chrjumpready[cnt] = bfalse;
          dojumptimer       = bfalse;
        };
      }
    }

    if(dojumptimer)
    {
      chrjumptime[cnt]  -= dUpdate;
      if(chrjumptime[cnt]<0) chrjumptime[cnt] = 0.0f;
    }

    // Characters with sticky butts lie on the surface of the mesh
    if (grounded && (chrstickybutt[cnt] || !chralive[cnt]) )
    {
      chrturnmaplr[cnt] = chrturnmaplr[cnt]*0.9 + maplrtwist[twist]*0.1;
      chrturnmapud[cnt] = chrturnmapud[cnt]*0.9 + mapudtwist[twist]*0.1;
    }

    // Animate the character
    chrflip[cnt] += dUpdate;
    while(chrflip[cnt] > 1.0f)
    {
      chrflip[cnt] -= 1.0f;
      chrlip[cnt] += 64;
      if (chrlip[cnt] == 192)
      {
        // Check frame effects
        if (madframefx[chrframe[cnt]]&MADFXACTLEFT)
          character_swipe(cnt, 0);
        if (madframefx[chrframe[cnt]]&MADFXACTRIGHT)
          character_swipe(cnt, 1);
        if (madframefx[chrframe[cnt]]&MADFXGRABLEFT)
          character_grab_stuff(cnt, GRIPLEFT, bfalse);
        if (madframefx[chrframe[cnt]]&MADFXGRABRIGHT)
          character_grab_stuff(cnt, GRIPRIGHT, bfalse);
        if (madframefx[chrframe[cnt]]&MADFXCHARLEFT)
          character_grab_stuff(cnt, GRIPLEFT, btrue);
        if (madframefx[chrframe[cnt]]&MADFXCHARRIGHT)
          character_grab_stuff(cnt, GRIPRIGHT, btrue);
        if (madframefx[chrframe[cnt]]&MADFXDROPLEFT)
          detach_character_from_mount(chrholdingwhich[cnt][0], bfalse, btrue);
        if (madframefx[chrframe[cnt]]&MADFXDROPRIGHT)
          detach_character_from_mount(chrholdingwhich[cnt][1], bfalse, btrue);
        if (madframefx[chrframe[cnt]]&MADFXPOOF && !chrisplayer[cnt])
          valuegopoof = btrue;
        if (madframefx[chrframe[cnt]]&MADFXFOOTFALL)
        {
          if (capwavefootfall[chrmodel[cnt]] != -1)
          {
            play_sound(chrxpos[cnt], chrypos[cnt], capwaveindex[chrmodel[cnt]][capwavefootfall[chrmodel[cnt]]]);
          }
        }
      }

      if (chrlip[cnt] == 0)
      {
        // Change frames
        chrlastframe[cnt] = chrframe[cnt];
        chrframe[cnt]++;
        if (chrframe[cnt] == madactionend[chrmodel[cnt]][chraction[cnt]])
        {
          // Action finished
          if (chrkeepaction[cnt])
          {
            // Keep the last frame going
            chrframe[cnt] = chrlastframe[cnt];
          }
          else
          {
            if (!chrloopaction[cnt])
            {
              // Go on to the next action
              chraction[cnt] = chrnextaction[cnt];
              chrnextaction[cnt] = ACTIONDA;
            }
            else
            {
              // See if the character is mounted...
              if (chrattachedto[cnt] != MAXCHR)
              {
                chraction[cnt] = ACTIONMI;
              }
            }
            chrframe[cnt] = madactionstart[chrmodel[cnt]][chraction[cnt]];
          }
          chractionready[cnt] = btrue;
        }
      }

    };



    // Do "Be careful!" delay
    chrcarefultime[cnt] -= dUpdate;
    if (chrcarefultime[cnt] <= 0) chrcarefultime[cnt] = 0;


    // Get running, walking, sneaking, or dancing, from speed
    if ( !chrkeepaction[cnt] && !chrloopaction[cnt] )
    {
      framelip = madframelip[chrframe[cnt]];  // 0 - 15...  Way through animation
      if (chractionready[cnt] && chrlip[cnt] == 0 && grounded && chrflyheight[cnt] == 0 && (framelip&7) < 2)
      {
        // Do the motion stuff
        speed = ABS(chrxvel[cnt]) + ABS(chryvel[cnt]);
        if (speed < chrsneakspd[cnt])
        {
          //                        chrnextaction[cnt] = ACTIONDA;
          // Do boredom
          chrboretime[cnt] -= dUpdate;
          if (chrboretime[cnt] <= 0) chrboretime[cnt] = 0;

          if (chrboretime[cnt] <= 0)
          {
            chralert[cnt] |= ALERTIFBORED;
            chrboretime[cnt] = BORETIME;
          }
          else
          {
            // Do standstill
            if (chraction[cnt] > ACTIONDD)
            {
              chraction[cnt] = ACTIONDA;
              chrframe[cnt] = madactionstart[chrmodel[cnt]][chraction[cnt]];
            }
          }
        }
        else
        {
          chrboretime[cnt] = BORETIME;
          if (speed < chrwalkspd[cnt])
          {
            chrnextaction[cnt] = ACTIONWA;
            if (chraction[cnt] != ACTIONWA)
            {
              chrframe[cnt] = madframeliptowalkframe[chrmodel[cnt]][LIPWA][framelip];
              chraction[cnt] = ACTIONWA;
            }
          }
          else
          {
            if (speed < chrrunspd[cnt])
            {
              chrnextaction[cnt] = ACTIONWB;
              if (chraction[cnt] != ACTIONWB)
              {
                chrframe[cnt] = madframeliptowalkframe[chrmodel[cnt]][LIPWB][framelip];
                chraction[cnt] = ACTIONWB;
              }
            }
            else
            {
              chrnextaction[cnt] = ACTIONWC;
              if (chraction[cnt] != ACTIONWC)
              {
                chrframe[cnt] = madframeliptowalkframe[chrmodel[cnt]][LIPWC][framelip];
                chraction[cnt] = ACTIONWC;
              }
            }
          }
        }
      }
    }

    // Do poofing
    if (valuegopoof)
    {
      if (chrattachedto[cnt] != MAXCHR)
        detach_character_from_mount(cnt, btrue, bfalse);
      if (chrholdingwhich[cnt][0] != MAXCHR)
        detach_character_from_mount(chrholdingwhich[cnt][0], btrue, bfalse);
      if (chrholdingwhich[cnt][1] != MAXCHR)
        detach_character_from_mount(chrholdingwhich[cnt][1], btrue, bfalse);
      free_inventory(cnt);
      free_one_character(cnt);
    }
  }


}

//--------------------------------------------------------------------------------------------
void setup_characters(char *modname)
{
  // ZZ> This function sets up character data, loaded from "SPAWN.TXT"
  int currentcharacter = 0, lastcharacter, passage, content, money, level, skin, cnt, tnc, localnumber = 0;
  bool_t stat, ghost;
  Uint8 team, cTmp;
  char *name;
  char itislocal;
  STRING myname, newloadname;
  Uint16 facing, grip, attach;
  Uint32 slot;
  float x, y, z;
  FILE *fileread;


  // Turn all characters off
  free_all_characters();


  // Turn some back on
  snprintf(newloadname, sizeof(newloadname), "%s%s/%s", modname, CData.gamedat_dir, CData.spawn_file);
  fileread = fs_fileOpen(PRI_FAIL, "setup_characters()", newloadname, "r");
  currentcharacter = MAXCHR;
  if (NULL != fileread)
  {
    while (goto_colon_yesno(fileread))
    {
      fscanf(fileread, "%s", myname);
      name = myname;
      if (myname[0] == 'N' && myname[1] == 'O' && myname[2] == 'N' &&
        myname[3] == 'E' && myname[4] == 0)
      {
        // Random name
        name = NULL;
      }
      cnt = 0;
      while (cnt < 256)
      {
        if (myname[cnt] == '_')  myname[cnt] = ' ';
        cnt++;
      }
      fscanf(fileread, "%d", &slot);
      fscanf(fileread, "%f%f%f", &x, &y, &z); x *= 128;  y *= 128;  z *= 128;
      cTmp = get_first_letter(fileread);
      attach = MAXCHR;
      facing = NORTH;
      grip = GRIPONLY;
      if (cTmp == 'S' || cTmp == 's')  facing = SOUTH;
      if (cTmp == 'E' || cTmp == 'e')  facing = EAST;
      if (cTmp == 'W' || cTmp == 'w')  facing = WEST;
      if (cTmp == 'L' || cTmp == 'l')  { attach = currentcharacter; grip = GRIPLEFT;   }
      if (cTmp == 'R' || cTmp == 'r')  { attach = currentcharacter; grip = GRIPRIGHT;  }
      if (cTmp == 'I' || cTmp == 'i')  { attach = currentcharacter; grip = INVENTORY;  }
      fscanf(fileread, "%d%d%d%d%d", &money, &skin, &passage, &content, &level);
      cTmp = get_first_letter(fileread);
      stat = bfalse;
      if (cTmp == 'T' || cTmp == 't') stat = btrue;
      cTmp = get_first_letter(fileread);
      ghost = bfalse;
      if (cTmp == 'T' || cTmp == 't') ghost = btrue;
      team = (get_first_letter(fileread) - 'A') % MAXTEAM;


      // Spawn the character
      {
        lastcharacter = spawn_one_character(x, y, z, slot, team, skin, facing, name, MAXCHR);
        if (lastcharacter < MAXCHR)
        {
          chrmoney[lastcharacter] += money;
          if (chrmoney[lastcharacter] > MAXMONEY)  chrmoney[lastcharacter] = MAXMONEY;
          if (chrmoney[lastcharacter] < 0)  chrmoney[lastcharacter] = 0;
          chraicontent[lastcharacter] = content;
          chrpassage[lastcharacter] = passage;
          if (attach == MAXCHR)
          {
            // Free character
            currentcharacter = lastcharacter;
          }
          else
          {
            // Attached character
            if (grip != INVENTORY)
            {
              // Wielded character
              attach_character_to_mount(lastcharacter, currentcharacter, grip);
              let_character_think(lastcharacter);  // Empty the grabbed messages
            }
            else
            {
              // Inventory character
              add_item_to_character_pack(lastcharacter, currentcharacter);
              chralert[lastcharacter] |= ALERTIFGRABBED;  // Make spellbooks change
              chrattachedto[lastcharacter] = currentcharacter;  // Make grab work
              let_character_think(lastcharacter);  // Empty the grabbed messages
              chrattachedto[lastcharacter] = MAXCHR;  // Fix grab
            }
          }
          // Turn on player input devices
          if (stat)
          {
            if (importamount == 0)
            {
              if (playeramount < 2)
              {
                if (numstat == 0)
                {
                  // Single player module
                  add_player(lastcharacter, numstat, INPUTMOUSE | INPUTKEY | INPUTJOYA | INPUTJOYB);
                }
              }
              else
              {
                if (!CData.networkon)
                {
                  if (playeramount == 2)
                  {
                    // Two player hack
                    if (numstat == 0)
                    {
                      // First player
                      add_player(lastcharacter, numstat, INPUTMOUSE | INPUTKEY | INPUTJOYB);
                    }
                    if (numstat == 1)
                    {
                      // Second player
                      add_player(lastcharacter, numstat, INPUTJOYA);
                    }
                  }
                  else
                  {
                    // Three player hack
                    if (numstat == 0)
                    {
                      // First player
                      add_player(lastcharacter, numstat, INPUTKEY);
                    }
                    if (numstat == 1)
                    {
                      // Second player
                      add_player(lastcharacter, numstat, INPUTJOYA);
                    }
                    if (numstat == 2)
                    {
                      // Third player
                      add_player(lastcharacter, numstat, INPUTJOYB | INPUTMOUSE);
                    }
                  }
                }
                else
                {
                  // One player per machine hack
                  if (localmachine == numstat)
                  {
                    add_player(lastcharacter, numstat, INPUTMOUSE | INPUTKEY | INPUTJOYA | INPUTJOYB);
                  }
                }
              }
            }

            if (numstat < importamount)
            {
              // Multiplayer import module
              itislocal = bfalse;
              tnc = 0;
              while (tnc < numimport)
              {
                if (capimportslot[chrmodel[lastcharacter]] == localslot[tnc])
                {
                  itislocal = btrue;
                  localnumber = tnc;
                  break;
                }
                tnc++;
              }


              if (itislocal)
              {
                // It's a local player
                add_player(lastcharacter, numstat, localcontrol[localnumber]);
              }
              else
              {
                // It's a remote player
                add_player(lastcharacter, numstat, INPUTNONE);
              }
            }
            // Turn on the stat display
            add_stat(lastcharacter);
          }

          // Set the starting level
          if (!chrisplayer[lastcharacter])
          {
            // Let the character gain levels
            level -= 1;
            while (chrexperiencelevel[lastcharacter] < level && chrexperience[lastcharacter] < MAXXP)
            {
              give_experience(lastcharacter, MAXXP, XPDIRECT);
            }
          }
          if (ghost)
          {
            // Make the character a ghost !!!BAD!!!  Can do with enchants
            chralpha[lastcharacter] = 128;
            chrlight[lastcharacter] = 255;
          }
        }
      }
    }
    fs_fileClose(fileread);
  }
  else
  {
    log_error("Error loading module: %s\n", newloadname); //Something went wrong
  }
  clear_messages();


  // Make sure local players are displayed first
  sort_stat();


  // Fix tilting trees problem
  tilt_characters_to_terrain();
}

//--------------------------------------------------------------------------------------------
void set_one_player_latch(Uint16 player)
{
  // ZZ> This function converts input readings to latch settings, so players can
  //     move around
  float newx, newy;
  Uint16 turnsin, turncos, character;
  Uint8 device;
  float dist;
  float inputx, inputy;


  // Check to see if we need to bother
  if (!plavalid[player] || INPUTNONE == pladevice[player]) return;

  // Make life easier
  character = plaindex[player];
  device    = pladevice[player];

  // Clear the player's latch buffers
  plalatchbutton[player] = 0;
  plalatchx[player] *= mousesustain;
  plalatchy[player] *= mousesustain;

  // Mouse routines
  if ( HAS_BITS(device, INPUTMOUSE) && mouseon)
  {
    // Movement
    newx = 0;
    newy = 0;
    if (CData.autoturncamera == 255 || !control_mouse_is_pressed(MOS_CAMERA))  // Don't allow movement in camera control mode
    {
      inputx = 0;
      inputy = 0;
      dist = mousedx*mousedx + mousedy*mousedy;

      if (dist > 0.0)
      {
        dist = sqrt(dist);
        inputx = (float)mousedx / (mousesense + dist);
        inputy = (float)mousedy / (mousesense + dist);
      }
      if (CData.autoturncamera == 255 && control_mouse_is_pressed(MOS_CAMERA) == 0)  inputx = 0;

      turnsin = camturnleftrightone*16383.0f;
      turnsin &= TRIGTABLE_MASK;
      turncos = (turnsin + TRIGTABLE_SHIFT) & TRIGTABLE_MASK;
      newx = ( inputx * turntosin[turncos] + inputy * turntosin[turnsin]);
      newy = (-inputx * turntosin[turnsin] + inputy * turntosin[turncos]);
    }
    
    plalatchx[player] += newx * mousecover * 5;
    plalatchy[player] += newy * mousecover * 5;

    // Read buttons
    if (control_mouse_is_pressed(MOS_JUMP))
    {
      if( (respawnanytime && somelocalpladead) || (alllocalpladead && respawnvalid) && MAXCHR != character && !chralive[character] )
      {
        plalatchbutton[player] |= LATCHBUTTONRESPAWN;
      }
      else
      {
        plalatchbutton[player] |= LATCHBUTTONJUMP;
      }
    };

    if (control_mouse_is_pressed(MOS_LEFT_USE))
      plalatchbutton[player] |= LATCHBUTTONLEFT;

    if (control_mouse_is_pressed(MOS_LEFT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTLEFT;

    if (control_mouse_is_pressed(MOS_LEFT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKLEFT;

    if (control_mouse_is_pressed(MOS_RIGHT_USE))
      plalatchbutton[player] |= LATCHBUTTONRIGHT;

    if (control_mouse_is_pressed(MOS_RIGHT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTRIGHT;

    if (control_mouse_is_pressed(MOS_RIGHT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKRIGHT;
  }


  // Joystick A routines
  if ( HAS_BITS(device,INPUTJOYA) && joyaon)
  {
    // Movement
    newx = 0;
    newy = 0;
    if (CData.autoturncamera == 255 || !control_joya_is_pressed(JOA_CAMERA))
    {
      inputx = joyax;
      inputy = joyay;
      dist = joyax*joyax + joyay*joyay;
      if (dist > 1.0)
      {
        dist = sqrt(dist);
        inputx /= dist;
        inputy /= dist;
      }
      if (CData.autoturncamera == 255 && !control_joya_is_pressed(JOA_CAMERA))  inputx = 0;

      turnsin = camturnleftrightone*16383.0f;
      turnsin &= TRIGTABLE_MASK;
      turncos = (turnsin + TRIGTABLE_SHIFT) & TRIGTABLE_MASK;
      newx = ( inputx * turntosin[turncos] + inputy * turntosin[turnsin]);
      newy = (-inputx * turntosin[turnsin] + inputy * turntosin[turncos]);
    }

    plalatchx[player] += newx * mousecover;
    plalatchy[player] += newy * mousecover;

    // Read buttons
    if (control_joya_is_pressed(JOA_JUMP))
    {
      if( (respawnanytime && somelocalpladead) || (alllocalpladead && respawnvalid) && MAXCHR != character && !chralive[character] )
      {
        plalatchbutton[player] |= LATCHBUTTONRESPAWN;
      }
      else
      {
        plalatchbutton[player] |= LATCHBUTTONJUMP;
      }
    }

    if (control_joya_is_pressed(JOA_LEFT_USE))
      plalatchbutton[player] |= LATCHBUTTONLEFT;

    if (control_joya_is_pressed(JOA_LEFT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTLEFT;

    if (control_joya_is_pressed(JOA_LEFT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKLEFT;

    if (control_joya_is_pressed(JOA_RIGHT_USE))
      plalatchbutton[player] |= LATCHBUTTONRIGHT;

    if (control_joya_is_pressed(JOA_RIGHT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTRIGHT;

    if (control_joya_is_pressed(JOA_RIGHT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKRIGHT;
  }


  // Joystick B routines
  if ( HAS_BITS(device, INPUTJOYB) && joybon)
  {
    // Movement
    newx = 0;
    newy = 0;
    if (CData.autoturncamera == 255 || !control_joyb_is_pressed(JOB_CAMERA))
    {
      inputx = joybx;
      inputy = joyby;
      dist = joybx*joybx + joyby*joyby;
      if (dist > 1.0)
      {
        dist = sqrt(dist);
        inputx = joybx / dist;
        inputy = joyby / dist;
      }
      if (CData.autoturncamera == 255 && !control_joyb_is_pressed(JOB_CAMERA))  inputx = 0;

      turnsin = camturnleftrightone*16383.0f;
      turnsin &= TRIGTABLE_MASK;
      turncos = (turnsin + TRIGTABLE_SHIFT) & TRIGTABLE_MASK;
      newx = ( inputx * turntosin[turncos] + inputy * turntosin[turnsin]);
      newy = (-inputx * turntosin[turnsin] + inputy * turntosin[turncos]);
    }

    plalatchx[player] += newx * mousecover;
    plalatchy[player] += newy * mousecover;

    // Read buttons
    if (control_joyb_is_pressed(JOB_JUMP))
    {
      if( (respawnanytime && somelocalpladead) || (alllocalpladead && respawnvalid) && MAXCHR != character && !chralive[character] )
      {
        plalatchbutton[player] |= LATCHBUTTONRESPAWN;
      }
      else
      {
        plalatchbutton[player] |= LATCHBUTTONJUMP;
      }
    }

    if (control_joyb_is_pressed(JOB_LEFT_USE))
      plalatchbutton[player] |= LATCHBUTTONLEFT;

    if (control_joyb_is_pressed(JOB_LEFT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTLEFT;

    if (control_joyb_is_pressed(JOB_LEFT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKLEFT;

    if (control_joyb_is_pressed(JOB_RIGHT_USE))
      plalatchbutton[player] |= LATCHBUTTONRIGHT;

    if (control_joyb_is_pressed(JOB_RIGHT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTRIGHT;

    if (control_joyb_is_pressed(JOB_RIGHT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKRIGHT;
  }

  // Keyboard routines
  if ( HAS_BITS(device, INPUTKEY) && keyon)
  {
    // Movement
    newx = 0;
    newy = 0;
    inputx = inputy = 0;
    if(control_key_is_pressed(KEY_RIGHT)) inputx += 1;
    if(control_key_is_pressed(KEY_LEFT )) inputx -= 1;
    if(control_key_is_pressed(KEY_DOWN )) inputy += 1;
    if(control_key_is_pressed(KEY_UP   )) inputy -= 1;
    dist = inputx*inputx + inputy*inputy;
    if (dist > 1.0)
    {
      dist = sqrt(dist);
      inputx /= dist;
      inputy /= dist;
    }
    if (CData.autoturncamera == 255 && numlocalpla == 1)  inputx = 0;

    turnsin = camturnleftrightone*16383.0f;
    turnsin &= TRIGTABLE_MASK;
    turncos = (turnsin + TRIGTABLE_SHIFT) & TRIGTABLE_MASK;
    newx = (inputx * turntosin[turncos]  + inputy * turntosin[turnsin]);
    newy = (-inputx * turntosin[turnsin] + inputy * turntosin[turncos]);

    plalatchx[player] += newx * mousecover;
    plalatchy[player] += newy * mousecover;

    // Read buttons
    if (control_key_is_pressed(KEY_JUMP))
    {
      if( (respawnanytime && somelocalpladead) || (alllocalpladead && respawnvalid) && MAXCHR != character && !chralive[character] )
      {
        plalatchbutton[player] |= LATCHBUTTONRESPAWN;
      }
      else
      {
        plalatchbutton[player] |= LATCHBUTTONJUMP;
      }
    }

    if (control_key_is_pressed(KEY_LEFT_USE))
      plalatchbutton[player] |= LATCHBUTTONLEFT;

    if (control_key_is_pressed(KEY_LEFT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTLEFT;

    if (control_key_is_pressed(KEY_LEFT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKLEFT;

    if (control_key_is_pressed(KEY_RIGHT_USE))
      plalatchbutton[player] |= LATCHBUTTONRIGHT;

    if (control_key_is_pressed(KEY_RIGHT_GET))
      plalatchbutton[player] |= LATCHBUTTONALTRIGHT;

    if (control_key_is_pressed(KEY_RIGHT_PACK))
      plalatchbutton[player] |= LATCHBUTTONPACKRIGHT;
  }

  dist = plalatchx[player]*plalatchx[player] + plalatchy[player]*plalatchy[player];
  if (dist > 1)
  {
    dist = sqrt(dist);
    plalatchx[player] /= dist;
    plalatchy[player] /= dist;
  };
}

//--------------------------------------------------------------------------------------------
void set_local_latches(void)
{
  // ZZ> This function emulates AI thinkin' by setting latches from input devices
  int cnt;

  cnt = 0;
  while (cnt < MAXPLAYER)
  {
    set_one_player_latch(cnt);
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void make_onwhichfan(void)
{
  // ZZ> This function figures out which fan characters are on and sets their level
  Uint16 character;
  int ripand, distance;
  //int volume;
  float level;


  // First figure out which fan each character is in
  
  for (character = 0; character < MAXCHR; character++)
  {
    if (!chron[character] || chrinpack[character])
    {
      chronwhichfan[character] = OFFEDGE;
    }
    else
    {
      chronwhichfan[character] = mesh_get_fan(chrxpos[character], chrypos[character]);
    };
  }

  // Get levels every update
  for (character = 0; character < MAXCHR; character++)
  {
    if (!chron[character] || chrinpack[character]) continue;

    if(OFFEDGE == chronwhichfan[character])
    {
      chrinwater[character] = bfalse;
      chrlevel[character]   = 0;
      continue;
    };

    level = get_level(chrxpos[character], chrypos[character], chronwhichfan[character], chrwaterwalk[character]) + RAISE;
    if (chralive[character])
    {
      if ( HAS_BITS(meshfx[chronwhichfan[character]], MESHFXDAMAGE) && chrzpos[character] <= chrlevel[character] + DAMAGERAISE && chrattachedto[character] == MAXCHR)
      {
        if ((chrdamagemodifier[character][damagetiletype]&DAMAGESHIFT) != 3 && !chrinvictus[character]) // 3 means they're pretty well immune
        {
          distance = ABS(camtrackx - chrxpos[character]) + ABS(camtracky - chrypos[character]);
          if (distance < damagetilemindistance)
          {
            damagetilemindistance = distance;
          }
          if (distance < damagetilemindistance + 256)
          {
            damagetilesoundtime = 0;
          }
          if (chrdamagetime[character] == 0)
          {
            damage_character(character, 32768, damagetileamount, 1, damagetiletype, DAMAGETEAM, chrbumplast[character], DAMFXBLOC | DAMFXARMO);
            chrdamagetime[character] = DAMAGETILETIME;
          }
          if (damagetileparttype != -1 && (wldframe&damagetilepartand) == 0)
          {
            spawn_one_particle(chrxpos[character], chrypos[character], chrzpos[character],
              0, MAXMODEL, damagetileparttype, MAXCHR, SPAWNLAST, NULLTEAM, MAXCHR, 0, MAXCHR);
          }
        }
        if (chrreaffirmdamagetype[character] == damagetiletype)
        {
          if ((wldframe&TILEREAFFIRMAND) == 0)
            reaffirm_attached_particles(character);
        }
      }
    }

    if ( chrzpos[character] < watersurfacelevel && HAS_BITS(meshfx[chronwhichfan[character]],MESHFXWATER) )
    {
      if (!chrinwater[character])
      {
        // Splash
        spawn_one_particle(chrxpos[character], chrypos[character], watersurfacelevel + RAISE,
          0, MAXMODEL, PRTPIP_SPLASH, MAXCHR, SPAWNLAST, NULLTEAM, MAXCHR, 0, MAXCHR);

        chrinwater[character] = btrue;
        if (wateriswater)
        {
          chralert[character] |= ALERTIFINWATER;
        }
      }
      else if ( capripple[chrmodel[character]] && 
                chrzpos[character] < watersurfacelevel && 
                chrzpos[character] + chrbumpheight[character] > watersurfacelevel - RIPPLETOLERANCE)
      {
        // Ripples
        ripand = (((int)chrxvel[character]) != 0) | (((int)chryvel[character]) != 0);
        ripand = RIPPLEAND >> ripand;
        if (0 == (wldframe&ripand))
        {
          spawn_one_particle(chrxpos[character], chrypos[character], watersurfacelevel,
            0, MAXMODEL, PRTPIP_RIPPLE, MAXCHR, SPAWNLAST, NULLTEAM, MAXCHR, 0, MAXCHR);
        }
      }
    }
    else
    {
      chrinwater[character] = bfalse;
    }

    chrlevel[character] = level;
  }

}

//--------------------------------------------------------------------------------------------
Uint32 mesh_get_fan(float x, float y)
{
  if(x<0.0f || y<0.0f || x>=meshedgex || y>=meshedgey ) 
    return OFFEDGE;

  return (((int)x) >> 7) + meshfanstart[((int)y) >> 7];
};

 //--------------------------------------------------------------------------------------------
 Uint32 mesh_get_block(float x, float y)
 {
   if(x<0.0f || y<0.0f || x>=meshedgex || y>=meshedgey ) 
     return OFFEDGE;

   return (((int)x) >> 9) + meshblockstart[((int)y) >> 9];
 };

 //--------------------------------------------------------------------------------------------
void attach_to_platform(Uint16 object, Uint16 platform)
{
  if(MAXCHR != chronwhichplatform[object])
  {
    chrholdingweight[chronwhichplatform[object]] -= chrweight[object];
    chronwhichplatform[object] = MAXCHR;
  };

  chronwhichplatform[object] = platform;
  chrholdingweight[platform]   = chrweight[object];
};

//--------------------------------------------------------------------------------------------
void bump_characters(float dUpdate)
{
  // ZZ> This function sets handles characters hitting other characters or particles
  Uint16 character, particle, entry, pip, direction, chara, charb;
  Uint32 fanblock, prtidparent, prtidtype, chridvulnerability, eveidremove;
  Sint8 hide;
  int cnt, tnc, dist, chrinblock, prtinblock, enchant, temp;
  float xa, ya, za, xb, yb, zb;
  float ax, ay, nx, ny, scale;  // For deflection
  Uint16 facing;

  float loc_platkeep, loc_platascend, loc_platstick;

  loc_platascend = pow(PLATASCEND, 1.0/dUpdate);
  loc_platkeep   = 1.0 - loc_platascend;
  loc_platstick  = pow(platstick, 1.0/dUpdate);


  // Clear the lists
  fanblock = 0;
  while (fanblock < numfanblock)
  {
    meshbumplistchrnum[fanblock] = 0;
    meshbumplistprtnum[fanblock] = 0;
    fanblock++;
  }


  // Fill 'em back up
  character = 0;
  while (character < MAXCHR)
  {
    if (chron[character] && (!chrinpack[character]) && (chrattachedto[character] == MAXCHR || chrreaffirmdamagetype[character] != DAMAGENULL))
    {
      hide = caphidestate[chrmodel[character]];
      if (hide == NOHIDE || hide != chraistate[character])
      {
        chrholdingweight[character] = 0;
        chronwhichplatform[character] = MAXCHR;
        fanblock = mesh_get_block(chrxpos[character],chrypos[character]);

        if(OFFEDGE != fanblock)
        {
          // Insert before any other characters on the block
          entry = meshbumplistchr[fanblock];
          chrbumpnext[character] = entry;
          meshbumplistchr[fanblock] = character;
          meshbumplistchrnum[fanblock]++;
        };
      }
    }
    character++;
  }

  particle = 0;
  while (particle < MAXPRT)
  {
    if (prton[particle] && prtbumpsize[particle])
    {
      fanblock = mesh_get_block(prtxpos[particle], prtypos[particle]);

      if(OFFEDGE != fanblock)
      {
        // Insert before any other particles on the block
        entry = meshbumplistprt[fanblock];
        prtbumpnext[particle] = entry;
        meshbumplistprt[fanblock] = particle;
        meshbumplistprtnum[fanblock]++;
      };
    }
    particle++;
  }


  // Check collisions with other characters and bump particles
  // Only check each pair once
  fanblock = 0;
  while (fanblock < numfanblock)
  {
    chara = meshbumplistchr[fanblock];
    chrinblock = meshbumplistchrnum[fanblock];
    prtinblock = meshbumplistprtnum[fanblock];
    cnt = 0;
    while (cnt < chrinblock)
    {
      xa = chrxpos[chara];
      ya = chrypos[chara];
      za = chrzpos[chara];
      chridvulnerability = capidsz[chrmodel[chara]][IDSZVULNERABILITY];
      // Don't let items bump
      if (chrbumpheight[chara])//!chrisitem[chara])
      {
        charb = chrbumpnext[chara];  // Don't collide with self
        tnc = cnt + 1;
        while (tnc < chrinblock)
        {
          if (chrbumpheight[charb])//!chrisitem[charb])
          {
            xb = chrxpos[charb];
            yb = chrypos[charb];
            zb = chrzpos[charb];

            // First check absolute value diamond
            xb = ABS(xa - xb);
            yb = ABS(ya - yb);
            dist = xb + yb;
            if (dist < chrbumpsizebig[chara] || dist < chrbumpsizebig[charb])
            {
              // Then check bounding box square...  Square+Diamond=Octagon
              if ((xb < chrbumpsize[chara] || xb < chrbumpsize[charb]) &&
                (yb < chrbumpsize[chara] || yb < chrbumpsize[charb]))
              {
                // Pretend that they collided
                chrbumplast[chara] = charb;
                chrbumplast[charb] = chara;
                // Now see if either is on top the other like a platform
                if (za > zb + chrbumpheight[charb] - PLATTOLERANCE + chrzvel[chara] - chrzvel[charb] && (capcanuseplatforms[chrmodel[chara]] || za > zb + chrbumpheight[charb]))
                {
                  // Is A falling on B?
                  if (za < zb + chrbumpheight[charb] && chrplatform[charb])//&&chrflyheight[chara]==0)
                  {
                    attach_to_platform(chara, charb);

                    // A is inside, coming from above
                    chrzpos[chara] = (chrzpos[chara]) * loc_platkeep + (chrzpos[charb] + chrbumpheight[charb] + PLATADD) * loc_platascend;
                    chrxvel[chara] = (chrxvel[chara]-chrxvel[charb]) * (1.0-loc_platstick) + chrxvel[charb];
                    chryvel[chara] = (chryvel[chara]-chryvel[charb]) * (1.0-loc_platstick) + chryvel[charb];
                    if (chrzvel[chara] < chrzvel[charb]) chrzvel[chara] = chrzvel[charb];
                    chrturnleftright[chara] += (chrturnleftright[charb] - chroldturn[charb]);
                    chrjumpready[chara] = btrue;
                    chrjumpnumber[chara] = chrjumpnumberreset[chara];
                  }
                }
                else
                {
                  if (zb > za + chrbumpheight[chara] - PLATTOLERANCE + chrzvel[charb]*dUpdate - chrzvel[chara]*dUpdate && (capcanuseplatforms[chrmodel[charb]] || zb > za + chrbumpheight[chara]))
                  {
                    // Is B falling on A?
                    if (zb < za + chrbumpheight[chara] && chrplatform[chara])//&&chrflyheight[charb]==0)
                    {
                      attach_to_platform(charb, chara);

                      // B is inside, coming from above
                      chrzpos[charb] = (chrzpos[charb]) * loc_platkeep + (chrzpos[chara] + chrbumpheight[chara] + PLATADD) * loc_platascend;
                      chrxvel[charb] = (chrxvel[charb]-chrxvel[chara]) * (1.0-loc_platstick) + chrxvel[chara];
                      chryvel[charb] = (chryvel[charb]-chryvel[chara]) * (1.0-loc_platstick) + chryvel[chara];
                      if (chrzvel[charb] < chrzvel[chara]) chrzvel[charb] = chrzvel[chara];
                      chrturnleftright[charb] += (chrturnleftright[chara] - chroldturn[chara]);
                      chrjumpready[charb] = btrue;
                    }
                  }
                  else
                  {
                    // They are inside each other, which ain't good
                    // Only collide if moving toward the other
                    if (chrxvel[chara] > 0)
                    {
                      if (chrxpos[chara] < chrxpos[charb]) { chrxvel[charb] += chrxvel[chara] * chrbumpdampen[charb];  chrxvel[chara] = -chrxvel[chara] * chrbumpdampen[chara];  chrxpos[chara] = chroldx[chara]; }
                    }
                    else
                    {
                      if (chrxpos[chara] > chrxpos[charb]) { chrxvel[charb] += chrxvel[chara] * chrbumpdampen[charb];  chrxvel[chara] = -chrxvel[chara] * chrbumpdampen[chara];  chrxpos[chara] = chroldx[chara]; }
                    }
                    if (chryvel[chara] > 0)
                    {
                      if (chrypos[chara] < chrypos[charb]) { chryvel[charb] += chryvel[chara] * chrbumpdampen[charb];  chryvel[chara] = -chryvel[chara] * chrbumpdampen[chara];  chrypos[chara] = chroldy[chara]; }
                    }
                    else
                    {
                      if (chrypos[chara] > chrypos[charb]) { chryvel[charb] += chryvel[chara] * chrbumpdampen[charb];  chryvel[chara] = -chryvel[chara] * chrbumpdampen[chara];  chrypos[chara] = chroldy[chara]; }
                    }
                    if (chrxvel[charb] > 0)
                    {
                      if (chrxpos[charb] < chrxpos[chara]) { chrxvel[chara] += chrxvel[charb] * chrbumpdampen[chara];  chrxvel[charb] = -chrxvel[charb] * chrbumpdampen[charb];  chrxpos[charb] = chroldx[charb]; }
                    }
                    else
                    {
                      if (chrxpos[charb] > chrxpos[chara]) { chrxvel[chara] += chrxvel[charb] * chrbumpdampen[chara];  chrxvel[charb] = -chrxvel[charb] * chrbumpdampen[charb];  chrxpos[charb] = chroldx[charb]; }
                    }
                    if (chryvel[charb] > 0)
                    {
                      if (chrypos[charb] < chrypos[chara]) { chryvel[chara] += chryvel[charb] * chrbumpdampen[chara];  chryvel[charb] = -chryvel[charb] * chrbumpdampen[charb];  chrypos[charb] = chroldy[charb]; }
                    }
                    else
                    {
                      if (chrypos[charb] > chrypos[chara]) { chryvel[chara] += chryvel[charb] * chrbumpdampen[chara];  chryvel[charb] = -chryvel[charb] * chrbumpdampen[charb];  chrypos[charb] = chroldy[charb]; }
                    }
                    xa = chrxpos[chara];
                    ya = chrypos[chara];
                    chralert[chara] |= ALERTIFBUMPED;
                    chralert[charb] |= ALERTIFBUMPED;
                  }
                }
              }
            }
          }
          charb = chrbumpnext[charb];
          tnc++;
        }


        // Now double check the last character we bumped into, in case it's a platform
        charb = chrbumplast[chara];
        if (chron[charb] && (!chrinpack[charb]) && charb != chara && chrattachedto[charb] == MAXCHR && chrbumpheight[chara] && chrbumpheight[charb])
        {
          xb = chrxpos[charb];
          yb = chrypos[charb];
          zb = chrzpos[charb];
          // First check absolute value diamond
          xb = ABS(xa - xb);
          yb = ABS(ya - yb);
          dist = xb + yb;
          if (dist < chrbumpsizebig[chara] || dist < chrbumpsizebig[charb])
          {
            // Then check bounding box square...  Square+Diamond=Octagon
            if ((xb < chrbumpsize[chara] || xb < chrbumpsize[charb]) &&
              (yb < chrbumpsize[chara] || yb < chrbumpsize[charb]))
            {
              // Now see if either is on top the other like a platform
              if (za > zb + chrbumpheight[charb] - PLATTOLERANCE + chrzvel[chara]*dUpdate - chrzvel[charb]*dUpdate && (capcanuseplatforms[chrmodel[chara]] || za > zb + chrbumpheight[charb]))
              {
                // Is A falling on B?
                if (za < zb + chrbumpheight[charb] && chrplatform[charb] && chralive[chara])//&&chrflyheight[chara]==0)
                {

                  attach_to_platform(chara, charb);

                  // A is inside, coming from above
                  chrzpos[chara] = (chrzpos[chara]) * loc_platkeep + (chrzpos[charb] + chrbumpheight[charb] + PLATADD) * loc_platascend;
                  chrxvel[chara] = (chrxvel[chara]-chrxvel[charb]) * (1.0-loc_platstick) + chrxvel[charb];
                  chryvel[chara] = (chryvel[chara]-chryvel[charb]) * (1.0-loc_platstick) + chryvel[charb];
                  if (chrzvel[chara] < chrzvel[charb]) chrzvel[chara] = chrzvel[charb];
                  chrturnleftright[chara] += (chrturnleftright[charb] - chroldturn[charb]);
                  chrjumpready[chara] = btrue;
                  chrjumpnumber[chara] = chrjumpnumberreset[chara];
                  if (madactionvalid[chrmodel[chara]][ACTIONMI] && chralive[chara] && chralive[charb] && chrismount[charb] && !chrisitem[chara] && chrholdingwhich[charb][0] == MAXCHR && chrattachedto[chara] == MAXCHR && chrjumptime[chara] == 0 && chrflyheight[chara] == 0)
                  {
                    attach_character_to_mount(chara, charb, GRIPONLY);
                    chrbumplast[chara] = chara;
                    chrbumplast[charb] = charb;
                  }
                }
              }
              else
              {
                if (zb > za + chrbumpheight[chara] - PLATTOLERANCE + chrzvel[charb]*dUpdate - chrzvel[chara]*dUpdate && (capcanuseplatforms[chrmodel[charb]] || zb > za + chrbumpheight[chara]))
                {
                  // Is B falling on A?
                  if (zb < za + chrbumpheight[chara] && chrplatform[chara] && chralive[charb])//&&chrflyheight[charb]==0)
                  {
                    attach_to_platform(charb, chara);

                    // B is inside, coming from above
                    chrzpos[charb] = (chrzpos[charb]) * loc_platkeep + (chrzpos[chara] + chrbumpheight[chara] + PLATADD) * loc_platascend;
                    chrxvel[charb] = (chrxvel[charb]-chrxvel[chara]) * (1.0-loc_platstick) + chrxvel[chara];
                    chryvel[charb] = (chryvel[charb]-chryvel[chara]) * (1.0-loc_platstick) + chryvel[chara];
                    if (chrzvel[charb] < chrzvel[chara]) chrzvel[charb] = chrzvel[chara];
                    chrturnleftright[charb] += (chrturnleftright[chara] - chroldturn[chara]);
                    chrjumpready[charb] = btrue;
                    chrjumpnumber[charb] = chrjumpnumberreset[charb];

                    if (madactionvalid[chrmodel[charb]][ACTIONMI] && chralive[chara] && chralive[charb] && chrismount[chara] && !chrisitem[charb] && chrholdingwhich[chara][0] == MAXCHR && chrattachedto[charb] == MAXCHR && chrjumptime[charb] == 0 && chrflyheight[charb] == 0)
                    {
                      attach_character_to_mount(charb, chara, GRIPONLY);
                      chrbumplast[chara] = chara;
                      chrbumplast[charb] = charb;
                    }

                    chrholdingweight[chara] += chrweight[charb];
                  }
                }
                else
                {
                  // They are inside each other, which ain't good
                  // Only collide if moving toward the other
                  if (chrxvel[chara] > 0)
                  {
                    if (chrxpos[chara] < chrxpos[charb]) { chrxvel[charb] += chrxvel[chara] * chrbumpdampen[charb];  chrxvel[chara] = -chrxvel[chara] * chrbumpdampen[chara];  chrxpos[chara] = chroldx[chara]; }
                  }
                  else
                  {
                    if (chrxpos[chara] > chrxpos[charb]) { chrxvel[charb] += chrxvel[chara] * chrbumpdampen[charb];  chrxvel[chara] = -chrxvel[chara] * chrbumpdampen[chara];  chrxpos[chara] = chroldx[chara]; }
                  }
                  if (chryvel[chara] > 0)
                  {
                    if (chrypos[chara] < chrypos[charb]) { chryvel[charb] += chryvel[chara] * chrbumpdampen[charb];  chryvel[chara] = -chryvel[chara] * chrbumpdampen[chara];  chrypos[chara] = chroldy[chara]; }
                  }
                  else
                  {
                    if (chrypos[chara] > chrypos[charb]) { chryvel[charb] += chryvel[chara] * chrbumpdampen[charb];  chryvel[chara] = -chryvel[chara] * chrbumpdampen[chara];  chrypos[chara] = chroldy[chara]; }
                  }
                  if (chrxvel[charb] > 0)
                  {
                    if (chrxpos[charb] < chrxpos[chara]) { chrxvel[chara] += chrxvel[charb] * chrbumpdampen[chara];  chrxvel[charb] = -chrxvel[charb] * chrbumpdampen[charb];  chrxpos[charb] = chroldx[charb]; }
                  }
                  else
                  {
                    if (chrxpos[charb] > chrxpos[chara]) { chrxvel[chara] += chrxvel[charb] * chrbumpdampen[chara];  chrxvel[charb] = -chrxvel[charb] * chrbumpdampen[charb];  chrxpos[charb] = chroldx[charb]; }
                  }
                  if (chryvel[charb] > 0)
                  {
                    if (chrypos[charb] < chrypos[chara]) { chryvel[chara] += chryvel[charb] * chrbumpdampen[chara];  chryvel[charb] = -chryvel[charb] * chrbumpdampen[charb];  chrypos[charb] = chroldy[charb]; }
                  }
                  else
                  {
                    if (chrypos[charb] > chrypos[chara]) { chryvel[chara] += chryvel[charb] * chrbumpdampen[chara];  chryvel[charb] = -chryvel[charb] * chrbumpdampen[charb];  chrypos[charb] = chroldy[charb]; }
                  }
                  xa = chrxpos[chara];
                  ya = chrypos[chara];
                  chralert[chara] |= ALERTIFBUMPED;
                  chralert[charb] |= ALERTIFBUMPED;
                }
              }
            }
          }
        }
      }

      // Now check collisions with every bump particle in same area
      if (chralive[chara])
      {
        particle = meshbumplistprt[fanblock];
        tnc = 0;
        while (tnc < prtinblock)
        {
          xb = prtxpos[particle];
          yb = prtypos[particle];
          zb = prtzpos[particle];
          // First check absolute value diamond
          xb = ABS(xa - xb);
          yb = ABS(ya - yb);
          dist = xb + yb;
          if (dist < chrbumpsizebig[chara] || dist < prtbumpsizebig[particle])
          {
            // Then check bounding box square...  Square+Diamond=Octagon
            if ((xb < chrbumpsize[chara]  || xb < prtbumpsize[particle]) &&
              (yb < chrbumpsize[chara]  || yb < prtbumpsize[particle]) &&
              (zb > za - prtbumpheight[particle] && zb < za + chrbumpheight[chara] + prtbumpheight[particle]))
            {
              pip = prtpip[particle];
              if (zb > za + chrbumpheight[chara] + prtzvel[particle] && prtzvel[particle] < 0 && chrplatform[chara] && prtattachedtocharacter[particle] == MAXCHR)
              {
                // Particle is falling on A
                prtzpos[particle] = za + chrbumpheight[chara];
                prtzvel[particle] *= - pipdampen[pip]*chrbumpdampen[chara];
                prtxvel[particle] = (prtxvel[particle]-chrxvel[chara]) * (1.0-loc_platstick) + chrxvel[chara];
                prtyvel[particle] = (prtyvel[particle]-chryvel[chara]) * (1.0-loc_platstick) + chryvel[chara];
              }

              // Check reaffirmation of particles
              if (prtattachedtocharacter[particle] != chara)
              {
                if (chrreloadtime[chara] == 0)
                {
                  if (chrreaffirmdamagetype[chara] == prtdamagetype[particle] && chrdamagetime[chara] == 0)
                  {
                    reaffirm_attached_particles(chara);
                  }
                }
              }

              // Check for missile treatment
              if ((chrdamagemodifier[chara][prtdamagetype[particle]]&3) < 2 ||
                chrmissiletreatment[chara] == MISNORMAL ||
                prtattachedtocharacter[particle] != MAXCHR ||
                (prtchr[particle] == chara && !pipfriendlyfire[pip]) ||
                (chrmana[chrmissilehandler[chara]] < (chrmissilecost[chara] << 4) && !chrcanchannel[chrmissilehandler[chara]]))
              {
                if ((teamhatesteam[prtteam[particle]][chrteam[chara]] || (pipfriendlyfire[pip] && ((chara != prtchr[particle] && chara != chrattachedto[prtchr[particle]]) || piponlydamagefriendly[pip]))) && !chrinvictus[chara])
                {
                  spawn_bump_particles(chara, particle); // Catch on fire
                  if ((prtdamagebase[particle] | prtdamagerand[particle]) > 1)
                  {
                    prtidparent = capidsz[prtmodel[particle]][IDSZPARENT];
                    prtidtype = capidsz[prtmodel[particle]][IDSZTYPE];
                    if (chrdamagetime[chara] == 0 && prtattachedtocharacter[particle] != chara && (pipdamfx[pip]&DAMFXARRO) == 0)
                    {
                      // Normal particle damage
                      if (pipallowpush[pip])
                      {
                        chrxvel[chara] *= chrbumpdampen[chara];
                        chryvel[chara] *= chrbumpdampen[chara];
                        chrzvel[chara] *= chrbumpdampen[chara];
                      }
                      direction = RAD_TO_TURN(atan2(prtyvel[particle], prtxvel[particle]));
                      direction = chrturnleftright[chara] - direction + 32768;
                      // Check all enchants to see if they are removed
                      enchant = chrfirstenchant[chara];
                      while (enchant != MAXENCHANT)
                      {
                        eveidremove = everemovedbyidsz[enceve[enchant]];
                        temp = encnextenchant[enchant];
                        if (eveidremove != IDSZNONE && (eveidremove == prtidtype || eveidremove == prtidparent))
                        {
                          remove_enchant(enchant);
                        }
                        enchant = temp;
                      }

                      //Apply intelligence/wisdom bonus damage for particles with the [IDAM] and [WDAM] expansions (Low ability gives penality)
                      if (pipintdamagebonus[pip]) prtdamagebase[particle] += ((chrintelligence[prtchr[particle]] - 3584) * 0.25);  //+1 bonus for every 4 points of intelligence
                      if (pipwisdamagebonus[pip]) prtdamagebase[particle] += ((chrwisdom[prtchr[particle]] - 3584) * 0.25); //and/or wisdom above 14. Below 14 gives -1 instead!

					  //Force Pancake animation?
					  if(pipcausepancake[pip])
					  {
							//TODO
					  }

                      // Damage the character
                      if (chridvulnerability != IDSZNONE && (chridvulnerability == prtidtype || chridvulnerability == prtidparent))
                      {
                        damage_character(chara, direction, prtdamagebase[particle] << 1, prtdamagerand[particle] << 1, prtdamagetype[particle], prtteam[particle], prtchr[particle], pipdamfx[pip]);
                        chralert[chara] |= ALERTIFHITVULNERABLE;
						cost_mana(chara, pipmanadrain[pip]*2, prtchr[particle]);	//Do mana drain too
                      }
                      else
                      {
                        damage_character(chara, direction, prtdamagebase[particle], prtdamagerand[particle], prtdamagetype[particle], prtteam[particle], prtchr[particle], pipdamfx[pip]);
						cost_mana(chara, pipmanadrain[pip], prtchr[particle]);	//Do mana drain too
					  }
                      // Do confuse effects
                      if (0==(madframefx[chrframe[chara]]&MADFXINVICTUS) || pipdamfx[pip]&DAMFXBLOC)
                      {
                        if (pipgrogtime[pip] != 0 && capcanbegrogged[chrmodel[chara]])
                        {
                          chrgrogtime[chara] += pipgrogtime[pip];
                          if (chrgrogtime[chara] < 0)  chrgrogtime[chara] = 32767;
                          chralert[chara] |= ALERTIFGROGGED;
                        }
                        if (pipdazetime[pip] != 0 && capcanbedazed[chrmodel[chara]])
                        {
                          chrdazetime[chara] += pipdazetime[pip];
                          if (chrdazetime[chara] < 0)  chrdazetime[chara] = 32767;
                          chralert[chara] |= ALERTIFDAZED;
                        }
                      }
                      // Notify the attacker of a scored hit
                      if (prtchr[particle] != MAXCHR)
                      {
                        chralert[prtchr[particle]] |= ALERTIFSCOREDAHIT;
                        chrhitlast[prtchr[particle]] = chara;
                      }
                    }
                    if ((wldframe&31) == 0 && prtattachedtocharacter[particle] == chara)
                    {
                      // Attached particle damage ( Burning )
                      if (pipxyvelbase[pip] == 0)
                      {
                        // Make character limp
                        chrxvel[chara] = 0;
                        chryvel[chara] = 0;
                      }
                      damage_character(chara, 32768, prtdamagebase[particle], prtdamagerand[particle], prtdamagetype[particle], prtteam[particle], prtchr[particle], pipdamfx[pip]);
				      cost_mana(chara, pipmanadrain[pip], prtchr[particle]);	//Do mana drain too
 
					}
                  }
                  if (pipendbump[pip])
                  {
                    if (pipbumpmoney[pip])
                    {
                      if (chrcangrabmoney[chara] && chralive[chara] && chrdamagetime[chara] == 0 && chrmoney[chara] != MAXMONEY)
                      {
                        if (chrismount[chara])
                        {
                          // Let mounts collect money for their riders
                          if (chrholdingwhich[chara][0] != MAXCHR)
                          {
                            chrmoney[chrholdingwhich[chara][0]] += pipbumpmoney[pip];
                            if (chrmoney[chrholdingwhich[chara][0]] > MAXMONEY) chrmoney[chrholdingwhich[chara][0]] = MAXMONEY;
                            if (chrmoney[chrholdingwhich[chara][0]] < 0) chrmoney[chrholdingwhich[chara][0]] = 0;
                            prtgopoof[particle] = btrue;
                          }
                        }
                        else
                        {
                          // Normal money collection
                          chrmoney[chara] += pipbumpmoney[pip];
                          if (chrmoney[chara] > MAXMONEY) chrmoney[chara] = MAXMONEY;
                          if (chrmoney[chara] < 0) chrmoney[chara] = 0;
                          prtgopoof[particle] = btrue;
                        }
                      }
                    }
                    else
                    {
                      prtgopoof[particle] = btrue;
                      // Only hit one character, not several
                      prtdamagebase[particle] = 0;
                      prtdamagerand[particle] = 1;
                    }
                  }
                }
              }
              else
              {
                if (prtchr[particle] != chara)
                {
                  cost_mana(chrmissilehandler[chara], (chrmissilecost[chara] << 4), prtchr[particle]);
                  // Treat the missile
                  if (chrmissiletreatment[chara] == MISDEFLECT)
                  {
                    // Use old position to find normal
                    ax = prtxpos[particle] - prtxvel[particle] * dUpdate;
                    ay = prtypos[particle] - prtyvel[particle] * dUpdate;
                    ax = chrxpos[chara] - ax;
                    ay = chrypos[chara] - ay;
                    // Find size of normal
                    scale = ax * ax + ay * ay;
                    if (scale > 0)
                    {
                      // Make the normal a unit normal
                      scale = sqrt(scale);
                      nx = ax / scale;
                      ny = ay / scale;
                      // Deflect the incoming ray off the normal
                      scale = (prtxvel[particle] * nx + prtyvel[particle] * ny) * 2;
                      ax = scale * nx;
                      ay = scale * ny;
                      prtxvel[particle] -= ax;
                      prtyvel[particle] -= ay;
                    }
                  }
                  else
                  {
                    // Reflect it back in the direction it came
                    prtxvel[particle] *= -1.0;
                    prtyvel[particle] *= -1.0;
                  }
                  // Change the owner of the missile
                  if (!piphoming[pip])
                  {
                    prtteam[particle] = chrteam[chara];
                    prtchr[particle] = chara;
                  }
                  // Change the direction of the particle
                  if (piprotatetoface[pip])
                  {
                    // Turn to face new direction
                    facing = RAD_TO_TURN(atan2(prtyvel[particle], prtxvel[particle]));
                    prtfacing[particle] = facing;
                  }
                }
              }
            }
          }
          particle = prtbumpnext[particle];
          tnc++;
        }
      }
      chara = chrbumpnext[chara];
      cnt++;
    }
    fanblock++;
  }
}


//--------------------------------------------------------------------------------------------
void stat_return(float dUpdate)
{
  // ZZ> This function brings mana and life back
  int cnt, owner, target, eve;


  // Do reload time
  cnt = 0;
  while (cnt < MAXCHR)
  {
    chrreloadtime[cnt] -= dUpdate;
    if(chrreloadtime[cnt]<0) chrreloadtime[cnt] = 0;
    cnt++;
  }



  // Do stats
  if (statclock == ONESECOND)
  {
    // Reset the clock
    statclock = 0;


    // Do all the characters
    cnt = 0;
    while (cnt < MAXCHR)
    {
      if (chron[cnt] && (!chrinpack[cnt]) && chralive[cnt])
      {
        chrmana[cnt] += chrmanareturn[cnt] >> MANARETURNSHIFT;
        if (chrmana[cnt] < 0)
          chrmana[cnt] = 0;
        if (chrmana[cnt] > chrmanamax[cnt])
          chrmana[cnt] = chrmanamax[cnt];
        chrlife[cnt] += chrlifereturn[cnt];
        if (chrlife[cnt] < 1)
          chrlife[cnt] = 1;
        if (chrlife[cnt] > chrlifemax[cnt])
          chrlife[cnt] = chrlifemax[cnt];
        if (chrgrogtime[cnt] > 0)
        {
          chrgrogtime[cnt]--;
          if (chrgrogtime[cnt] < 0)
            chrgrogtime[cnt] = 0;
        }
        if (chrdazetime[cnt] > 0)
        {
          chrdazetime[cnt]--;
          if (chrdazetime[cnt] < 0)
            chrdazetime[cnt] = 0;
        }
      }
      cnt++;
    }


    // Run through all the enchants as well
    cnt = 0;
    while (cnt < MAXENCHANT)
    {
      if (encon[cnt])
      {
        if (enctime[cnt] != 0)
        {
          if (enctime[cnt] > 0)
          {
            enctime[cnt]--;
          }
          owner = encowner[cnt];
          target = enctarget[cnt];
          eve = enceve[cnt];


          // Do drains
          if (chralive[owner])
          {
            // Change life
            chrlife[owner] += encownerlife[cnt];
            if (chrlife[owner] < 1)
            {
              chrlife[owner] = 1;
              kill_character(owner, target);
            }
            if (chrlife[owner] > chrlifemax[owner])
            {
              chrlife[owner] = chrlifemax[owner];
            }
            // Change mana
            if (!cost_mana(owner, -encownermana[cnt], target) && eveendifcantpay[eve])
            {
              remove_enchant(cnt);
            }
          }
          else
          {
            if (!evestayifnoowner[eve])
            {
              remove_enchant(cnt);
            }
          }
          if (encon[cnt])
          {
            if (chralive[target])
            {
              // Change life
              chrlife[target] += enctargetlife[cnt];
              if (chrlife[target] < 1)
              {
                chrlife[target] = 1;
                kill_character(target, owner);
              }
              if (chrlife[target] > chrlifemax[target])
              {
                chrlife[target] = chrlifemax[target];
              }
              // Change mana
              if (!cost_mana(target, -enctargetmana[cnt], owner) && eveendifcantpay[eve])
              {
                remove_enchant(cnt);
              }
            }
            else
            {
              remove_enchant(cnt);
            }
          }
        }
        else
        {
          remove_enchant(cnt);
        }
      }
      cnt++;
    }
  }
}

//--------------------------------------------------------------------------------------------
void pit_kill(float dUpdate)
{
  // ZZ> This function kills any character in a deep pit...
  int cnt;

  if (pitskill)
  {
    if (pitclock > 19)
    {
      pitclock = 0;


      // Kill any particles that fell in a pit, if they die in water...
      
      for (cnt = 0; cnt < MAXPRT; cnt++)
      {
        if (!prton[cnt]) continue;

        if (prtzpos[cnt] < PITDEPTH && pipendwater[prtpip[cnt]])
        {
          prtgopoof[cnt] = btrue;
        }
      }



      // Kill any characters that fell in a pit...
      cnt = 0;
      while (cnt < MAXCHR)
      {
        if (chron[cnt] && chralive[cnt] && !chrinpack[cnt])
        {
          if (!chrinvictus[cnt] && chrzpos[cnt] < PITDEPTH && chrattachedto[cnt] == MAXCHR)
          {
            // Got one!
            kill_character(cnt, MAXCHR);
            chrxvel[cnt] = 0;
            chryvel[cnt] = 0;
          }
        }
        cnt++;
      }
    }
    else
    {
      pitclock += dUpdate;
    }
  }
}

//--------------------------------------------------------------------------------------------
void reset_players()
{
  // ZZ> This function clears the player list data
  int cnt;

  // Reset the local data stuff
  localseekurse = bfalse;
  localseeinvisible = bfalse;
  somelocalpladead = bfalse;
  alllocalpladead = bfalse;

  // Reset the initial player data and latches
  cnt = 0;
  while (cnt < MAXPLAYER)
  {
    plavalid[cnt] = bfalse;
    plaindex[cnt] = 0;
    plalatchx[cnt] = 0;
    plalatchy[cnt] = 0;
    plalatchbutton[cnt] = 0;
    pladevice[cnt] = INPUTNONE;
    cnt++;
  }
  numpla = 0;

  cl_reset(&AClientState);
  sv_reset(&AServerState);
}

//--------------------------------------------------------------------------------------------
void resize_characters(float dUpdate)
{
  // ZZ> This function makes the characters get bigger or smaller, depending
  //     on their sizegoto and sizegototime
  int cnt, item, mount;
  bool_t willgetcaught;
  float newsize;
  float fkeep;

  fkeep = pow(0.9, dUpdate);


  cnt = 0;
  while (cnt < MAXCHR)
  {
    if (chron[cnt] && chrsizegototime[cnt]>0)
    {
      // Make sure it won't get caught in a wall
      willgetcaught = bfalse;
      if (chrsizegoto[cnt] > chrfat[cnt])
      {
        chrbumpsize[cnt] += 10;
        if(0 != __chrhitawall(cnt))
        {
          willgetcaught = btrue;
        }
        chrbumpsize[cnt] -= 10;
      }


      // If it is getting caught, simply halt growth until later
      if (!willgetcaught)
      {
        // Figure out how big it is
        chrsizegototime[cnt] -= dUpdate;
        if(chrsizegototime[cnt] < 0) 
        {
          chrsizegototime[cnt] = 0;
        }

        newsize = chrsizegoto[cnt];
        if (chrsizegototime[cnt] > 0)
        {
          newsize = (chrfat[cnt] * (1.0f-fkeep)) + (newsize * fkeep);
        }
        else if(chrsizegototime[cnt] == 0)
        {
          newsize = chrfat[cnt];
        }


        // Make it that big...
        chrfat[cnt] = newsize;
        chrshadowsize[cnt] = chrshadowsizesave[cnt] * newsize;
        chrbumpsize[cnt] = chrbumpsizesave[cnt] * newsize;
        chrbumpsizebig[cnt] = chrbumpsizebigsave[cnt] * newsize;
        chrbumpheight[cnt] = chrbumpheightsave[cnt] * newsize;
        chrweight[cnt] = capweight[chrmodel[cnt]] * newsize;
        if (capweight[chrmodel[cnt]] == 255) chrweight[cnt] = 65535;


        // Now come up with the magic number
        mount = chrattachedto[cnt];
        if (mount == MAXCHR)
        {
          chrscale[cnt] = newsize * madscale[chrmodel[cnt]] * 4;
        }
        else
        {
          chrscale[cnt] = newsize / (chrfat[mount] * 1280);
        }


        // Make in hand items stay the same size...
        newsize *= 1280;
        item = chrholdingwhich[cnt][0];
        if (item != MAXCHR)
          chrscale[item] = chrfat[item] / newsize;
        item = chrholdingwhich[cnt][1];
        if (item != MAXCHR)
          chrscale[item] = chrfat[item] / newsize;
      }
    }
    cnt++;
  }
}

//--------------------------------------------------------------------------------------------
void export_one_character_name(char *szSaveName, Uint16 character)
{
  // ZZ> This function makes the naming.txt file for the character
  FILE* filewrite;
  int profile;
  char cTmp;
  int cnt, tnc;


  // Can it export?
  profile = chrmodel[character];
  filewrite = fs_fileOpen(PRI_FAIL, "export_one_character_name()", szSaveName, "w");
  if (filewrite)
  {
    cnt = 0;
    cTmp = chrname[character][0];
    cnt++;
    while (cnt < MAXCAPNAMESIZE && cTmp != 0)
    {
      fprintf(filewrite, ":");
      tnc = 0;
      while (tnc < 8 && cTmp != 0)
      {
        if (cTmp == ' ')
        {
          fprintf(filewrite, "_");
        }
        else
        {
          fprintf(filewrite, "%c", cTmp);
        }
        cTmp = chrname[character][cnt];
        tnc++;
        cnt++;
      }
      fprintf(filewrite, "\n");
      fprintf(filewrite, ":STOP\n\n");
    }
    fs_fileClose(filewrite);
  }
  else log_error("Error writing file (%s)\n", szSaveName);
}

//--------------------------------------------------------------------------------------------
void export_one_character_profile(char *szSaveName, Uint16 character)
{
  // ZZ> This function creates a data.txt file for the given character.
  //     it is assumed that all enchantments have been done away with
  FILE* filewrite;
  int profile;
  int damagetype, skin;
  char types[10] = "SCPHEFIZ";
  char codes[4];


  // General stuff
  profile = chrmodel[character];


  // Open the file
  filewrite = fs_fileOpen(PRI_NONE, NULL, szSaveName, "w");
  if (filewrite)
  {
    // Real general data
    fprintf(filewrite, "Slot number    : -1\n");  // -1 signals a flexible load thing
    funderf(filewrite, "Class name     : ", capclassname[profile]);
    ftruthf(filewrite, "Uniform light  : ", capuniformlit[profile]);
    fprintf(filewrite, "Maximum ammo   : %d\n", capammomax[profile]);
    fprintf(filewrite, "Current ammo   : %d\n", chrammo[character]);
    fgendef(filewrite, "Gender         : ", chrgender[character]);
    fprintf(filewrite, "\n");



    // Object stats
    fprintf(filewrite, "Life color     : %d\n", chrlifecolor[character]);
    fprintf(filewrite, "Mana color     : %d\n", chrmanacolor[character]);
    fprintf(filewrite, "Life           : %4.2f\n", chrlifemax[character] / 256.0);
    fpairof(filewrite, "Life up        : ", caplifeperlevelbase[profile], caplifeperlevelrand[profile]);
    fprintf(filewrite, "Mana           : %4.2f\n", chrmanamax[character] / 256.0);
    fpairof(filewrite, "Mana up        : ", capmanaperlevelbase[profile], capmanaperlevelrand[profile]);
    fprintf(filewrite, "Mana return    : %4.2f\n", chrmanareturn[character] / 256.0);
    fpairof(filewrite, "Mana return up : ", capmanareturnperlevelbase[profile], capmanareturnperlevelrand[profile]);
    fprintf(filewrite, "Mana flow      : %4.2f\n", chrmanaflow[character] / 256.0);
    fpairof(filewrite, "Mana flow up   : ", capmanaflowperlevelbase[profile], capmanaflowperlevelrand[profile]);
    fprintf(filewrite, "STR            : %4.2f\n", chrstrength[character] / 256.0);
    fpairof(filewrite, "STR up         : ", capstrengthperlevelbase[profile], capstrengthperlevelrand[profile]);
    fprintf(filewrite, "WIS            : %4.2f\n", chrwisdom[character] / 256.0);
    fpairof(filewrite, "WIS up         : ", capwisdomperlevelbase[profile], capwisdomperlevelrand[profile]);
    fprintf(filewrite, "INT            : %4.2f\n", chrintelligence[character] / 256.0);
    fpairof(filewrite, "INT up         : ", capintelligenceperlevelbase[profile], capintelligenceperlevelrand[profile]);
    fprintf(filewrite, "DEX            : %4.2f\n", chrdexterity[character] / 256.0);
    fpairof(filewrite, "DEX up         : ", capdexterityperlevelbase[profile], capdexterityperlevelrand[profile]);
    fprintf(filewrite, "\n");



    // More physical attributes
    fprintf(filewrite, "Size           : %4.2f\n", chrsizegoto[character]);
    fprintf(filewrite, "Size up        : %4.2f\n", capsizeperlevel[profile]);
    fprintf(filewrite, "Shadow size    : %d\n", capshadowsize[profile]);
    fprintf(filewrite, "Bump size      : %d\n", capbumpsize[profile]);
    fprintf(filewrite, "Bump height    : %d\n", capbumpheight[profile]);
    fprintf(filewrite, "Bump dampen    : %4.2f\n", capbumpdampen[profile]);
    fprintf(filewrite, "Weight         : %d\n", capweight[profile]);
    fprintf(filewrite, "Jump power     : %4.2f\n", capjump[profile]);
    fprintf(filewrite, "Jump number    : %d\n", capjumpnumber[profile]);
    fprintf(filewrite, "Sneak speed    : %d\n", capsneakspd[profile]);
    fprintf(filewrite, "Walk speed     : %d\n", capwalkspd[profile]);
    fprintf(filewrite, "Run speed      : %d\n", caprunspd[profile]);
    fprintf(filewrite, "Fly to height  : %d\n", capflyheight[profile]);
    fprintf(filewrite, "Flashing AND   : %d\n", capflashand[profile]);
    fprintf(filewrite, "Alpha blending : %d\n", capalpha[profile]);
    fprintf(filewrite, "Light blending : %d\n", caplight[profile]);
    ftruthf(filewrite, "Transfer blend : ", captransferblend[profile]);
    fprintf(filewrite, "Sheen          : %d\n", capsheen[profile]);
    ftruthf(filewrite, "Phong mapping  : ", capenviro[profile]);
    fprintf(filewrite, "Texture X add  : %4.2f\n", capuoffvel[profile] / 65535.0);
    fprintf(filewrite, "Texture Y add  : %4.2f\n", capvoffvel[profile] / 65535.0);
    ftruthf(filewrite, "Sticky butt    : ", capstickybutt[profile]);
    fprintf(filewrite, "\n");



    // Invulnerability data
    ftruthf(filewrite, "Invictus       : ", capinvictus[profile]);
    fprintf(filewrite, "NonI facing    : %d\n", capnframefacing[profile]);
    fprintf(filewrite, "NonI angle     : %d\n", capnframeangle[profile]);
    fprintf(filewrite, "I facing       : %d\n", capiframefacing[profile]);
    fprintf(filewrite, "I angle        : %d\n", capiframeangle[profile]);
    fprintf(filewrite, "\n");



    // Skin defenses
    fprintf(filewrite, "Base defense   : ");
    for (skin = 0; skin < MAXSKIN; skin++) { fprintf(filewrite, "%3d ", 255 - capdefense[profile][skin]); }
    fprintf(filewrite, "\n");

    for (damagetype = 0; damagetype < MAXDAMAGETYPE; damagetype++)
    {
      fprintf(filewrite, "%c damage shift :", types[damagetype]);
      for (skin = 0; skin < MAXSKIN; skin++) { fprintf(filewrite, "%3d ", capdamagemodifier[profile][damagetype][skin]&DAMAGESHIFT); };
      fprintf(filewrite, "\n");
    }

    for (damagetype = 0; damagetype < MAXDAMAGETYPE; damagetype++)
    {
      fprintf(filewrite, "%c damage code  : ", types[damagetype]);
      for (skin = 0; skin < MAXSKIN; skin++)
      {
        codes[skin] = 'F';
        if (capdamagemodifier[profile][damagetype][skin]&DAMAGECHARGE) codes[skin] = 'C';
        if (capdamagemodifier[profile][damagetype][skin]&DAMAGEINVERT) codes[skin] = 'T';
        fprintf(filewrite, "%3c ", codes[skin]);
      }
      fprintf(filewrite, "\n");
    }

    fprintf(filewrite, "Acceleration   : ");
    for (skin = 0; skin < MAXSKIN; skin++)
    {
      fprintf(filewrite, "%3.0f ", capmaxaccel[profile][skin]*80);
    }
    fprintf(filewrite, "\n");



    // Experience and level data
    fprintf(filewrite, "EXP for 2nd    : %d\n", capexperienceforlevel[profile][1]);
    fprintf(filewrite, "EXP for 3rd    : %d\n", capexperienceforlevel[profile][2]);
    fprintf(filewrite, "EXP for 4th    : %d\n", capexperienceforlevel[profile][3]);
    fprintf(filewrite, "EXP for 5th    : %d\n", capexperienceforlevel[profile][4]);
    fprintf(filewrite, "EXP for 6th    : %d\n", capexperienceforlevel[profile][5]);
    fprintf(filewrite, "Starting EXP   : %d\n", chrexperience[character]);
    fprintf(filewrite, "EXP worth      : %d\n", capexperienceworth[profile]);
    fprintf(filewrite, "EXP exchange   : %5.3f\n", capexperienceexchange[profile]);
    fprintf(filewrite, "EXPSECRET      : %4.2f\n", capexperiencerate[profile][0]);
    fprintf(filewrite, "EXPQUEST       : %4.2f\n", capexperiencerate[profile][1]);
    fprintf(filewrite, "EXPDARE        : %4.2f\n", capexperiencerate[profile][2]);
    fprintf(filewrite, "EXPKILL        : %4.2f\n", capexperiencerate[profile][3]);
    fprintf(filewrite, "EXPMURDER      : %4.2f\n", capexperiencerate[profile][4]);
    fprintf(filewrite, "EXPREVENGE     : %4.2f\n", capexperiencerate[profile][5]);
    fprintf(filewrite, "EXPTEAMWORK    : %4.2f\n", capexperiencerate[profile][6]);
    fprintf(filewrite, "EXPROLEPLAY    : %4.2f\n", capexperiencerate[profile][7]);
    fprintf(filewrite, "\n");



    // IDSZ identification tags
    fprintf(filewrite, "IDSZ Parent    : [%s]\n", undo_idsz(capidsz[profile][0]));
    fprintf(filewrite, "IDSZ Type      : [%s]\n", undo_idsz(capidsz[profile][1]));
    fprintf(filewrite, "IDSZ Skill     : [%s]\n", undo_idsz(capidsz[profile][2]));
    fprintf(filewrite, "IDSZ Special   : [%s]\n", undo_idsz(capidsz[profile][3]));
    fprintf(filewrite, "IDSZ Hate      : [%s]\n", undo_idsz(capidsz[profile][4]));
    fprintf(filewrite, "IDSZ Vulnie    : [%s]\n", undo_idsz(capidsz[profile][5]));
    fprintf(filewrite, "\n");



    // Item and damage flags
    ftruthf(filewrite, "Is an item     : ", capisitem[profile]);
    ftruthf(filewrite, "Is a mount     : ", capismount[profile]);
    ftruthf(filewrite, "Is stackable   : ", capisstackable[profile]);
    ftruthf(filewrite, "Name known     : ", chrnameknown[character]);
    ftruthf(filewrite, "Usage known    : ", capusageknown[profile]);
    ftruthf(filewrite, "Is exportable  : ", capcancarrytonextmodule[profile]);
    ftruthf(filewrite, "Requires skill : ", capneedskillidtouse[profile]);
    ftruthf(filewrite, "Is platform    : ", capplatform[profile]);
    ftruthf(filewrite, "Collects money : ", capcangrabmoney[profile]);
    ftruthf(filewrite, "Can open stuff : ", capcanopenstuff[profile]);
    fprintf(filewrite, "\n");



    // Other item and damage stuff
    fdamagf(filewrite, "Damage type    : ", capdamagetargettype[profile]);
    factiof(filewrite, "Attack type    : ", capweaponaction[profile]);
    fprintf(filewrite, "\n");



    // Particle attachments
    fprintf(filewrite, "Attached parts : %d\n", capattachedprtamount[profile]);
    fdamagf(filewrite, "Reaffirm type  : ", capattachedprtreaffirmdamagetype[profile]);
    fprintf(filewrite, "Particle type  : %d\n", capattachedprttype[profile]);
    fprintf(filewrite, "\n");



    // Character hands
    ftruthf(filewrite, "Left valid     : ", capgripvalid[profile][0]);
    ftruthf(filewrite, "Right valid    : ", capgripvalid[profile][1]);
    fprintf(filewrite, "\n");



    // Particle spawning on attack
    ftruthf(filewrite, "Part on weapon : ", capattackattached[profile]);
    fprintf(filewrite, "Part type      : %d\n", capattackprttype[profile]);
    fprintf(filewrite, "\n");



    // Particle spawning for GoPoof
    fprintf(filewrite, "Poof amount    : %d\n", capgopoofprtamount[profile]);
    fprintf(filewrite, "Facing add     : %d\n", capgopoofprtfacingadd[profile]);
    fprintf(filewrite, "Part type      : %d\n", capgopoofprttype[profile]);
    fprintf(filewrite, "\n");



    // Particle spawning for blud
    ftruthf(filewrite, "Blud valid    : ", capbludvalid[profile]);
    fprintf(filewrite, "Part type      : %d\n", capbludprttype[profile]);
    fprintf(filewrite, "\n");



    // Extra stuff
    ftruthf(filewrite, "Waterwalking   : ", capwaterwalk[profile]);
    fprintf(filewrite, "Bounce dampen  : %5.3f\n", capdampen[profile]);
    fprintf(filewrite, "\n");



    // More stuff
    fprintf(filewrite, "Life healing   : %5.3f\n", caplifeheal[profile] / 256.0);
    fprintf(filewrite, "Mana cost      : %5.3f\n", capmanacost[profile] / 256.0);
    fprintf(filewrite, "Life return    : %d\n", caplifereturn[profile]);
    fprintf(filewrite, "Stopped by     : %d\n", capstoppedby[profile]);

    for (skin = 0; skin < MAXSKIN; skin++)
    {
      STRING stmp;
      snprintf(stmp, sizeof(stmp), "Skin %d name    : ", skin);
      funderf(filewrite, stmp, capskinname[profile][skin]);
    };

    for (skin = 0; skin < MAXSKIN; skin++)
    {
      fprintf(filewrite, "Skin %d cost    : %d\n", skin, capskincost[profile][skin]);
    };

    fprintf(filewrite, "STR dampen     : %5.3f\n", capstrengthdampen[profile]);
    fprintf(filewrite, "\n");



    // Another memory lapse
    ftruthf(filewrite, "No rider attak : ", btrue - capridercanattack[profile]);
    ftruthf(filewrite, "Can be dazed   : ", capcanbedazed[profile]);
    ftruthf(filewrite, "Can be grogged : ", capcanbegrogged[profile]);
    fprintf(filewrite, "NOT USED       : 0\n");
    fprintf(filewrite, "NOT USED       : 0\n");
    ftruthf(filewrite, "Can see invisi : ", capcanseeinvisible[profile]);
    fprintf(filewrite, "Kursed chance  : %d\n", chriskursed[character]*100);
    fprintf(filewrite, "Footfall sound : %d\n", capwavefootfall[profile]);
    fprintf(filewrite, "Jump sound     : %d\n", capwavejump[profile]);
    fprintf(filewrite, "\n");


    // Expansions
    fprintf(filewrite, ":[GOLD] %d\n", chrmoney[character]);

    if (capskindressy[profile]&1) fprintf(filewrite, ":[DRES] 0\n");
    if (capskindressy[profile]&2) fprintf(filewrite, ":[DRES] 1\n");
    if (capskindressy[profile]&4) fprintf(filewrite, ":[DRES] 2\n");
    if (capskindressy[profile]&8) fprintf(filewrite, ":[DRES] 3\n");
    if (capresistbumpspawn[profile]) fprintf(filewrite, ":[STUK] 0\n");
    if (capistoobig[profile]) fprintf(filewrite, ":[PACK] 0\n");
    if (!capreflect[profile]) fprintf(filewrite, ":[VAMP] 1\n");
    if (capalwaysdraw[profile]) fprintf(filewrite, ":[DRAW] 1\n");
    if (capisranged[profile]) fprintf(filewrite, ":[RANG] 1\n");
    if (caphidestate[profile]!=NOHIDE) fprintf(filewrite, ":[HIDE] %d\n", caphidestate[profile]);
    if (capisequipment[profile]) fprintf(filewrite, ":[EQUI] 1\n");
    if (chrbumpsizebig[character]==chrbumpsize[character]*2) fprintf(filewrite, ":[SQUA] 1\n");
    if (chricon[character]!=capusageknown[profile]) fprintf(filewrite, ":[ICON] %d\n", chricon[character]);
    if (capforceshadow[profile]) fprintf(filewrite, ":[SHAD] 1\n");

    //Skill expansions
    if (chrcanseekurse[character])  fprintf(filewrite, ":[CKUR] 1\n");
    if (chrcanusearcane[character]) fprintf(filewrite, ":[WMAG] 1\n");
    if (chrcanjoust[character])     fprintf(filewrite, ":[JOUS] 1\n");
    if (chrcanusedivine[character]) fprintf(filewrite, ":[HMAG] 1\n");
    if (chrcandisarm[character])    fprintf(filewrite, ":[DISA] 1\n");
    if (chrcanusetech[character])   fprintf(filewrite, ":[TECH] 1\n");
    if (chrcanbackstab[character])  fprintf(filewrite, ":[STAB] 1\n");
    if (chrcanuseadvancedweapons[character]) fprintf(filewrite, ":[AWEP] 1\n");
    if (chrcanusepoison[character]) fprintf(filewrite, ":[POIS] 1\n");
    if (chrcanread[character])		fprintf(filewrite, ":[READ] 1\n");

    //General exported character information
    fprintf(filewrite, ":[PLAT] %d\n", capcanuseplatforms[profile]);
    fprintf(filewrite, ":[SKIN] %d\n", (chrtexture[character] - madskinstart[profile]) % MAXSKIN);
    fprintf(filewrite, ":[CONT] %d\n", chraicontent[character]);
    fprintf(filewrite, ":[STAT] %d\n", chraistate[character]);
    fprintf(filewrite, ":[LEVL] %d\n", chrexperiencelevel[character]);
    fs_fileClose(filewrite);
  }
}

//--------------------------------------------------------------------------------------------
void export_one_character_skin(char *szSaveName, Uint16 character)
{
  // ZZ> This function creates a skin.txt file for the given character.
  FILE* filewrite;
  int profile;


  // General stuff
  profile = chrmodel[character];


  // Open the file
  filewrite = fs_fileOpen(PRI_NONE, NULL, szSaveName, "w");
  if (filewrite)
  {
    fprintf(filewrite, "This file is used only by the import menu\n");
    fprintf(filewrite, ": %d\n", (chrtexture[character] - madskinstart[profile]) % MAXSKIN);
    fs_fileClose(filewrite);
  }
}

//--------------------------------------------------------------------------------------------
int load_one_character_profile(char *szLoadName)
{
  // ZZ> This function fills a character profile with data from CData.data_file, returning
  // the object slot that the profile was stuck into.  It may cause the program
  // to abort if bad things happen.
  FILE* fileread;
  int object=MAXMODEL, skin, cnt;
  int iTmp;
  float fTmp;
  char cTmp;
  int damagetype, level, xptype;
  IDSZ idsz;

  // Open the file
  fileread = fs_fileOpen(PRI_FAIL, "load_one_character_profile()", szLoadName, "r");
  //printf(" DIAG: trying to read %s\n",szLoadName);
  if (fileread != NULL)
  {
    globalname = szLoadName;
    // Read in the object slot
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp); object = iTmp;
    if (object < 0)
    {
      if (importobject < 0)
      {
        log_error("Object slot number %i is invalid. (%s) \n", object, szLoadName);
      }
      else
      {
        object = importobject;
      }
    }

    // Read in the real general data
    goto_colon(fileread);  get_name(fileread, capclassname[object]);


    // Make sure we don't load over an existing model
    if (madused[object])
    {
      log_error("Object slot %i is already used. (%s)\n", object, szLoadName);
    }
    madused[object] = btrue;


    // Light cheat
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capuniformlit[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capuniformlit[object] = btrue;

    // Ammo
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capammomax[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capammo[object] = iTmp;
    // Gender
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capgender[object] = GENOTHER;
    if (cTmp == 'F' || cTmp == 'f')  capgender[object] = GENFEMALE;
    if (cTmp == 'M' || cTmp == 'm')  capgender[object] = GENMALE;
    if (cTmp == 'R' || cTmp == 'r')  capgender[object] = GENRANDOM;
    // Read in the object stats
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  caplifecolor[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capmanacolor[object] = iTmp;
    goto_colon(fileread);  read_pair(fileread);
    caplifebase[object] = pairbase;  capliferand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    caplifeperlevelbase[object] = pairbase;  caplifeperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanabase[object] = pairbase;  capmanarand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanaperlevelbase[object] = pairbase;  capmanaperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanareturnbase[object] = pairbase;  capmanareturnrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanareturnperlevelbase[object] = pairbase;  capmanareturnperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanaflowbase[object] = pairbase;  capmanaflowrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capmanaflowperlevelbase[object] = pairbase;  capmanaflowperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capstrengthbase[object] = pairbase;  capstrengthrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capstrengthperlevelbase[object] = pairbase;  capstrengthperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capwisdombase[object] = pairbase;  capwisdomrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capwisdomperlevelbase[object] = pairbase;  capwisdomperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capintelligencebase[object] = pairbase;  capintelligencerand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capintelligenceperlevelbase[object] = pairbase;  capintelligenceperlevelrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capdexteritybase[object] = pairbase;  capdexterityrand[object] = pairrand;
    goto_colon(fileread);  read_pair(fileread);
    capdexterityperlevelbase[object] = pairbase;  capdexterityperlevelrand[object] = pairrand;

    // More physical attributes
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capsize[object] = fTmp;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capsizeperlevel[object] = fTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capshadowsize[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capbumpsize[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capbumpheight[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capbumpdampen[object] = fTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capweight[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capjump[object] = fTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capjumpnumber[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capsneakspd[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capwalkspd[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  caprunspd[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capflyheight[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capflashand[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capalpha[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  caplight[object] = iTmp;
    if (caplight[object] < 0xff)
    {
      capalpha[object] = MIN(capalpha[object], 0xff - caplight[object]);
    };

    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    captransferblend[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  captransferblend[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capsheen[object] = iTmp;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capenviro[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capenviro[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capuoffvel[object] = fTmp * 65535;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capvoffvel[object] = fTmp * 65535;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capstickybutt[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capstickybutt[object] = btrue;


    // Invulnerability data
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capinvictus[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capinvictus[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capnframefacing[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capnframeangle[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capiframefacing[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capiframeangle[object] = iTmp;
    // Resist burning and stuck arrows with nframe angle of 1 or more
    if (capnframeangle[object] > 0)
    {
      if (capnframeangle[object] == 1)
      {
        capnframeangle[object] = 0;
      }
    }


    // Skin defenses ( 4 skins )
    goto_colon(fileread);
    for (skin = 0; skin < MAXSKIN; skin++)
    { fscanf(fileread, "%d", &iTmp);  capdefense[object][skin] = 255 - iTmp; };

    for (damagetype = 0; damagetype < MAXDAMAGETYPE; damagetype++)
    {
      goto_colon(fileread);
      for (skin = 0;skin < MAXSKIN;skin++)
      { fscanf(fileread, "%d", &iTmp);  capdamagemodifier[object][damagetype][skin] = iTmp; };
    }

    for (damagetype = 0; damagetype < MAXDAMAGETYPE; damagetype++)
    {
      goto_colon(fileread);
      for (skin = 0; skin < MAXSKIN; skin++)
      {
        cTmp = get_first_letter(fileread);
        if (cTmp == 'T' || cTmp == 't')  capdamagemodifier[object][damagetype][skin] |= DAMAGEINVERT;
        if (cTmp == 'C' || cTmp == 'c')  capdamagemodifier[object][damagetype][skin] |= DAMAGECHARGE;
      }
    }

    goto_colon(fileread);
    for (skin = 0;skin < MAXSKIN;skin++)
    { fscanf(fileread, "%f", &fTmp);  capmaxaccel[object][skin] = fTmp / 80.0; };


    // Experience and level data
    capexperienceforlevel[object][0] = 0;
    for (level = 1; level < MAXLEVEL; level++)
    { goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capexperienceforlevel[object][level] = iTmp; }
    capexperiencepower[object]  = log(capexperienceforlevel[object][MAXLEVEL-1]) - log(capexperienceforlevel[object][1]);
    capexperiencepower[object] /= log( (MAXLEVEL-1) ) - log(1);
    capexperiencecoeff[object] = capexperienceforlevel[object][MAXLEVEL-1] / pow(MAXLEVEL-1,capexperiencepower[object]);


    goto_colon(fileread);  read_pair(fileread);
    pairbase >>= 8;
    pairrand >>= 8;
    if (pairrand < 1)  pairrand = 1;
    capexperiencebase[object] = pairbase;
    capexperiencerand[object] = pairrand;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capexperienceworth[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capexperienceexchange[object] = fTmp;

    for (xptype = 0; xptype < MAXEXPERIENCETYPE; xptype++)
    { goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capexperiencerate[object][xptype] = fTmp + 0.001f; }


    // IDSZ tags
    for (cnt = 0; cnt < MAXIDSZ; cnt++)
    { goto_colon(fileread);  iTmp = get_idsz(fileread);  capidsz[object][cnt] = iTmp; }


    // Item and damage flags
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capisitem[object] = bfalse;  capripple[object] = btrue;
    if (cTmp == 'T' || cTmp == 't')  { capisitem[object] = btrue; capripple[object] = bfalse; }
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capismount[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capismount[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capisstackable[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capisstackable[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capnameknown[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capnameknown[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capusageknown[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capusageknown[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capcancarrytonextmodule[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcancarrytonextmodule[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capneedskillidtouse[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capneedskillidtouse[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capplatform[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capplatform[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capcangrabmoney[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcangrabmoney[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capcanopenstuff[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcanopenstuff[object] = btrue;



    // More item and damage stuff
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    if (cTmp == 'S' || cTmp == 's')  capdamagetargettype[object] = DAMAGESLASH;
    if (cTmp == 'C' || cTmp == 'c')  capdamagetargettype[object] = DAMAGECRUSH;
    if (cTmp == 'P' || cTmp == 'p')  capdamagetargettype[object] = DAMAGEPOKE;
    if (cTmp == 'H' || cTmp == 'h')  capdamagetargettype[object] = DAMAGEHOLY;
    if (cTmp == 'E' || cTmp == 'e')  capdamagetargettype[object] = DAMAGEEVIL;
    if (cTmp == 'F' || cTmp == 'f')  capdamagetargettype[object] = DAMAGEFIRE;
    if (cTmp == 'I' || cTmp == 'i')  capdamagetargettype[object] = DAMAGEICE;
    if (cTmp == 'Z' || cTmp == 'z')  capdamagetargettype[object] = DAMAGEZAP;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capweaponaction[object] = what_action(cTmp);


    // Particle attachments
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capattachedprtamount[object] = iTmp;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    if (cTmp == 'N' || cTmp == 'n')  capattachedprtreaffirmdamagetype[object] = DAMAGENULL;
    if (cTmp == 'S' || cTmp == 's')  capattachedprtreaffirmdamagetype[object] = DAMAGESLASH;
    if (cTmp == 'C' || cTmp == 'c')  capattachedprtreaffirmdamagetype[object] = DAMAGECRUSH;
    if (cTmp == 'P' || cTmp == 'p')  capattachedprtreaffirmdamagetype[object] = DAMAGEPOKE;
    if (cTmp == 'H' || cTmp == 'h')  capattachedprtreaffirmdamagetype[object] = DAMAGEHOLY;
    if (cTmp == 'E' || cTmp == 'e')  capattachedprtreaffirmdamagetype[object] = DAMAGEEVIL;
    if (cTmp == 'F' || cTmp == 'f')  capattachedprtreaffirmdamagetype[object] = DAMAGEFIRE;
    if (cTmp == 'I' || cTmp == 'i')  capattachedprtreaffirmdamagetype[object] = DAMAGEICE;
    if (cTmp == 'Z' || cTmp == 'z')  capattachedprtreaffirmdamagetype[object] = DAMAGEZAP;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capattachedprttype[object] = iTmp;


    // Character hands
    capgripvalid[object][0] = bfalse;
    capgripvalid[object][1] = bfalse;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    if (cTmp == 'T' || cTmp == 't')  capgripvalid[object][0] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    if (cTmp == 'T' || cTmp == 't')  capgripvalid[object][1] = btrue;




    // Attack order ( weapon )
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capattackattached[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capattackattached[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capattackprttype[object] = iTmp;


    // GoPoof
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capgopoofprtamount[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capgopoofprtfacingadd[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capgopoofprttype[object] = iTmp;


    // Blud
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capbludvalid[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capbludvalid[object] = btrue;
    if (cTmp == 'U' || cTmp == 'u')  capbludvalid[object] = ULTRABLOODY;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capbludprttype[object] = iTmp;


    // Stuff I forgot
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capwaterwalk[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capwaterwalk[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capdampen[object] = fTmp;


    // More stuff I forgot
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  caplifeheal[object] = fTmp * 256;
    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capmanacost[object] = fTmp * 256;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  caplifereturn[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capstoppedby[object] = iTmp | MESHFXIMPASS;

    for (skin = 0;skin < MAXSKIN;skin++)
    { goto_colon(fileread);  get_name(fileread, capskinname[object][skin]); };

    for (skin = 0;skin < MAXSKIN;skin++)
    { goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);  capskincost[object][skin] = iTmp; };

    goto_colon(fileread);  fscanf(fileread, "%f", &fTmp);  capstrengthdampen[object] = fTmp;



    // Another memory lapse
    goto_colon(fileread);  cTmp = get_first_letter(fileread);
    capridercanattack[object] = btrue;
    if (cTmp == 'T' || cTmp == 't')  capridercanattack[object] = bfalse;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);  // Can be dazed
    capcanbedazed[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcanbedazed[object] = btrue;
    goto_colon(fileread);  cTmp = get_first_letter(fileread);  // Can be grogged
    capcanbegrogged[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcanbegrogged[object] = btrue;
    goto_colon(fileread);  // !!!BAD!!! Life add
    goto_colon(fileread);  // !!!BAD!!! Mana add
    goto_colon(fileread);  cTmp = get_first_letter(fileread);  // Can see invisible
    capcanseeinvisible[object] = bfalse;
    if (cTmp == 'T' || cTmp == 't')  capcanseeinvisible[object] = btrue;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);      // Chance of kursed
    capkursechance[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);      // Footfall sound
    if (iTmp < -1) iTmp = -1;
    if (iTmp > MAXWAVE - 1) iTmp = MAXWAVE - 1;
    capwavefootfall[object] = iTmp;
    goto_colon(fileread);  fscanf(fileread, "%d", &iTmp);      // Jump sound
    if (iTmp < -1) iTmp = -1;
    if (iTmp > MAXWAVE - 1) iTmp = MAXWAVE - 1;
    capwavejump[object] = iTmp;


    // Clear expansions...
    capskindressy[object] = bfalse;
    capresistbumpspawn[object] = bfalse;
    capistoobig[object] = bfalse;
    capreflect[object] = btrue;
    capalwaysdraw[object] = bfalse;
    capisranged[object] = bfalse;
    caphidestate[object] = NOHIDE;
    capisequipment[object] = bfalse;
    capbumpsizebig[object] = capbumpsize[object] + (capbumpsize[object] >> 1);
    capmoney[object] = 0;
    capicon[object] = capusageknown[object];
    capforceshadow[object] = bfalse;
    capskinoverride[object] = NOSKINOVERRIDE;
    capcontentoverride[object] = 0;
    capstateoverride[object] = 0;
    capleveloverride[object] = 0;
    capcanuseplatforms[object] = !capplatform[object];

    //Reset Skill Expansions
    capcanseekurse[object] = bfalse;
    capcanusearcane[object] = bfalse;
    capcanjoust[object] = bfalse;
    capcanusedivine[object] = bfalse;
    capcandisarm[object] = bfalse;
    capcanusetech[object] = bfalse;
    capcanbackstab[object] = bfalse;
    capcanusepoison[object] = bfalse;
    capcanuseadvancedweapons[object] = bfalse;

    // Read expansions
    while (goto_colon_yesno(fileread))
    {
      idsz = get_idsz(fileread);
      fscanf(fileread, "%c%d", &cTmp, &iTmp);
      if (MAKE_IDSZ("GOLD") == idsz)  capmoney[object] = iTmp;
      if (MAKE_IDSZ("STUK") == idsz)  capresistbumpspawn[object] = 1 - iTmp;
      if (MAKE_IDSZ("PACK") == idsz)  capistoobig[object] = 1 - iTmp;
      if (MAKE_IDSZ("VAMP") == idsz)  capreflect[object] = 1 - iTmp;
      if (MAKE_IDSZ("DRAW") == idsz)  capalwaysdraw[object] = iTmp;
      if (MAKE_IDSZ("RANG") == idsz)  capisranged[object] = iTmp;
      if (MAKE_IDSZ("HIDE") == idsz)  caphidestate[object] = iTmp;
      if (MAKE_IDSZ("EQUI") == idsz)  capisequipment[object] = iTmp;
      if (MAKE_IDSZ("SQUA") == idsz)  capbumpsizebig[object] = capbumpsize[object] << 1;
      if (MAKE_IDSZ("ICON") == idsz)  capicon[object] = iTmp;
      if (MAKE_IDSZ("SHAD") == idsz)  capforceshadow[object] = iTmp;
      if (MAKE_IDSZ("SKIN") == idsz)  capskinoverride[object] = iTmp % MAXSKIN;
      if (MAKE_IDSZ("CONT") == idsz)  capcontentoverride[object] = iTmp;
      if (MAKE_IDSZ("STAT") == idsz)  capstateoverride[object] = iTmp;
      if (MAKE_IDSZ("LEVL") == idsz)  capleveloverride[object] = iTmp;
      if (MAKE_IDSZ("PLAT") == idsz)  capcanuseplatforms[object] = iTmp;
      if (MAKE_IDSZ("RIPP") == idsz)  capripple[object] = iTmp;

      //Skill Expansions
      // [CKUR] Can it see kurses?
      if (MAKE_IDSZ("CKUR") == idsz)  capcanseekurse[object]  = iTmp;
      // [WMAG] Can the character use arcane spellbooks?
      if (MAKE_IDSZ("WMAG") == idsz)  capcanusearcane[object] = iTmp;
      // [JOUS] Can the character joust with a lance?
      if (MAKE_IDSZ("JOUS") == idsz)  capcanusearcane[object] = iTmp;
      // [HMAG] Can the character use divine spells?
      if (MAKE_IDSZ("HMAG") == idsz)  capcanusedivine[object] = iTmp;
      // [TECH] Able to use items technological items?
      if (MAKE_IDSZ("TECH") == idsz)  capcanusetech[object]   = iTmp;
      // [DISA] Find and disarm traps?
      if (MAKE_IDSZ("DISA") == idsz)  capcanusearcane[object] = iTmp;
      // [STAB] Backstab and murder?
      if (idsz == MAKE_IDSZ("STAB"))  capcanbackstab[object] = iTmp;
      // [AWEP] Profiency with advanced weapons?
      if (idsz == MAKE_IDSZ("AWEP"))  capcanuseadvancedweapons[object] = iTmp;
      // [POIS] Use poison without err?
      if (idsz == MAKE_IDSZ("POIS"))  capcanusepoison[object] = iTmp;
    }

    fs_fileClose(fileread);
  }
  else
  {
    // The data file wasn't found
    log_error("Data.txt could not be correctly read! (%s) \n", szLoadName);
  }

  return object;
}

//--------------------------------------------------------------------------------------------
int get_skin(char *filename)
{
  // ZZ> This function reads the skin.txt file...
  FILE*   fileread;
  int skin;


  skin = 0;
  fileread = fs_fileOpen(PRI_NONE, NULL, filename, "r");
  if (NULL != fileread)
  {
    goto_colon_yesno(fileread);
    fscanf(fileread, "%d", &skin);
    skin %= MAXSKIN;
    fs_fileClose(fileread);
  }
  return skin;
}

//--------------------------------------------------------------------------------------------
void check_player_import(char *dirname)
{
  // ZZ> This function figures out which players may be imported, and loads basic
  //     data for each
  char searchname[128];
  char filename[128];
  int skin;
  bool_t keeplooking;
  const char *foundfile;


  // Set up...
  numloadplayer = 0;

  // Search for all objects
  snprintf(searchname, sizeof(searchname), "%s/*.obj", dirname);
  foundfile = fs_findFirstFile(dirname, "obj");
  keeplooking = 1;
  if (foundfile != NULL)
  {
    while (keeplooking && numloadplayer < MAXLOADPLAYER)
    {
      //fprintf(stderr,"foundfile=%s keeplooking=%d numload=%d/%d\n",foundfile,keeplooking,numloadplayer,MAXLOADPLAYER);
      prime_names();
      strncpy(loadplayerdir[numloadplayer], foundfile, sizeof(loadplayerdir[numloadplayer]));

      snprintf(filename, sizeof(filename), "%s/%s/%s", dirname, foundfile, CData.skin_file);
      skin = get_skin(filename);

      snprintf(filename, sizeof(filename), "%s/%s/tris.md2", dirname, foundfile);
      load_one_md2(filename, numloadplayer);

      snprintf(filename, sizeof(filename), "%s/%s/icon%d.bmp", dirname, foundfile, skin);
      load_one_icon(filename);

      snprintf(filename, sizeof(filename), "%s/%s/naming.txt", dirname, foundfile);
      read_naming(0, filename);
      naming_names(0);
      strncpy(loadplayername[numloadplayer], namingnames, sizeof(loadplayername[numloadplayer]));

      numloadplayer++;

      foundfile = fs_findNextFile();
      if (foundfile == NULL) keeplooking = 0; else keeplooking = 1;
    }
  }
  fs_findClose();

  nullicon = globalnumicon;
  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.nullicon_bitmap);
  load_one_icon(CStringTmp1);

  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.keybicon_bitmap);
  keybicon = globalnumicon;
  load_one_icon(CStringTmp1);

  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.mousicon_bitmap);
  mousicon = globalnumicon;
  load_one_icon(CStringTmp1);

  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.joyaicon_bitmap);
  joyaicon = globalnumicon;
  load_one_icon(CStringTmp1);

  snprintf(CStringTmp1, sizeof(CStringTmp1), "%s/%s", CData.basicdat_dir, CData.joybicon_bitmap);
  joybicon = globalnumicon;
  load_one_icon(CStringTmp1);


  keybplayer = 0;
  mousplayer = 0;
  joyaplayer = 0;
  joybplayer = 0;
}

//--------------------------------------------------------------------------------------------
bool_t check_skills(int who, Uint32 whichskill)
{
  // ZF> This checks if the specified character has the required skill. Returns btrue if true
  // and bfalse if not. Also checks Skill expansions.
  bool_t result = bfalse;

  // First check the character Skill ID matches
  // Then check for expansion skills too.
  if(capidsz[chrmodel[who]][IDSZSKILL] == whichskill) result = btrue;
  else if(MAKE_IDSZ("CKUR") == whichskill) result = chrcanseekurse[who];
  else if(MAKE_IDSZ("WMAG") == whichskill) result = chrcanusearcane[who];
  else if(MAKE_IDSZ("JOUS") == whichskill) result = chrcanjoust[who];
  else if(MAKE_IDSZ("HMAG") == whichskill) result = chrcanusedivine[who];
  else if(MAKE_IDSZ("DISA") == whichskill) result = chrcandisarm[who];
  else if(MAKE_IDSZ("TECH") == whichskill) result = chrcanusetech[who];
  else if(MAKE_IDSZ("AWEP") == whichskill) result = chrcanuseadvancedweapons[who];
  else if(MAKE_IDSZ("STAB") == whichskill) result = chrcanbackstab[who];
  else if(MAKE_IDSZ("POIS") == whichskill) result = chrcanusepoison[who];
  else if(MAKE_IDSZ("READ") == whichskill) result = chrcanread[who];

  return result;
}

//--------------------------------------------------------------------------------------------
int check_player_quest(char *whichplayer, IDSZ idsz)
{
  // ZF> This function checks if the specified player has the IDSZ in his or her quest.txt
  // and returns the quest level of that specific quest (Or -2 if it is not found, -1 if it is finished)

  //TODO: should also check if the IDSZ isnt beaten
  FILE *fileread;
  STRING newloadname;
  IDSZ newidsz;
  bool_t foundidsz = bfalse;
  int result = -2;
  int iTmp;

  //Always return "true" for [NONE] IDSZ checks
  if (idsz == IDSZNONE) result = 0;

  snprintf(newloadname, sizeof(newloadname), "%s/%s/%s", CData.players_dir, whichplayer, CData.quest_file);
  fileread = fs_fileOpen(PRI_NONE, NULL, newloadname, "r");
  if (NULL == fileread)
  {
    log_warning("File could not be read. (%s)\n", newloadname); 
    return result;
  };

  // Check each expansion
  while (goto_colon_yesno(fileread) && !foundidsz)
  {
    newidsz = get_idsz(fileread);
    if (newidsz == idsz)
    {
      foundidsz = btrue;
      //fscanf(fileread, "%o", &iTmp);		//Read value behind colon (TODO)
      iTmp = 0; //BAD should be read value
      result = iTmp;	
    }
  }
  fs_fileClose(fileread);

  return result;
}

//--------------------------------------------------------------------------------------------
bool_t add_quest_idsz(char *whichplayer, IDSZ idsz)
{
  // ZF> This function writes a IDSZ into a player quest.txt file, returns btrue if succeeded
  FILE *filewrite;
  STRING newloadname;
  bool_t result = bfalse;

  // Only add quest IDSZ if it doesnt have it already
  if (check_player_quest(whichplayer, idsz) > -2)
  {
    return result;
  };

  // Try to open the file in read and append mode
  snprintf(newloadname, sizeof(newloadname), "%s/%s/%s", CData.players_dir, whichplayer, CData.quest_file);
  filewrite = fs_fileOpen(PRI_NONE, NULL, newloadname, "a+");
  if (NULL == filewrite)
  {
    log_warning("Could not write into %s\n", newloadname);
    return result;
  };

  fprintf(filewrite, "\n:[%4s]: 0", undo_idsz(idsz));
  fs_fileClose(filewrite);

  return result;
}

//--------------------------------------------------------------------------------------------
int modify_quest_level(char *whichplayer, IDSZ idsz, int adjustment)
// ZF> This function increases or decreases a Quest IDSZ quest level by the amount determined in
// adjustment. It then returns the current quest level it now has (Or -1 if not found).
{
  //TODO
  return -1;
}

//--------------------------------------------------------------------------------------------
bool_t beat_quest_idsz(char *whichplayer, IDSZ idsz)
{
  // ZF> This function marks a IDSZ in the quest.txt file as beaten (Followed by a !)
  //     and returns btrue if it succeeded.
  FILE *filewrite;
  STRING newloadname;
  bool_t result = bfalse;
  bool_t foundidsz = bfalse;
  IDSZ newidsz;
  int QuestLevel;

  //TODO: This also needs to be done

  // Try to open the file in read/write mode
  snprintf(newloadname, sizeof(newloadname), "%s/%s/%s", CData.players_dir, whichplayer, CData.quest_file);
  filewrite = fs_fileOpen(PRI_NONE, NULL, newloadname, "w+");
  if (NULL == filewrite)
  {
    log_warning("Could not write into %s\n", newloadname);
    return result;
  };

  //Now check each expansion until we find correct IDSZ
  while (goto_colon_yesno(filewrite) && !foundidsz)
  {
    newidsz = get_idsz(filewrite);
    if (newidsz == idsz)
    {
      foundidsz = btrue;
      fscanf(filewrite, "%d", &QuestLevel);
      if(QuestLevel == -1) result = bfalse;		//Is quest is already finished?
      break;
    }
  }
  fs_fileClose(filewrite);

  return result;
}
